@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -30,9 +30,9 @@ public:
 
 private:
 protected:
-	CString	m_sPre;
-	CString	m_sPost;
-	bool	m_bIncNick;
+	CString m_sPre;
+	CString m_sPost;
+	bool    m_bIncNick;
 };
 
 class CBuffer : private deque<CBufLine> {
@@ -43,6 +43,8 @@ public:
 	int AddLine(const CString& sPre, const CString& sPost, bool bIncNick = true);
 	/// Same as AddLine, but if there is already a line with sPre it is replaced.
 	int UpdateLine(const CString& sPre, const CString& sPost, bool bIncNick = true);
+	/// Same as UpdateLine, but does nothing if this exact line already exists
+	int UpdateExactLine(const CString& sPre, const CString& sPost, bool bIncNick = true);
 	bool GetNextLine(const CString& sTarget, CString& sRet);
 	bool GetLine(const CString& sTarget, CString& sRet, unsigned int uIdx) const;
 	bool IsEmpty() const { return empty(); }
@@ -57,7 +59,7 @@ public:
 	// !Getters
 private:
 protected:
-	unsigned int	m_uLineCount;
+	unsigned int m_uLineCount;
 };
 
 #endif // !_BUFFER_H
