@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -63,6 +63,8 @@ bool CChan::WriteConfig(CFile& File) {
 	if (!GetDefaultModes().empty())
 		m_pUser->PrintLine(File, "\tModes", GetDefaultModes());
 
+	MODULECALL(OnWriteChanConfig(File, *this), m_pUser, NULL,);
+
 	File.Write("\t</Chan>\n");
 	return true;
 }
@@ -111,14 +113,14 @@ void CChan::JoinUser(bool bForce, const CString& sKey, CClient* pClient) {
 	CString sPerm, sNick;
 
 	vector<CClient*>& vpClients = m_pUser->GetClients();
-	for (vector<CClient*>::iterator it = vpClients.begin(); it != vpClients.end(); it++) {
+	for (vector<CClient*>::iterator it = vpClients.begin(); it != vpClients.end(); ++it) {
 		CClient* pThisClient;
 		if (!pClient)
 			pThisClient = *it;
 		else
 			pThisClient = pClient;
 
-		for (map<CString,CNick*>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); a++) {
+		for (map<CString,CNick*>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); ++a) {
 			if (pThisClient->HasNamesx()) {
 				sPerm = a->second->GetPermStr();
 			} else {
@@ -172,7 +174,7 @@ void CChan::AttachUser() {
 CString CChan::GetModeString() const {
 	CString sModes, sArgs;
 
-	for (map<unsigned char, CString>::const_iterator it = m_musModes.begin(); it != m_musModes.end(); it++) {
+	for (map<unsigned char, CString>::const_iterator it = m_musModes.begin(); it != m_musModes.end(); ++it) {
 		sModes += it->first;
 		if (it->second.size()) {
 			sArgs += " " + it->second;
@@ -185,7 +187,7 @@ CString CChan::GetModeString() const {
 CString CChan::GetModeForNames() const {
 	CString sMode;
 
-	for (map<unsigned char, CString>::const_iterator it = m_musModes.begin(); it != m_musModes.end(); it++) {
+	for (map<unsigned char, CString>::const_iterator it = m_musModes.begin(); it != m_musModes.end(); ++it) {
 		if (it->first == 's') {
 			sMode = "@";
 		} else if ((it->first == 'p') && sMode.empty()){
@@ -215,13 +217,11 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 	CString sArgs = sModes.Token(1, true);
 	bool bAdd = true;
 
-#ifdef _MODULES
 	CNick* pOpNick = FindNick(sOpNick);
 
 	if (pOpNick) {
 		MODULECALL(OnRawMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, );
 	}
-#endif
 
 	for (unsigned int a = 0; a < sModeArg.size(); a++) {
 		const unsigned char& uMode = sModeArg[a];
@@ -250,7 +250,6 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 							RemPerm(uPerm);
 						}
 					}
-#ifdef _MODULES
 					bool bNoChange = (pNick->HasPerm(uPerm) == bAdd);
 
 					if (uMode && pOpNick) {
@@ -270,7 +269,6 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 							}
 						}
 					}
-#endif
 				}
 			}
 		} else {
@@ -295,7 +293,6 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 					break;
 			}
 
-#ifdef _MODULES
 			bool bNoChange;
 			if (bList) {
 				bNoChange = false;
@@ -305,10 +302,9 @@ void CChan::ModeChange(const CString& sModes, const CString& sOpNick) {
 				bNoChange = !HasMode(uMode);
 			}
 			MODULECALL(OnMode(*pOpNick, *this, uMode, sArg, bAdd, bNoChange), m_pUser, NULL, );
-#endif
 
 			if (!bList) {
-				(bAdd) ? AddMode(uMode, sArg) : RemMode(uMode, sArg);
+				(bAdd) ? AddMode(uMode, sArg) : RemMode(uMode);
 			}
 		}
 	}
@@ -349,7 +345,7 @@ bool CChan::AddMode(unsigned char uMode, const CString& sArg) {
 	return true;
 }
 
-bool CChan::RemMode(unsigned char uMode, const CString& sArg) {
+bool CChan::RemMode(unsigned char uMode) {
 	if (!HasMode(uMode)) {
 		return false;
 	}
@@ -365,7 +361,7 @@ CString CChan::GetModeArg(CString& sArgs) const {
 }
 
 void CChan::ClearNicks() {
-	for (map<CString,CNick*>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); a++) {
+	for (map<CString,CNick*>::iterator a = m_msNicks.begin(); a != m_msNicks.end(); ++a) {
 		delete a->second;
 	}
 
@@ -379,7 +375,7 @@ int CChan::AddNicks(const CString& sNicks) {
 
 	sNicks.Split(" ", vsNicks, false);
 
-	for (it = vsNicks.begin(); it != vsNicks.end(); it++) {
+	for (it = vsNicks.begin(); it != vsNicks.end(); ++it) {
 		if (AddNick(*it)) {
 			iRet++;
 		}
@@ -439,7 +435,7 @@ map<char, unsigned int> CChan::GetPermCounts() const {
 	map<char, unsigned int> mRet;
 
 	map<CString,CNick*>::const_iterator it;
-	for (it = m_msNicks.begin(); it != m_msNicks.end(); it++) {
+	for (it = m_msNicks.begin(); it != m_msNicks.end(); ++it) {
 		CString sPerms = it->second->GetPermStr();
 
 		for (unsigned int p = 0; p < sPerms.size(); p++) {
