@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -10,40 +10,40 @@
 #include "Chan.h"
 #include "DCCBounce.h"
 #include "IRCSock.h"
-#include "Timers.h"
 #include "User.h"
 #include "znc.h"
+#include "WebModules.h"
 
-#define CALLMOD(MOD, CLIENT, USER, FUNC) {				\
-	CModule* pModule = CZNC::Get().GetModules().FindModule(MOD);	\
-	if (pModule) {							\
-		try {							\
-			pModule->SetClient(CLIENT);			\
-			pModule->SetUser(USER);				\
-			pModule->FUNC;					\
-			pModule->SetClient(NULL);			\
-			pModule->SetUser(NULL);				\
-		} catch (CModule::EModException e) {			\
-			if (e == CModule::UNLOAD) {			\
-				CZNC::Get().GetModules().UnloadModule(MOD);	\
-			}						\
-		}							\
-	} else {							\
-		pModule = (USER)->GetModules().FindModule(MOD);	\
-		if (pModule) {						\
-			try {						\
-				pModule->SetClient(CLIENT);		\
-				pModule->FUNC;				\
-				pModule->SetClient(NULL);		\
-			} catch (CModule::EModException e) {		\
-				if (e == CModule::UNLOAD) {		\
-					(USER)->GetModules().UnloadModule(MOD);	\
-				}					\
-			}						\
-		} else {						\
-			PutStatus("No such module [" + MOD + "]");	\
-		}							\
-	}								\
+#define CALLMOD(MOD, CLIENT, USER, FUNC) {                                      \
+	CModule* pModule = CZNC::Get().GetModules().FindModule(MOD);            \
+	if (pModule) {                                                          \
+		try {                                                           \
+			pModule->SetClient(CLIENT);                             \
+			pModule->SetUser(USER);                                 \
+			pModule->FUNC;                                          \
+			pModule->SetClient(NULL);                               \
+			pModule->SetUser(NULL);                                 \
+		} catch (CModule::EModException e) {                            \
+			if (e == CModule::UNLOAD) {                             \
+				CZNC::Get().GetModules().UnloadModule(MOD);     \
+			}                                                       \
+		}                                                               \
+	} else {                                                                \
+		pModule = (USER)->GetModules().FindModule(MOD);                 \
+		if (pModule) {                                                  \
+			try {                                                   \
+				pModule->SetClient(CLIENT);                     \
+				pModule->FUNC;                                  \
+				pModule->SetClient(NULL);                       \
+			} catch (CModule::EModException e) {                    \
+				if (e == CModule::UNLOAD) {                     \
+					(USER)->GetModules().UnloadModule(MOD); \
+				}                                               \
+			}                                                       \
+		} else {                                                        \
+			PutStatus("No such module [" + MOD + "]");              \
+		}                                                               \
+	}                                                                       \
 }
 
 CClient::~CClient() {
@@ -64,14 +64,13 @@ void CClient::ReadLine(const CString& sData) {
 
 	DEBUG("(" << ((m_pUser) ? m_pUser->GetUserName() : GetRemoteIP()) << ") CLI -> ZNC [" << sLine << "]");
 
-#ifdef _MODULES
 	if (IsAttached()) {
 		MODULECALL(OnUserRaw(sLine), m_pUser, this, return);
 	} else {
-		if (CZNC::Get().GetModules().OnUnknownUserRaw(this, sLine))
+		if (CZNC::Get().GetModules().OnUnknownUserRaw(this, sLine)) {
 			return;
+		}
 	}
-#endif
 
 	CString sCommand = sLine.Token(0);
 	if (sCommand.Left(1) == ":") {
@@ -97,7 +96,7 @@ void CClient::ReadLine(const CString& sData) {
 				AuthUser();
 			}
 
-			return;		// Don't forward this msg.  ZNC has already registered us.
+			return;  // Don't forward this msg.  ZNC has already registered us.
 		}
 	} else if (sCommand.Equals("NICK")) {
 		CString sNick = sLine.Token(1);
@@ -112,12 +111,7 @@ void CClient::ReadLine(const CString& sData) {
 			if ((m_bGotPass) && (m_bGotUser)) {
 				AuthUser();
 			}
-			return;		// Don't forward this msg.  ZNC will handle nick changes until auth is complete
-		}
-
-		if (!m_pIRCSock) {
-			// No need to forward it
-			return;
+			return;  // Don't forward this msg.  ZNC will handle nick changes until auth is complete
 		}
 	} else if (sCommand.Equals("USER")) {
 		if (!IsAttached()) {
@@ -135,7 +129,7 @@ void CClient::ReadLine(const CString& sData) {
 					"Try /quote PASS <username>:<password>");
 			}
 
-			return;		// Don't forward this msg.  ZNC has already registered us.
+			return;  // Don't forward this msg.  ZNC has already registered us.
 		}
 	}
 
@@ -161,12 +155,10 @@ void CClient::ReadLine(const CString& sData) {
 			else
 				UserCommand(sModCommand);
 		} else {
-#ifdef _MODULES
 			if (sModCommand.empty())
 				CALLMOD(sTarget, this, m_pUser, PutModule("Hello. How may I help you?"))
 			else
 				CALLMOD(sTarget, this, m_pUser, OnModCommand(sModCommand))
-#endif
 		}
 		return;
 	} else if (sCommand.Equals("DETACH")) {
@@ -192,12 +184,12 @@ void CClient::ReadLine(const CString& sData) {
 		// If the client meant to ping us or we can be sure the server
 		// won't answer the ping (=no server connected) -> PONG back.
 		// else: It's the server's job to send a PONG.
-		if (sTarget.Equals("irc.znc.in") || !m_pIRCSock) {
+		if (sTarget.Equals("irc.znc.in") || !GetIRCSock()) {
 			PutClient("PONG " + sLine.substr(5));
 			return;
 		}
 	} else if (sCommand.Equals("PONG")) {
-		return;	// Block pong replies, we already responded to the pings
+		return;  // Block pong replies, we already responded to the pings
 	} else if (sCommand.Equals("JOIN")) {
 		CString sChans = sLine.Token(1);
 		CString sKey = sLine.Token(2);
@@ -295,19 +287,21 @@ void CClient::ReadLine(const CString& sData) {
 	} else if (sCommand.Equals("QUIT")) {
 		m_pUser->UserDisconnected(this);
 
-		Close(Csock::CLT_AFTERWRITE);	// Treat a client quit as a detach
-		return;		// Don't forward this msg.  We don't want the client getting us disconnected.
+		Close(Csock::CLT_AFTERWRITE); // Treat a client quit as a detach
+		return;                       // Don't forward this msg.  We don't want the client getting us disconnected.
 	} else if (sCommand.Equals("PROTOCTL")) {
-		unsigned int i = 1;
-		while (!sLine.Token(i).empty()) {
-			if (sLine.Token(i).Equals("NAMESX")) {
+		VCString vsTokens;
+		VCString::const_iterator it;
+		sLine.Token(1, true).Split(" ", vsTokens, false);
+
+		for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
+			if (*it == "NAMESX") {
 				m_bNamesx = true;
-			} else if (sLine.Token(i).Equals("UHNAMES")) {
+			} else if (*it == "UHNAMES") {
 				m_bUHNames = true;
 			}
-			i++;
 		}
-		return;	// If the server understands it, we already enabled namesx / uhnames
+		return;  // If the server understands it, we already enabled namesx / uhnames
 	} else if (sCommand.Equals("NOTICE")) {
 		CString sTarget = sLine.Token(1);
 		CString sMsg = sLine.Token(2, true);
@@ -317,11 +311,9 @@ void CClient::ReadLine(const CString& sData) {
 		}
 
 		if (sTarget.TrimPrefix(m_pUser->GetStatusPrefix())) {
-#ifdef _MODULES
 			if (!sTarget.Equals("status")) {
 				CALLMOD(sTarget, this, m_pUser, OnModNotice(sMsg));
 			}
-#endif
 			return;
 		}
 
@@ -329,11 +321,10 @@ void CClient::ReadLine(const CString& sData) {
 		// make sense. Comment this out and wait for complaints.
 #if 0
 		if (sMsg.WildCmp("DCC * (*)")) {
-			sMsg = "DCC " + sLine.Token(3) + " (" + ((m_pIRCSock) ? m_pIRCSock->GetLocalIP() : GetLocalIP()) + ")";
+			sMsg = "DCC " + sLine.Token(3) + " (" + ((GetIRCSock()) ? GetIRCSock()->GetLocalIP() : GetLocalIP()) + ")";
 		}
 #endif
 
-#ifdef _MODULES
 		if (sMsg.WildCmp("\001*\001")) {
 			CString sCTCP = sMsg;
 			sCTCP.LeftChomp();
@@ -345,9 +336,8 @@ void CClient::ReadLine(const CString& sData) {
 		} else {
 			MODULECALL(OnUserNotice(sTarget, sMsg), m_pUser, this, return);
 		}
-#endif
 
-		if (!m_pIRCSock) {
+		if (!GetIRCSock()) {
 			// Some lagmeters do a NOTICE to their own nick, ignore those.
 			if (!sTarget.Equals(m_sNick))
 				PutStatus("Your notice to [" + sTarget + "] got lost, "
@@ -483,9 +473,7 @@ void CClient::ReadLine(const CString& sData) {
 				if (sTarget.Equals("status")) {
 					StatusCTCP(sCTCP);
 				} else {
-#ifdef _MODULES
 					CALLMOD(sTarget, this, m_pUser, OnModCTCP(sCTCP));
-#endif
 				}
 				return;
 			}
@@ -525,16 +513,14 @@ void CClient::ReadLine(const CString& sData) {
 			if (sTarget.Equals("status")) {
 				UserCommand(sMsg);
 			} else {
-#ifdef _MODULES
 				CALLMOD(sTarget, this, m_pUser, OnModCommand(sMsg));
-#endif
 			}
 			return;
 		}
 
 		MODULECALL(OnUserMsg(sTarget, sMsg), m_pUser, this, return);
 
-		if (!m_pIRCSock) {
+		if (!GetIRCSock()) {
 			// Some lagmeters do a PRIVMSG to their own nick, ignore those.
 			if (!sTarget.Equals(m_sNick))
 				PutStatus("Your message to [" + sTarget + "] got lost, "
@@ -574,6 +560,14 @@ void CClient::SetNick(const CString& s) {
 	m_sNick = s;
 }
 
+const CIRCSock* CClient::GetIRCSock() const {
+	return m_pUser->GetIRCSock();
+}
+
+CIRCSock* CClient::GetIRCSock() {
+	return m_pUser->GetIRCSock();
+}
+
 void CClient::StatusCTCP(const CString& sLine) {
 	CString sCommand = sLine.Token(0);
 
@@ -592,33 +586,13 @@ bool CClient::SendMotd() {
 	}
 
 	for (unsigned int a = 0; a < vsMotd.size(); a++) {
-		PutStatusNotice(vsMotd[a]);
+		PutStatusNotice(m_pUser->ExpandString(vsMotd[a]));
 	}
 
 	return true;
 }
 
 void CClient::AuthUser() {
-	/*
-#ifdef _MODULES
-	if (CZNC::Get().GetModules().OnLoginAttempt(m_sUser, m_sPass, *this)) {
-		return;
-	}
-#endif
-
-	CUser* pUser = CZNC::Get().GetUser(m_sUser);
-
-	if (pUser && pUser->CheckPass(m_sPass)) {
-		AcceptLogin(*pUser);
-	} else {
-		if (pUser) {
-			pUser->PutStatus("Another client attempted to login as you, with a bad password.");
-		}
-
-		RefuseLogin();
-	}
-	*/
-
 	m_spAuth = new CClientAuth(this, m_sUser, m_sPass);
 
 	CZNC::Get().AuthUser(m_spAuth);
@@ -666,9 +640,7 @@ void CAuthBase::RefuseLogin(const CString& sReason) {
 				"to login as you, but was rejected [" + sReason + "].");
 	}
 
-#ifdef _MODULES
 	CZNC::Get().GetModules().OnFailedLogin(GetUsername(), GetRemoteIP());
-#endif
 	RefusedLogin(sReason);
 	Invalidate();
 }
@@ -695,7 +667,6 @@ void CClient::AcceptLogin(CUser& User) {
 
 	SetSockName("USR::" + m_pUser->GetUserName());
 
-	m_pIRCSock = (CIRCSock*) CZNC::Get().FindSockByName("IRC::" + m_pUser->GetUserName());
 	m_pUser->UserConnected(this);
 
 	SendMotd();
@@ -721,8 +692,6 @@ void CClient::Disconnected() {
 		m_pUser->UserDisconnected(this);
 	}
 
-	m_pIRCSock = NULL;
-
 	MODULECALL(OnClientDisconnect(), m_pUser, this, );
 }
 
@@ -734,24 +703,13 @@ void CClient::ReachedMaxBuffer() {
 	Close();
 }
 
-void CClient::IRCConnected(CIRCSock* pIRCSock) {
-	m_pIRCSock = pIRCSock;
-}
-
 void CClient::BouncedOff() {
 	PutStatusNotice("You are being disconnected because another user just authenticated as you.");
-	m_pIRCSock = NULL;
 	Close(Csock::CLT_AFTERWRITE);
 }
 
-void CClient::IRCDisconnected() {
-	m_pIRCSock = NULL;
-}
-
 void CClient::PutIRC(const CString& sLine) {
-	if (m_pIRCSock) {
-		m_pIRCSock->PutIRC(sLine);
-	}
+	m_pUser->PutIRC(sLine);
 }
 
 void CClient::PutClient(const CString& sLine) {
@@ -796,16 +754,16 @@ void CClient::PutModule(const CString& sModule, const CString& sLine) {
 CString CClient::GetNick(bool bAllowIRCNick) const {
 	CString sRet;
 
-	if ((bAllowIRCNick) && (IsAttached()) && (m_pIRCSock)) {
-		sRet = m_pIRCSock->GetNick();
+	if ((bAllowIRCNick) && (IsAttached()) && (GetIRCSock())) {
+		sRet = GetIRCSock()->GetNick();
 	}
 
 	return (sRet.empty()) ? m_sNick : sRet;
 }
 
 CString CClient::GetNickMask() const {
-	if (m_pIRCSock && m_pIRCSock->IsAuthed()) {
-		return m_pIRCSock->GetNickMask();
+	if (GetIRCSock() && GetIRCSock()->IsAuthed()) {
+		return GetIRCSock()->GetNickMask();
 	}
 
 	CString sHost = m_pUser->GetVHost();
