@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -14,6 +14,7 @@
 #include "Server.h"
 #include "User.h"
 #include "znc.h"
+#include "Listener.h"
 
 void CClient::UserCommand(CString& sLine) {
 	if (!m_pUser) {
@@ -26,7 +27,7 @@ void CClient::UserCommand(CString& sLine) {
 
 	MODULECALL(OnStatusCommand(sLine), m_pUser, this, return);
 
-	CString sCommand = sLine.Token(0);
+	const CString sCommand = sLine.Token(0);
 
 	if (sCommand.Equals("HELP")) {
 		HelpUser();
@@ -71,7 +72,7 @@ void CClient::UserCommand(CString& sLine) {
 		Table.AddColumn("Ident");
 		Table.AddColumn("Host");
 
-		for (map<CString,CNick*>::const_iterator a = msNicks.begin(); a != msNicks.end(); a++) {
+		for (map<CString,CNick*>::const_iterator a = msNicks.begin(); a != msNicks.end(); ++a) {
 			Table.AddRow();
 
 			for (unsigned int b = 0; b < sPerms.size(); b++) {
@@ -166,8 +167,9 @@ void CClient::UserCommand(CString& sLine) {
 		Table.AddColumn("OnIRC");
 		Table.AddColumn("IRC Server");
 		Table.AddColumn("IRC User");
+		Table.AddColumn("Channels");
 
-		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); ++it) {
 			Table.AddRow();
 			Table.SetCell("Username", it->first);
 			Table.SetCell("Clients", CString(it->second->GetClients().size()));
@@ -177,6 +179,7 @@ void CClient::UserCommand(CString& sLine) {
 				Table.SetCell("OnIRC", "Yes");
 				Table.SetCell("IRC Server", it->second->GetIRCServer());
 				Table.SetCell("IRC User", it->second->GetIRCNick().GetNickMask());
+				Table.SetCell("Channels", CString(it->second->GetChans().size()));
 			}
 		}
 
@@ -231,8 +234,8 @@ void CClient::UserCommand(CString& sLine) {
 			return;
 		}
 
-		if (m_pIRCSock) {
-			m_pIRCSock->Quit();
+		if (GetIRCSock()) {
+			GetIRCSock()->Quit();
 			PutStatus("Jumping to the next server in the list...");
 		} else {
 			PutStatus("Connecting...");
@@ -242,11 +245,12 @@ void CClient::UserCommand(CString& sLine) {
 		m_pUser->CheckIRCConnect();
 		return;
 	} else if (sCommand.Equals("DISCONNECT")) {
-		// m_pIRCSock is only set after the low level connection
+		// GetIRCSock() is only set after the low level connection
 		// to the IRC server was established. Before this we can
 		// only find the IRC socket by its name.
-		if (m_pIRCSock) {
-			m_pIRCSock->Quit();
+		if (GetIRCSock()) {
+			CString sQuitMsg = sLine.Token(1, true);
+			GetIRCSock()->Quit(sQuitMsg);
 		} else {
 			Csock* pIRCSock;
 			CString sSockName = "IRC::" + m_pUser->GetUserName();
@@ -275,12 +279,29 @@ void CClient::UserCommand(CString& sLine) {
 			PutStatus("Channel [" + sChan + "] enabled.");
 		}
 	} else if (sCommand.Equals("LISTCHANS")) {
-		const vector<CChan*>& vChans = m_pUser->GetChans();
-		CIRCSock* pIRCSock = (!m_pUser) ? NULL : m_pUser->GetIRCSock();
+		CUser* pUser = m_pUser;
+		const CString sNick = sLine.Token(1);
+
+		if (!sNick.empty()) {
+			if (!m_pUser->IsAdmin()) {
+				PutStatus("Usage: ListChans");
+				return;
+			}
+
+			pUser = CZNC::Get().FindUser(sNick);
+
+			if (!pUser) {
+				PutStatus("No such user [" + sNick + "]");
+				return;
+			}
+		}
+
+		const vector<CChan*>& vChans = pUser->GetChans();
+		CIRCSock* pIRCSock = pUser->GetIRCSock();
 		const CString& sPerms = (pIRCSock) ? pIRCSock->GetPerms() : "";
 
 		if (!vChans.size()) {
-			PutStatus("You have no channels defined");
+			PutStatus("There are no channels defined.");
 			return;
 		}
 
@@ -298,8 +319,11 @@ void CClient::UserCommand(CString& sLine) {
 			Table.AddColumn(sPerm);
 		}
 
+		unsigned int uNumDetached = 0, uNumDisabled = 0,
+			uNumJoined = 0;
+
 		for (unsigned int a = 0; a < vChans.size(); a++) {
-			CChan* pChan = vChans[a];
+			const CChan* pChan = vChans[a];
 			Table.AddRow();
 			Table.SetCell("Name", pChan->GetPermStr() + pChan->GetName());
 			Table.SetCell("Status", ((vChans[a]->IsOn()) ? ((vChans[a]->IsDetached()) ? "Detached" : "Joined") : ((vChans[a]->IsDisabled()) ? "Disabled" : "Trying")));
@@ -313,9 +337,15 @@ void CClient::UserCommand(CString& sLine) {
 				char cPerm = sPerms[b];
 				Table.SetCell(CString(cPerm), CString(mPerms[cPerm]));
 			}
+
+			if(pChan->IsDetached()) uNumDetached++;
+			if(pChan->IsOn()) uNumJoined++;
+			if(pChan->IsDisabled()) uNumDisabled++;
 		}
 
 		PutStatus(Table);
+		PutStatus("Total: " + CString(vChans.size()) + " - Joined: " + CString(uNumJoined) +
+			" - Detached: " + CString(uNumDetached) + " - Disabled: " + CString(uNumDisabled));
 	} else if (sCommand.Equals("ADDSERVER")) {
 		CString sServer = sLine.Token(1);
 
@@ -519,7 +549,6 @@ void CClient::UserCommand(CString& sLine) {
 			PutStatus("You have no active DCCs.");
 		}
 	} else if (sCommand.Equals("LISTMODS") || sCommand.Equals("LISTMODULES")) {
-#ifdef _MODULES
 		if (m_pUser->IsAdmin()) {
 			CModules& GModules = CZNC::Get().GetModules();
 
@@ -559,12 +588,8 @@ void CClient::UserCommand(CString& sLine) {
 
 			PutStatus(Table);
 		}
-#else
-		PutStatus("Modules are not enabled.");
-#endif
 		return;
 	} else if (sCommand.Equals("LISTAVAILMODS") || sCommand.Equals("LISTAVAILABLEMODULES")) {
-#ifdef _MODULES
 		if (m_pUser->DenyLoadMod()) {
 			PutStatus("Access Denied.");
 			return;
@@ -574,7 +599,7 @@ void CClient::UserCommand(CString& sLine) {
 			set<CModInfo> ssGlobalMods;
 			CZNC::Get().GetModules().GetAvailableMods(ssGlobalMods, true);
 
-			if (!ssGlobalMods.size()) {
+			if (ssGlobalMods.empty()) {
 				PutStatus("No global modules available.");
 			} else {
 				PutStatus("Global modules:");
@@ -583,7 +608,7 @@ void CClient::UserCommand(CString& sLine) {
 				GTable.AddColumn("Description");
 				set<CModInfo>::iterator it;
 
-				for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); it++) {
+				for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); ++it) {
 					const CModInfo& Info = *it;
 					GTable.AddRow();
 					GTable.SetCell("Name", (CZNC::Get().GetModules().FindModule(Info.GetName()) ? "*" : " ") + Info.GetName());
@@ -606,7 +631,7 @@ void CClient::UserCommand(CString& sLine) {
 			Table.AddColumn("Description");
 			set<CModInfo>::iterator it;
 
-			for (it = ssUserMods.begin(); it != ssUserMods.end(); it++) {
+			for (it = ssUserMods.begin(); it != ssUserMods.end(); ++it) {
 				const CModInfo& Info = *it;
 				Table.AddRow();
 				Table.SetCell("Name", (m_pUser->GetModules().FindModule(Info.GetName()) ? "*" : " ") + Info.GetName());
@@ -615,9 +640,6 @@ void CClient::UserCommand(CString& sLine) {
 
 			PutStatus(Table);
 		}
-#else
-		PutStatus("Modules are not enabled.");
-#endif
 		return;
 	} else if (sCommand.Equals("LOADMOD") || sCommand.Equals("LOADMODULE")) {
 		CString sMod;
@@ -636,7 +658,6 @@ void CClient::UserCommand(CString& sLine) {
 			return;
 		}
 
-#ifdef _MODULES
 		CModInfo ModInfo;
 		CString sRetMsg;
 		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod, sRetMsg)) {
@@ -666,9 +687,6 @@ void CClient::UserCommand(CString& sLine) {
 		}
 
 		PutStatus(sModRet);
-#else
-		PutStatus("Unable to load [" + sMod + "] Modules are not enabled.");
-#endif
 		return;
 	} else if (sCommand.Equals("UNLOADMOD") || sCommand.Equals("UNLOADMODULE")) {
 		CString sMod;
@@ -678,7 +696,6 @@ void CClient::UserCommand(CString& sLine) {
 			PutStatus("Unable to unload [" + sMod + "] Access Denied.");
 			return;
 		}
-#ifdef _MODULES
 		if (sMod.empty()) {
 			PutStatus("Usage: UnloadMod <module>");
 			return;
@@ -695,9 +712,6 @@ void CClient::UserCommand(CString& sLine) {
 		}
 
 		PutStatus(sModRet);
-#else
-		PutStatus("Unable to unload [" + sMod + "] Modules are not enabled.");
-#endif
 		return;
 	} else if (sCommand.Equals("RELOADMOD") || sCommand.Equals("RELOADMODULE")) {
 		CString sMod;
@@ -707,10 +721,15 @@ void CClient::UserCommand(CString& sLine) {
 		sArgs = sLine.Token(2, true);
 
 		if (m_pUser->DenyLoadMod()) {
-			PutStatus("Unable to reload [" + sMod + "] Access Denied.");
+			PutStatus("Unable to reload modules. Access Denied.");
 			return;
 		}
-#ifdef _MODULES
+
+		if (sMod.empty()) {
+			PutStatus("Usage: ReloadMod <module> [args]");
+			return;
+		}
+
 		CModInfo ModInfo;
 		CString sRetMsg;
 		if (!CZNC::Get().GetModules().GetModInfo(ModInfo, sMod, sRetMsg)) {
@@ -725,11 +744,6 @@ void CClient::UserCommand(CString& sLine) {
 			return;
 		}
 
-		if (sMod.empty()) {
-			PutStatus("Usage: ReloadMod <module> [args]");
-			return;
-		}
-
 		CString sModRet;
 
 		if (bGlobal) {
@@ -739,16 +753,15 @@ void CClient::UserCommand(CString& sLine) {
 		}
 
 		PutStatus(sModRet);
-#else
-		PutStatus("Unable to unload [" + sMod + "] Modules are not enabled.");
-#endif
 		return;
 	} else if ((sCommand.Equals("UPDATEMOD") || sCommand.Equals("UPDATEMODULE")) && m_pUser->IsAdmin() ) {
-#ifndef _MODULES
-		PutStatus("Modules are not enabled.");
-#else
 		CString sMod = sLine.Token(1);
 
+		if (sMod.empty()) {
+			PutStatus("Usage: UpdateMod <module>");
+			return;
+		}
+
 		if (m_pUser->DenyLoadMod() || !m_pUser->IsAdmin()) {
 			PutStatus("Unable to reload [" + sMod + "] Access Denied.");
 			return;
@@ -761,7 +774,6 @@ void CClient::UserCommand(CString& sLine) {
 			PutStatus("Done, but there were errors, some users no longer have ["
 					+ sMod + "] loaded");
 		}
-#endif
 	} else if (sCommand.Equals("ADDVHOST") && m_pUser->IsAdmin()) {
 		CString sVHost = sLine.Token(1);
 
@@ -800,7 +812,7 @@ void CClient::UserCommand(CString& sLine) {
 		Table.AddColumn("VHost");
 
 		VCString::const_iterator it;
-		for (it = vsVHosts.begin(); it != vsVHosts.end(); it++) {
+		for (it = vsVHosts.begin(); it != vsVHosts.end(); ++it) {
 			Table.AddRow();
 			Table.SetCell("VHost", *it);
 		}
@@ -823,7 +835,7 @@ void CClient::UserCommand(CString& sLine) {
 			VCString::const_iterator it;
 			bool bFound = false;
 
-			for (it = vsVHosts.begin(); it != vsVHosts.end(); it++) {
+			for (it = vsVHosts.begin(); it != vsVHosts.end(); ++it) {
 				if (sVHost.Equals(*it)) {
 					bFound = true;
 					break;
@@ -893,7 +905,7 @@ void CClient::UserCommand(CString& sLine) {
 		vector<CChan*>::const_iterator it;
 		const vector<CChan*>& vChans = m_pUser->GetChans();
 
-		for (it = vChans.begin(); it != vChans.end(); it++) {
+		for (it = vChans.begin(); it != vChans.end(); ++it) {
 			(*it)->ClearBuffer();
 		}
 		PutStatus("All channel buffers have been cleared");
@@ -934,7 +946,7 @@ void CClient::UserCommand(CString& sLine) {
 		Table.AddColumn("Out");
 		Table.AddColumn("Total");
 
-		for (it = traffic.begin(); it != traffic.end(); it++) {
+		for (it = traffic.begin(); it != traffic.end(); ++it) {
 			Table.AddRow();
 			Table.SetCell("Username", it->first);
 			Table.SetCell("In", CString::ToByteStr(it->second.first));
@@ -963,11 +975,112 @@ void CClient::UserCommand(CString& sLine) {
 		PutStatus(Table);
 	} else if (sCommand.Equals("UPTIME")) {
 		PutStatus("Running for " + CZNC::Get().GetUptime());
+	} else if (m_pUser->IsAdmin() &&
+			(sCommand.Equals("LISTPORTS") || sCommand.Equals("ADDPORT") || sCommand.Equals("DELPORT"))) {
+		UserPortCommand(sLine);
 	} else {
 		PutStatus("Unknown command [" + sCommand + "] try 'Help'");
 	}
 }
 
+void CClient::UserPortCommand(CString& sLine) {
+	const CString sCommand = sLine.Token(0);
+
+	if (sCommand.Equals("LISTPORTS")) {
+		CTable Table;
+		Table.AddColumn("Port");
+		Table.AddColumn("BindHost");
+		Table.AddColumn("SSL");
+		Table.AddColumn("Proto");
+		Table.AddColumn("IRC/Web");
+
+		vector<CListener*>::const_iterator it;
+		const vector<CListener*>& vpListeners = CZNC::Get().GetListeners();
+
+		for (it = vpListeners.begin(); it < vpListeners.end(); ++it) {
+			Table.AddRow();
+			Table.SetCell("Port", CString((*it)->GetPort()));
+			Table.SetCell("BindHost", ((*it)->GetBindHost().empty() ? CString("*") : (*it)->GetBindHost()));
+			Table.SetCell("SSL", CString((*it)->IsSSL()));
+
+			EAddrType eAddr = (*it)->GetAddrType();
+			Table.SetCell("Proto", (eAddr == ADDR_ALL ? "All" : (eAddr == ADDR_IPV4ONLY ? "IPv4" : "IPv6")));
+
+			CListener::EAcceptType eAccept = (*it)->GetAcceptType();
+			Table.SetCell("IRC/Web", (eAccept == CListener::ACCEPT_ALL ? "All" : (eAccept == CListener::ACCEPT_IRC ? "IRC" : "Web")));
+		}
+
+		PutStatus(Table);
+
+		return;
+	}
+
+	CString sPort = sLine.Token(1);
+	CString sAddr = sLine.Token(2);
+	EAddrType eAddr = ADDR_ALL;
+
+	if (sAddr.Equals("IPV4")) {
+		eAddr = ADDR_IPV4ONLY;
+	} else if (sAddr.Equals("IPV6")) {
+		eAddr = ADDR_IPV6ONLY;
+	} else if (sAddr.Equals("ALL")) {
+		eAddr = ADDR_ALL;
+	} else {
+		sAddr.clear();
+	}
+
+	unsigned short uPort = sPort.ToUShort();
+
+	if (sCommand.Equals("ADDPORT")) {
+		CListener::EAcceptType eAccept = CListener::ACCEPT_ALL;
+		CString sAccept = sLine.Token(3);
+
+		if (sAccept.Equals("WEB")) {
+			eAccept = CListener::ACCEPT_HTTP;
+		} else if (sAccept.Equals("IRC")) {
+			eAccept = CListener::ACCEPT_IRC;
+		} else if (sAccept.Equals("ALL")) {
+			eAccept = CListener::ACCEPT_ALL;
+		} else {
+			sAccept.clear();
+		}
+
+		if (sPort.empty() || sAddr.empty() || sAccept.empty()) {
+			PutStatus("Usage: AddPort <[+]port> <ipv4|ipv6|all> <web|irc|all> [bindhost]");
+		} else {
+			bool bSSL = (sPort.Left(1).Equals("+"));
+			const CString sBindHost = sLine.Token(4);
+
+			CListener* pListener = new CListener(uPort, sBindHost, bSSL, eAddr, eAccept);
+
+			if (!pListener->Listen()) {
+				delete pListener;
+				PutStatus("Unable to bind [" + CString(strerror(errno)) + "]");
+			} else {
+				if (CZNC::Get().AddListener(pListener))
+					PutStatus("Port Added");
+				else
+					PutStatus("Error?!");
+			}
+		}
+	} else if (sCommand.Equals("DELPORT")) {
+		if (sPort.empty() || sAddr.empty()) {
+			PutStatus("Usage: DelPort <port> <ipv4|ipv6|all> [bindhost]");
+		} else {
+			const CString sBindHost = sLine.Token(3);
+
+			CListener* pListener = CZNC::Get().FindListener(uPort, sBindHost, eAddr);
+
+			if (pListener) {
+				CZNC::Get().DelListener(pListener);
+				PutStatus("Deleted Port");
+			} else {
+				PutStatus("Unable to find a matching port");
+			}
+		}
+	}
+}
+
 void CClient::HelpUser() {
 	CTable Table;
 	Table.AddColumn("Command");
@@ -976,44 +1089,39 @@ void CClient::HelpUser() {
 
 	Table.AddRow();
 	Table.SetCell("Command", "Version");
-	Table.SetCell("Arguments", "");
-	Table.SetCell("Description", "Prints which version of znc this is");
+	Table.SetCell("Description", "Print which version of ZNC this is");
 
 	Table.AddRow();
 	Table.SetCell("Command", "ListDCCs");
-	Table.SetCell("Arguments", "");
 	Table.SetCell("Description", "List all active DCCs");
 
 	Table.AddRow();
 	Table.SetCell("Command", "ListMods");
-	Table.SetCell("Arguments", "");
 	Table.SetCell("Description", "List all loaded modules");
 
 	Table.AddRow();
 	Table.SetCell("Command", "ListAvailMods");
-	Table.SetCell("Arguments", "");
 	Table.SetCell("Description", "List all available modules");
 
-	Table.AddRow();
-	Table.SetCell("Command", "ListChans");
-	Table.SetCell("Arguments", "");
-	Table.SetCell("Description", "List all channels");
+	if (!m_pUser->IsAdmin()) { // If they are an admin we will add this command below with an argument
+		Table.AddRow();
+		Table.SetCell("Command", "ListChans");
+		Table.SetCell("Description", "List all channels");
+	}
 
 	Table.AddRow();
 	Table.SetCell("Command", "ListNicks");
 	Table.SetCell("Arguments", "<#chan>");
 	Table.SetCell("Description", "List all nicks on a channel");
 
-	if (!m_pUser->IsAdmin()) { // If they are an admin we will add this command below with an argument
+	if (!m_pUser->IsAdmin()) {
 		Table.AddRow();
 		Table.SetCell("Command", "ListClients");
-		Table.SetCell("Arguments", "");
-		Table.SetCell("Description", "List all clients connected to your znc user");
+		Table.SetCell("Description", "List all clients connected to your ZNC user");
 	}
 
 	Table.AddRow();
 	Table.SetCell("Command", "ListServers");
-	Table.SetCell("Arguments", "");
 	Table.SetCell("Description", "List all servers");
 
 	Table.AddRow();
@@ -1023,7 +1131,7 @@ void CClient::HelpUser() {
 
 	Table.AddRow();
 	Table.SetCell("Command", "RemServer");
-	Table.SetCell("Arguments", "<host>");
+	Table.SetCell("Arguments", "<host> [port] [pass]");
 	Table.SetCell("Description", "Remove a server from the list");
 
 	Table.AddRow();
@@ -1038,8 +1146,7 @@ void CClient::HelpUser() {
 
 	Table.AddRow();
 	Table.SetCell("Command", "Topics");
-	Table.SetCell("Arguments", "");
-	Table.SetCell("Description", "Show topics in all channels");
+	Table.SetCell("Description", "Show topics in all your channels");
 
 	Table.AddRow();
 	Table.SetCell("Command", "PlayBuffer");
@@ -1053,7 +1160,6 @@ void CClient::HelpUser() {
 
 	Table.AddRow();
 	Table.SetCell("Command", "ClearAllChannelBuffers");
-	Table.SetCell("Arguments", "");
 	Table.SetCell("Description", "Clear the channel buffers");
 
 	Table.AddRow();
@@ -1064,7 +1170,7 @@ void CClient::HelpUser() {
 	if (m_pUser->IsAdmin()) {
 		Table.AddRow();
 		Table.SetCell("Command", "AddVHost");
-		Table.SetCell("Arguments", "<vhost (ip preferred)>");
+		Table.SetCell("Arguments", "<vhost (IP preferred)>");
 		Table.SetCell("Description", "Adds a VHost for normal users to use");
 
 		Table.AddRow();
@@ -1076,33 +1182,29 @@ void CClient::HelpUser() {
 	if (m_pUser->IsAdmin() || !m_pUser->DenySetVHost()) {
 		Table.AddRow();
 		Table.SetCell("Command", "ListVHosts");
-		Table.SetCell("Arguments", "");
 		Table.SetCell("Description", "Shows the configured list of vhosts");
 
 		Table.AddRow();
 		Table.SetCell("Command", "SetVHost");
-		Table.SetCell("Arguments", "<vhost (ip preferred)>");
+		Table.SetCell("Arguments", "<vhost (IP preferred)>");
 		Table.SetCell("Description", "Set the VHost for this connection");
 
 		Table.AddRow();
 		Table.SetCell("Command", "ClearVHost");
-		Table.SetCell("Arguments", "");
 		Table.SetCell("Description", "Clear the VHost for this connection");
 	}
 
 	Table.AddRow();
 	Table.SetCell("Command", "Jump");
-	Table.SetCell("Arguments", "");
 	Table.SetCell("Description", "Jump to the next server in the list");
 
 	Table.AddRow();
 	Table.SetCell("Command", "Disconnect");
-	Table.SetCell("Arguments", "");
+	Table.SetCell("Arguments", "[message]");
 	Table.SetCell("Description", "Disconnect from IRC");
 
 	Table.AddRow();
 	Table.SetCell("Command", "Connect");
-	Table.SetCell("Arguments", "");
 	Table.SetCell("Description", "Reconnect to IRC");
 
 	Table.AddRow();
@@ -1117,8 +1219,7 @@ void CClient::HelpUser() {
 
 	Table.AddRow();
 	Table.SetCell("Command", "Uptime");
-	Table.SetCell("Arguments", "");
-	Table.SetCell("Description", "Show how long ZNC is already running");
+	Table.SetCell("Description", "Show for how long ZNC has been running");
 
 	if (!m_pUser->DenyLoadMod()) {
 		Table.AddRow();
@@ -1146,64 +1247,77 @@ void CClient::HelpUser() {
 
 	Table.AddRow();
 	Table.SetCell("Command", "ShowMOTD");
-	Table.SetCell("Arguments", "");
-	Table.SetCell("Description", "Show the message of the day");
+	Table.SetCell("Description", "Show ZNC's message of the day");
 
 	if (m_pUser->IsAdmin()) {
 		Table.AddRow();
 		Table.SetCell("Command", "SetMOTD");
 		Table.SetCell("Arguments", "<Message>");
-		Table.SetCell("Description", "Set the message of the day");
+		Table.SetCell("Description", "Set ZNC's message of the day");
 
 		Table.AddRow();
 		Table.SetCell("Command", "AddMOTD");
 		Table.SetCell("Arguments", "<Message>");
-		Table.SetCell("Description", "Append <Message> to MOTD");
+		Table.SetCell("Description", "Append <Message> to ZNC's MOTD");
 
 		Table.AddRow();
 		Table.SetCell("Command", "ClearMOTD");
-		Table.SetCell("Arguments", "");
-		Table.SetCell("Description", "Clear the MOTD");
+		Table.SetCell("Description", "Clear ZNC's MOTD");
+
+		Table.AddRow();
+		Table.SetCell("Command", "ListPorts");
+		Table.SetCell("Description", "Show all active listeners");
+
+		Table.AddRow();
+		Table.SetCell("Command", "AddPort");
+		Table.SetCell("Arguments", "<[+]port> <ipv4|ipv6|all> <web|irc|all> [bindhost]");
+		Table.SetCell("Description", "Add another port for ZNC to listen on");
+
+		Table.AddRow();
+		Table.SetCell("Command", "DelPort");
+		Table.SetCell("Arguments", "<port> <ipv4|ipv6|all> [bindhost]");
+		Table.SetCell("Description", "Remove a port from ZNC");
 
 		Table.AddRow();
 		Table.SetCell("Command", "Rehash");
-		Table.SetCell("Arguments", "");
 		Table.SetCell("Description", "Reload znc.conf from disk");
 
 		Table.AddRow();
 		Table.SetCell("Command", "SaveConfig");
-		Table.SetCell("Arguments", "");
 		Table.SetCell("Description", "Save the current settings to disk");
 
 		Table.AddRow();
 		Table.SetCell("Command", "ListUsers");
-		Table.SetCell("Arguments", "");
-		Table.SetCell("Description", "List all users/clients connected to znc");
+		Table.SetCell("Description", "List all ZNC users and their connection status");
+
+		Table.AddRow();
+		Table.SetCell("Command", "ListChans");
+		Table.SetCell("Arguments", "[User]");
+		Table.SetCell("Description", "List all channels");
 
 		Table.AddRow();
 		Table.SetCell("Command", "ListClients");
 		Table.SetCell("Arguments", "[User]");
-		Table.SetCell("Description", "List all clients connected to your znc user");
+		Table.SetCell("Description", "List all connected clients");
 
 		Table.AddRow();
 		Table.SetCell("Command", "Traffic");
-		Table.SetCell("Arguments", "");
-		Table.SetCell("Description", "Show basic traffic stats for all znc users");
+		Table.SetCell("Description", "Show basic traffic stats for all ZNC users");
 
 		Table.AddRow();
 		Table.SetCell("Command", "Broadcast");
 		Table.SetCell("Arguments", "[message]");
-		Table.SetCell("Description", "Broadcast a message to all users");
+		Table.SetCell("Description", "Broadcast a message to all ZNC users");
 
 		Table.AddRow();
 		Table.SetCell("Command", "Shutdown");
 		Table.SetCell("Arguments", "[message]");
-		Table.SetCell("Description", "Shutdown znc completely");
+		Table.SetCell("Description", "Shut down ZNC completely");
 
 		Table.AddRow();
 		Table.SetCell("Command", "Restart");
 		Table.SetCell("Arguments", "[message]");
-		Table.SetCell("Description", "Restarts znc");
+		Table.SetCell("Description", "Restart ZNC");
 	}
 
 	PutStatus(Table);
