@@ -28,7 +28,7 @@
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *
-* $Revision: 1.105 $
+* $Revision: 1.138 $
 */
 
 #include "Csocket.h"
@@ -36,6 +36,10 @@
 #include <sys/param.h>
 #endif /* __NetBSD__ */
 
+#ifdef HAVE_LIBSSL
+#include <openssl/conf.h>
+#include <openssl/engine.h>
+#endif /* HAVE_LIBSSL */
 
 #include <list>
 
@@ -43,6 +47,8 @@
 
 using namespace std;
 
+#define CREATE_ARES_VER( a, b, c ) ((a<<16)|(b<<8)|c)
+
 #ifndef _NO_CSOCKET_NS // some people may not want to use a namespace
 namespace Csocket
 {
@@ -78,26 +84,50 @@ static const char *inet_ntop(int af, const void *src, char *dst, socklen_t cnt)
 	return( NULL );
 }
 
-static inline void set_non_blocking(int fd)
+#if defined(_WIN32) && (!defined(_WIN32_WINNT) || (_WIN32_WINNT < 0x0600))
+//! thanks to KiNgMaR @ #znc for this wrapper
+static int inet_pton( int af, const char *src, void *dst )
+{
+	sockaddr_storage aAddress;
+	int iAddrLen = sizeof( sockaddr_storage );
+	memset( &aAddress, 0, iAddrLen );
+	char *pTmp = strdup( src );
+	aAddress.ss_family = af; // this is important:
+	// The function fails if the sin_family member of the SOCKADDR_IN structure is not set to AF_INET or AF_INET6.
+	int iRet = WSAStringToAddressA( pTmp, af, NULL, (sockaddr *)&aAddress, &iAddrLen );
+	free( pTmp );
+	if( iRet == 0 )
+	{
+		if( af == AF_INET6 )
+			 memcpy(dst, &((sockaddr_in6 *)&aAddress)->sin6_addr, sizeof(in6_addr));
+		else
+			 memcpy(dst, &((sockaddr_in *)&aAddress)->sin_addr, sizeof(in_addr));
+		return( 1 );
+	}
+	return( -1 );
+}
+#endif
+
+static inline void set_non_blocking(cs_sock_t fd)
 {
 	u_long iOpts = 1;
 	ioctlsocket( fd, FIONBIO, &iOpts );
 }
 
-static inline void set_blocking(int fd)
+static inline void set_blocking(cs_sock_t fd)
 {
 	u_long iOpts = 0;
 	ioctlsocket( fd, FIONBIO, &iOpts );
 }
 
-static inline void set_close_on_exec(int fd)
+static inline void set_close_on_exec(cs_sock_t fd)
 {
 	// TODO add this for windows
 	// see http://gcc.gnu.org/ml/java-patches/2002-q1/msg00696.html
 	// for infos on how to do this
 }
 #else
-static inline void set_non_blocking(int fd)
+static inline void set_non_blocking(cs_sock_t fd)
 {
 	int fdflags = fcntl(fd, F_GETFL, 0);
 	if ( fdflags < 0 )
@@ -105,7 +135,7 @@ static inline void set_non_blocking(int fd)
 	fcntl( fd, F_SETFL, fdflags|O_NONBLOCK );
 }
 
-static inline void set_blocking(int fd)
+static inline void set_blocking(cs_sock_t fd)
 {
 	int fdflags = fcntl(fd, F_GETFL, 0);
 	if ( fdflags < 0 )
@@ -114,7 +144,7 @@ static inline void set_blocking(int fd)
 	fcntl( fd, F_SETFL, fdflags );
 }
 
-static inline void set_close_on_exec(int fd)
+static inline void set_close_on_exec(cs_sock_t fd)
 {
 	int fdflags = fcntl(fd, F_GETFD, 0);
 	if ( fdflags < 0 )
@@ -197,6 +227,49 @@ static int __GetHostByName( const CS_STRING & sHostName, struct in_addr *paddr,
 }
 #endif /* !HAVE_IPV6 */
 
+#ifdef HAVE_C_ARES
+void Csock::FreeAres()
+{
+	if( m_pARESChannel )
+	{
+		ares_destroy( m_pARESChannel );
+		m_pARESChannel = NULL;
+	}
+}
+#endif /* HAVE_C_ARES */
+
+#ifdef HAVE_C_ARES
+static void AresHostCallback( void *pArg, int status, int timeouts, struct hostent *hent )
+{
+	Csock *pSock = (Csock *)pArg;
+	if( status == ARES_SUCCESS && hent )
+	{
+		CSSockAddr *pSockAddr = pSock->GetCurrentAddr();
+		if( hent->h_addrtype == AF_INET )
+		{
+			pSock->SetIPv6( false );
+			memcpy( pSockAddr->GetAddr(), hent->h_addr_list[0], sizeof( *(pSockAddr->GetAddr()) ) );
+		}
+#ifdef HAVE_IPV6
+		else if( hent->h_addrtype == AF_INET6 )
+		{
+			pSock->SetIPv6( true );
+			memcpy( pSockAddr->GetAddr6(), hent->h_addr_list[0], sizeof( *(pSockAddr->GetAddr6()) ) );
+		}
+#endif /* HAVE_IPV6 */
+		else
+		{
+			status = ARES_ENOTFOUND;
+		}
+	}
+	else
+	{
+		CS_DEBUG( ares_strerror( status ) );
+	}
+	pSock->SetAresFinished( status );
+}
+#endif /* HAVE_C_ARES */
+
 int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockAddr )
 {
 #ifndef HAVE_IPV6
@@ -294,6 +367,7 @@ int GetAddrInfo( const CS_STRING & sHostname, Csock *pSock, CSSockAddr & csSockA
 					bFound = true;
 					break;
 				}
+				pSock->CloseSocksFD();
 			}
 			else if( bTryConnect )
 			{
@@ -319,6 +393,12 @@ bool InitCsocket()
 	if( iResult != NO_ERROR )
 		return( false );
 #endif /* _WIN32 */
+#ifdef HAVE_C_ARES
+#if ARES_VERSION >= CREATE_ARES_VER( 1, 6, 1 )
+	if( ares_library_init( ARES_LIB_INIT_ALL ) != 0 )
+		return( false );
+#endif /* ARES_VERSION >= CREATE_ARES_VER( 1, 6, 1 ) */
+#endif /* HAVE_C_ARES */
 #ifdef HAVE_LIBSSL
 	if( !InitSSL() )
 		return( false );
@@ -329,8 +409,18 @@ bool InitCsocket()
 void ShutdownCsocket()
 {
 #ifdef HAVE_LIBSSL
+	ERR_remove_state(0);
+	ENGINE_cleanup();
+	CONF_modules_unload(1);
 	ERR_free_strings();
+	EVP_cleanup();
+	CRYPTO_cleanup_all_ex_data();
 #endif /* HAVE_LIBSSL */
+#ifdef HAVE_C_ARES
+#if ARES_VERSION >= CREATE_ARES_VER( 1, 6, 1 )
+	ares_library_cleanup();
+#endif /* ARES_VERSION >= CREATE_ARES_VER( 1, 6, 1 ) */
+#endif /* HAVE_C_ARES */
 #ifdef _WIN32
 	WSACleanup();
 #endif /* _WIN32 */
@@ -530,31 +620,43 @@ Csock *Csock::GetSockObj( const CS_STRING & sHostname, u_short iPort )
 
 Csock::~Csock()
 {
+#ifdef HAVE_C_ARES
+	if( m_pARESChannel )
+		ares_cancel( m_pARESChannel );
+	FreeAres();
+#endif /* HAVE_C_ARES */
+
 #ifdef HAVE_LIBSSL
 	FREE_SSL();
 	FREE_CTX();
 #endif /* HAVE_LIBSSL */
 
-	if ( m_iReadSock != m_iWriteSock )
-	{
-		if( m_iReadSock >= 0 )
-			CS_CLOSE( m_iReadSock );
-		if( m_iWriteSock >= 0 )
-			CS_CLOSE( m_iWriteSock );
-	} else if( m_iReadSock >= 0 )
-		CS_CLOSE( m_iReadSock );
-
-	m_iReadSock = -1;
-	m_iWriteSock = -1;
+	CloseSocksFD();
 
 	// delete any left over crons
 	for( vector<CCron *>::size_type i = 0; i < m_vcCrons.size(); i++ )
 		CS_Delete( m_vcCrons[i] );
 }
 
+void Csock::CloseSocksFD()
+{
+	if ( m_iReadSock != m_iWriteSock )
+	{
+		if( m_iReadSock != CS_INVALID_SOCK )
+			CS_CLOSE( m_iReadSock );
+		if( m_iWriteSock != CS_INVALID_SOCK )
+			CS_CLOSE( m_iWriteSock );
+	} else if( m_iReadSock != CS_INVALID_SOCK )
+		CS_CLOSE( m_iReadSock );
+
+	m_iReadSock = CS_INVALID_SOCK;
+	m_iWriteSock = CS_INVALID_SOCK;
+}
+
+
 void Csock::Dereference()
 {
-	m_iWriteSock = m_iReadSock = -1;
+	m_iWriteSock = m_iReadSock = CS_INVALID_SOCK;
 
 #ifdef HAVE_LIBSSL
 	m_ssl = NULL;
@@ -569,7 +671,7 @@ void Csock::Copy( const Csock & cCopy )
 {
 	m_iTcount		= cCopy.m_iTcount;
 	m_iLastCheckTimeoutTime	=	cCopy.m_iLastCheckTimeoutTime;
-	m_iport 		= cCopy.m_iport;
+	m_uPort 		= cCopy.m_uPort;
 	m_iRemotePort	= cCopy.m_iRemotePort;
 	m_iLocalPort	= cCopy.m_iLocalPort;
 	m_iReadSock		= cCopy.m_iReadSock;
@@ -610,6 +712,11 @@ void Csock::Copy( const Csock & cCopy )
 	m_bindhost			= cCopy.m_bindhost;
 	m_bIsIPv6			= cCopy.m_bIsIPv6;
 	m_bSkipConnect		= cCopy.m_bSkipConnect;
+#ifdef HAVE_C_ARES
+	FreeAres(); // Not copying this state, but making sure its nulled out
+	m_iARESStatus = -1; // set it to unitialized
+	m_pCurrAddr = NULL;
+#endif /* HAVE_C_ARES */
 
 #ifdef HAVE_LIBSSL
 	m_iRequireClientCertFlags = cCopy.m_iRequireClientCertFlags;
@@ -624,7 +731,7 @@ void Csock::Copy( const Csock & cCopy )
 
 #endif /* HAVE_LIBSSL */
 
-	if( m_vcCrons.size() )
+	if( !m_vcCrons.empty() )
 	{
 		for( u_long a = 0; a < m_vcCrons.size(); a++ )
 		{
@@ -791,7 +898,7 @@ bool Csock::Connect( const CS_STRING & sBindHost, bool bSkipSetup )
 
 int Csock::WriteSelect()
 {
-	if ( m_iWriteSock < 0 )
+	if ( m_iWriteSock == CS_INVALID_SOCK )
 		return( SEL_ERR );
 
 	struct timeval tv;
@@ -821,7 +928,7 @@ int Csock::WriteSelect()
 
 int Csock::ReadSelect()
 {
-	if ( m_iReadSock < 0 )
+	if ( m_iReadSock == CS_INVALID_SOCK )
 		return( SEL_ERR );
 
 	struct timeval tv;
@@ -863,14 +970,26 @@ bool Csock::Listen( u_short iPort, int iMaxConns, const CS_STRING & sBindHost, u
 			return( false );
 	}
 
-	m_iReadSock = m_iWriteSock = SOCKET( true );
+	m_iReadSock = m_iWriteSock = CreateSocket( true );
 
-	if ( m_iReadSock == -1 )
+	if ( m_iReadSock == CS_INVALID_SOCK )
 		return( false );
 
+#ifdef HAVE_IPV6
+// there's no IPPROTO_IPV6 below Win XP. - KiNgMaR
+#if (!defined(_WIN32) && defined(IPV6_V6ONLY)) || (defined(_WIN32_WINNT) && _WIN32_WINNT >= 0x0501)
+	if( GetIPv6() )
+	{
+		// per RFC3493#5.3
+		const int on = ( m_address.GetAFRequire() == CSSockAddr::RAF_INET6 ? 1 : 0 );
+		if( setsockopt( m_iReadSock, IPPROTO_IPV6, IPV6_V6ONLY, (char *)&on, sizeof( on ) ) != 0 )
+			PERROR( "IPV6_V6ONLY" );
+	}
+#endif /* IPV6_V6ONLY */
+#endif /* HAVE_IPV6 */
+
 	m_address.SinFamily();
 	m_address.SinPort( iPort );
-
 	if( !GetIPv6() )
 	{
 		if ( bind( m_iReadSock, (struct sockaddr *) m_address.GetSockAddr(), m_address.GetSockAddrLen() ) == -1 )
@@ -896,41 +1015,37 @@ bool Csock::Listen( u_short iPort, int iMaxConns, const CS_STRING & sBindHost, u
 	return( true );
 }
 
-int Csock::Accept( CS_STRING & sHost, u_short & iRPort )
+cs_sock_t Csock::Accept( CS_STRING & sHost, u_short & iRPort )
 {
-	int iSock = -1;
+	cs_sock_t iSock = CS_INVALID_SOCK;
 	if( !GetIPv6() )
 	{
 		struct sockaddr_in client;
 		socklen_t clen = sizeof( client );
 		iSock = accept( m_iReadSock, (struct sockaddr *) &client, &clen );
-		if( iSock != -1 )
+		if( iSock != CS_INVALID_SOCK )
 		{
 			getpeername( iSock, (struct sockaddr *) &client, &clen );
-			sHost = inet_ntoa( client.sin_addr );
+			sHost = ConvertAddress( &client.sin_addr, false );
 			iRPort = ntohs( client.sin_port );
 		}
 	}
 #ifdef HAVE_IPV6
 	else
 	{
-		char straddr[INET6_ADDRSTRLEN];
 		struct sockaddr_in6 client;
 		socklen_t clen = sizeof( client );
 		iSock = accept( m_iReadSock, (struct sockaddr *) &client, &clen );
-		if( iSock != -1 )
+		if( iSock != CS_INVALID_SOCK )
 		{
 			getpeername( iSock, (struct sockaddr *) &client, &clen );
-			if( inet_ntop( AF_INET6, &client.sin6_addr, straddr, sizeof(straddr) ) > 0 )
-			{
-				sHost = straddr;
-				iRPort = ntohs( client.sin6_port );
-			}
+			sHost = ConvertAddress( &client.sin6_addr, true );
+			iRPort = ntohs( client.sin6_port );
 		}
 	}
 #endif /* HAVE_IPV6 */
 
-	if ( iSock != -1 )
+	if ( iSock != CS_INVALID_SOCK )
 	{
 		// Make it close-on-exec
 		set_close_on_exec( iSock );
@@ -944,7 +1059,7 @@ int Csock::Accept( CS_STRING & sHost, u_short & iRPort )
 		if ( !ConnectionFrom( sHost, iRPort ) )
 		{
 			CS_CLOSE( iSock );
-			iSock = -1;
+			iSock = CS_INVALID_SOCK;
 		}
 
 	}
@@ -988,6 +1103,15 @@ bool Csock::SSLClientSetup()
 	FREE_SSL();
 	FREE_CTX();
 
+#ifdef _WIN64
+	if( m_iReadSock != (int)m_iReadSock || m_iWriteSock != (int)m_iWriteSock )
+	{
+		// sanity check the FD to be sure its compatible with openssl
+		CS_DEBUG( "ERROR: sockfd larger than OpenSSL can handle" );
+		return( false );
+	}
+#endif /* _WIN64 */
+
 	switch( m_iMethod )
 	{
 		case SSL2:
@@ -1053,8 +1177,8 @@ bool Csock::SSLClientSetup()
 	if ( !m_ssl )
 		return( false );
 
-	SSL_set_rfd( m_ssl, m_iReadSock );
-	SSL_set_wfd( m_ssl, m_iWriteSock );
+	SSL_set_rfd( m_ssl, (int)m_iReadSock );
+	SSL_set_wfd( m_ssl, (int)m_iWriteSock );
 	SSL_set_verify( m_ssl, SSL_VERIFY_PEER, ( m_pCerVerifyCB ? m_pCerVerifyCB : CertVerifyCB ) );
 	SSL_set_ex_data( m_ssl, GetCsockClassIdx(), this );
 
@@ -1073,6 +1197,16 @@ bool Csock::SSLServerSetup()
 	FREE_SSL();
 	FREE_CTX();
 
+#ifdef _WIN64
+	if( m_iReadSock != (int)m_iReadSock || m_iWriteSock != (int)m_iWriteSock )
+	{
+		// sanity check the FD to be sure its compatible with openssl
+		CS_DEBUG( "ERROR: sockfd larger than OpenSSL can handle" );
+		return( false );
+	}
+#endif /* _WIN64 */
+
+
 	switch( m_iMethod )
 	{
 		case SSL2:
@@ -1154,8 +1288,8 @@ bool Csock::SSLServerSetup()
 		return( false );
 
 	// Call for client Verification
-	SSL_set_rfd( m_ssl, m_iReadSock );
-	SSL_set_wfd( m_ssl, m_iWriteSock );
+	SSL_set_rfd( m_ssl, (int)m_iReadSock );
+	SSL_set_wfd( m_ssl, (int)m_iWriteSock );
 	SSL_set_accept_state( m_ssl );
 	if ( m_iRequireClientCertFlags )
 	{
@@ -1173,7 +1307,7 @@ bool Csock::SSLServerSetup()
 bool Csock::ConnectSSL( const CS_STRING & sBindhost )
 {
 #ifdef HAVE_LIBSSL
-	if ( m_iReadSock == -1 )
+	if ( m_iReadSock == CS_INVALID_SOCK )
 		if ( !Connect( sBindhost ) )
 			return( false );
 	if ( !m_ssl )
@@ -1236,7 +1370,7 @@ bool Csock::AllowWrite( unsigned long long & iNOW ) const
 	return( true );
 }
 
-bool Csock::Write( const char *data, int len )
+bool Csock::Write( const char *data, size_t len )
 {
 	m_sSend.append( data, len );
 
@@ -1253,7 +1387,7 @@ bool Csock::Write( const char *data, int len )
 
 	}
 	// rate shaping
-	u_int iBytesToSend = 0;
+	u_long iBytesToSend = 0;
 
 #ifdef HAVE_LIBSSL
 	if( m_bssl && m_sSSLBuffer.empty() && !m_bsslEstablished )
@@ -1298,7 +1432,7 @@ bool Csock::Write( const char *data, int len )
 		if ( m_sSSLBuffer.empty() ) // on retrying to write data, ssl wants the data in the SAME spot and the SAME size
 			m_sSSLBuffer.append( m_sSend.data(), iBytesToSend );
 
-		int iErr = SSL_write( m_ssl, m_sSSLBuffer.data(), m_sSSLBuffer.length() );
+		int iErr = SSL_write( m_ssl, m_sSSLBuffer.data(), (int)m_sSSLBuffer.length() );
 
 		if ( ( iErr < 0 ) && ( GetSockError() == ECONNREFUSED ) )
 		{
@@ -1351,9 +1485,9 @@ bool Csock::Write( const char *data, int len )
 	}
 #endif /* HAVE_LIBSSL */
 #ifdef _WIN32
-	int bytes = send( m_iWriteSock, m_sSend.data(), iBytesToSend, 0 );
+	cs_ssize_t bytes = send( m_iWriteSock, m_sSend.data(), iBytesToSend, 0 );
 #else
-	int bytes = write( m_iWriteSock, m_sSend.data(), iBytesToSend );
+	cs_ssize_t bytes = write( m_iWriteSock, m_sSend.data(), iBytesToSend );
 #endif /* _WIN32 */
 
 	if ( ( bytes == -1 ) && ( GetSockError() == ECONNREFUSED ) )
@@ -1387,9 +1521,9 @@ bool Csock::Write( const CS_STRING & sData )
 	return( Write( sData.c_str(), sData.length() ) );
 }
 
-int Csock::Read( char *data, int len )
+cs_ssize_t Csock::Read( char *data, size_t len )
 {
-	int bytes = 0;
+	cs_ssize_t bytes = 0;
 
 	if ( ( IsReadPaused() ) && ( SslIsEstablished() ) )
 		return( READ_EAGAIN ); // allow the handshake to complete first
@@ -1409,7 +1543,7 @@ int Csock::Read( char *data, int len )
 
 #ifdef HAVE_LIBSSL
 	if ( m_bssl )
-		bytes = SSL_read( m_ssl, data, len );
+		bytes = SSL_read( m_ssl, data, (int)len );
 	else
 #endif /* HAVE_LIBSSL */
 #ifdef _WIN32
@@ -1436,7 +1570,7 @@ int Csock::Read( char *data, int len )
 #ifdef HAVE_LIBSSL
 		if ( m_bssl )
 		{
-			int iErr = SSL_get_error( m_ssl, bytes );
+			int iErr = SSL_get_error( m_ssl, (int)bytes );
 			if ( ( iErr != SSL_ERROR_WANT_READ ) && ( iErr != SSL_ERROR_WANT_WRITE ) )
 				return( READ_ERR );
 			else
@@ -1458,9 +1592,9 @@ CS_STRING Csock::GetLocalIP()
 	if ( !m_sLocalIP.empty() )
 		return( m_sLocalIP );
 
-	int iSock = GetSock();
+	cs_sock_t iSock = GetSock();
 
-	if ( iSock < 0 )
+	if ( iSock == CS_INVALID_SOCK )
 		return( "" );
 
 	if( !GetIPv6() )
@@ -1492,47 +1626,59 @@ CS_STRING Csock::GetRemoteIP()
 	if ( !m_sRemoteIP.empty() )
 		return( m_sRemoteIP );
 
-	int iSock = GetSock();
+	cs_sock_t iSock = GetSock();
 
-	if ( iSock < 0 )
-	{
-		std::cerr << "What the hell is wrong with my fd!?" << endl;
+	if ( iSock == CS_INVALID_SOCK )
 		return( "" );
-	}
 
 	if( !GetIPv6() )
 	{
 		struct sockaddr_in mRemoteAddr;
 		socklen_t mRemoteLen = sizeof( mRemoteAddr );
 		if ( getpeername( iSock, (struct sockaddr *) &mRemoteAddr, &mRemoteLen ) == 0 )
-			m_sRemoteIP = inet_ntoa( mRemoteAddr.sin_addr );
+			m_sRemoteIP = ConvertAddress( &mRemoteAddr.sin_addr, false );
 	}
 #ifdef HAVE_IPV6
 	else
 	{
-		char straddr[INET6_ADDRSTRLEN];
 		struct sockaddr_in6 mRemoteAddr;
 		socklen_t mRemoteLen = sizeof( mRemoteAddr );
-		if ( ( getpeername( iSock, (struct sockaddr *) &mRemoteAddr, &mRemoteLen ) == 0 )
-			&& ( inet_ntop( AF_INET6, &mRemoteAddr.sin6_addr, straddr, sizeof(straddr) ) ) )
-		{
-			m_sRemoteIP = straddr;
-		}
+		if ( getpeername( iSock, (struct sockaddr *) &mRemoteAddr, &mRemoteLen ) == 0 )
+			m_sRemoteIP = ConvertAddress( &mRemoteAddr.sin6_addr, true );
 	}
 #endif /* HAVE_IPV6 */
 
 	return( m_sRemoteIP );
 }
 
+CS_STRING Csock::ConvertAddress( void *addr, bool bIPv6 )
+{
+	CS_STRING sRet;
+
+	if( !bIPv6 ) 
+	{
+		in_addr *p = (in_addr*) addr;
+		sRet = inet_ntoa(*p);
+	} 
+	else 
+	{
+		char straddr[INET6_ADDRSTRLEN];
+		if( inet_ntop( AF_INET6, addr, straddr, sizeof(straddr) ) > 0 )
+			sRet = straddr;
+	}
+
+	return( sRet );
+}
+
 bool Csock::IsConnected() { return( m_bIsConnected ); }
 void Csock::SetIsConnected( bool b ) { m_bIsConnected = b; }
 
-int & Csock::GetRSock() { return( m_iReadSock ); }
-void Csock::SetRSock( int iSock ) { m_iReadSock = iSock; }
-int & Csock::GetWSock() { return( m_iWriteSock ); }
-void Csock::SetWSock( int iSock ) { m_iWriteSock = iSock; }
-void Csock::SetSock( int iSock ) { m_iWriteSock = iSock; m_iReadSock = iSock; }
-int & Csock::GetSock() { return( m_iReadSock ); }
+cs_sock_t & Csock::GetRSock() { return( m_iReadSock ); }
+void Csock::SetRSock( cs_sock_t iSock ) { m_iReadSock = iSock; }
+cs_sock_t & Csock::GetWSock() { return( m_iWriteSock ); }
+void Csock::SetWSock( cs_sock_t iSock ) { m_iWriteSock = iSock; }
+void Csock::SetSock( cs_sock_t iSock ) { m_iWriteSock = iSock; m_iReadSock = iSock; }
+cs_sock_t & Csock::GetSock() { return( m_iReadSock ); }
 void Csock::ResetTimer() { m_iLastCheckTimeoutTime = 0; m_iTcount = 0; }
 void Csock::PauseRead() { m_bPauseRead = true; }
 bool Csock::IsReadPaused() { return( m_bPauseRead ); }
@@ -1599,12 +1745,12 @@ bool Csock::CheckTimeout( time_t iNow )
 	return( false );
 }
 
-void Csock::PushBuff( const char *data, int len, bool bStartAtZero )
+void Csock::PushBuff( const char *data, size_t len, bool bStartAtZero )
 {
 	if ( !m_bEnableReadLine )
 		return;	// If the ReadLine event is disabled, just ditch here
 
-	u_int iStartPos = ( m_sbuffer.empty() || bStartAtZero ? 0 : m_sbuffer.length() - 1 );
+	size_t iStartPos = ( m_sbuffer.empty() || bStartAtZero ? 0 : m_sbuffer.length() - 1 );
 
 	if ( data )
 		m_sbuffer.append( data, len );
@@ -1671,9 +1817,9 @@ u_short Csock::GetRemotePort()
 	if ( m_iRemotePort > 0 )
 		return( m_iRemotePort );
 
-	int iSock = GetSock();
+	cs_sock_t iSock = GetSock();
 
-	if ( iSock >= 0 )
+	if ( iSock != CS_INVALID_SOCK )
 	{
 		if( !GetIPv6() )
 		{
@@ -1701,9 +1847,9 @@ u_short Csock::GetLocalPort()
 	if ( m_iLocalPort > 0 )
 		return( m_iLocalPort );
 
-	int iSock = GetSock();
+	cs_sock_t iSock = GetSock();
 
-	if ( iSock >= 0 )
+	if ( iSock != CS_INVALID_SOCK )
 	{
 		if( !GetIPv6() )
 		{
@@ -1726,8 +1872,8 @@ u_short Csock::GetLocalPort()
 	return( m_iLocalPort );
 }
 
-u_short Csock::GetPort() { return( m_iport ); }
-void Csock::SetPort( u_short iPort ) { m_iport = iPort; }
+u_short Csock::GetPort() { return( m_uPort ); }
+void Csock::SetPort( u_short iPort ) { m_uPort = iPort; }
 void Csock::Close( ECloseType eCloseType )
 {
 	m_eCloseType = eCloseType;
@@ -1764,7 +1910,7 @@ int Csock::PemPassCB( char *buf, int size, int rwflag, void *pcSocket )
 	memset( buf, '\0', size );
 	strncpy( buf, sPassword.c_str(), size );
 	buf[size-1] = '\0';
-	return( strlen( buf ) );
+	return( (int)strlen( buf ) );
 }
 
 int Csock::CertVerifyCB( int preverify_ok, X509_STORE_CTX *x509_ctx )
@@ -1904,6 +2050,29 @@ CS_STRING Csock::GetPeerPubKey()
 	}
 	return( sKey );
 }
+int Csock::GetPeerFingerprint( CS_STRING & sFP )
+{
+	sFP.clear();
+
+	if ( !GetSSL() )
+		return 0;
+
+	X509* pCert = getX509();
+
+	// Inspired by charybdis
+	if ( pCert )
+	{
+		for (int i = 0; i < SHA_DIGEST_LENGTH; i++)
+		{
+			char buf[3];
+			snprintf(buf, 3, "%02x", pCert->sha1_hash[i]);
+			sFP += buf;
+		}
+		X509_free(pCert);
+	}
+
+	return SSL_get_verify_result(m_ssl);
+}
 unsigned int Csock::GetRequireClientCertFlags() { return( m_iRequireClientCertFlags ); }
 void Csock::SetRequiresClientCert( bool bRequiresCert ) { m_iRequireClientCertFlags = ( bRequiresCert ? SSL_VERIFY_FAIL_IF_NO_PEER_CERT|SSL_VERIFY_PEER : 0 ); }
 
@@ -1996,7 +2165,24 @@ int Csock::GetPending()
 {
 #ifdef HAVE_LIBSSL
 	if( m_ssl )
-		return( SSL_pending( m_ssl ) );
+	{
+		// in v23 method, the pending function is initialized to ssl_undefined_const_function
+		// which throws SSL_UNDEFINED_CONST_FUNCTION on to the error stack
+		// this is one of the more stupid things in openssl, it seems bizarre that even though SSL_pending
+		// returns an int, they don't bother returning in error to notify us, so basically
+		// we have to always clear errors here generated by SSL_pending, otherwise the stack could
+		// have a lame error on it causing SSL_write to fail in certain instances.
+#if defined( OPENSSL_VERSION_NUMBER ) && OPENSSL_VERSION_NUMBER >= 0x00908000
+		ERR_set_mark();
+		int iBytes = SSL_pending( m_ssl );
+		ERR_pop_to_mark();
+		return( iBytes );
+#else 
+		int iBytes = SSL_pending( m_ssl );
+		ERR_clear_error(); // to get safer handling, upgrade your openssl version!
+		return( iBytes );
+#endif /* OPENSSL_VERSION_NUMBER */
+	}
 	else
 		return( 0 );
 #else
@@ -2006,6 +2192,78 @@ int Csock::GetPending()
 
 int Csock::GetAddrInfo( const CS_STRING & sHostname, CSSockAddr & csSockAddr )
 {
+#ifdef HAVE_IPV6
+	if( csSockAddr.GetAFRequire() != AF_INET && inet_pton( AF_INET6, sHostname.c_str(), csSockAddr.GetAddr6() ) > 0 )
+	{
+		SetIPv6( true );
+		return( 0 );
+	}
+#endif /* HAVE_IPV6 */
+	if( inet_pton( AF_INET, sHostname.c_str(), csSockAddr.GetAddr() ) > 0 )
+	{
+#ifdef HAVE_IPV6
+		SetIPv6( false );
+#endif /* HAVE_IPV6 */
+		return( 0 );
+	}
+
+#ifdef HAVE_C_ARES
+	if( GetType() != LISTENER )
+	{ // right now the current function in Listen() is it blocks, the easy way around this at the moment is to use ip
+		// need to compute this up here
+		if( !m_pARESChannel )
+		{
+			if( ares_init( &m_pARESChannel ) != ARES_SUCCESS )
+			{ // TODO throw some debug?
+				FreeAres();
+				return( ETIMEDOUT );
+			}
+			m_pCurrAddr = &csSockAddr; // flag its starting
+
+			int iFamily = AF_INET;
+#ifdef HAVE_IPV6
+			// as of ares 1.6.0 if it fails on af_inet6, it falls back to af_inet, this code was here in the previous Csocket version, just adding the comment as a reminder
+			iFamily = csSockAddr.GetAFRequire() == CSSockAddr::RAF_ANY ? AF_INET6 : csSockAddr.GetAFRequire();
+#endif /* HAVE_IPV6 */
+			ares_gethostbyname( m_pARESChannel, sHostname.c_str(), iFamily, AresHostCallback, this );
+		}
+		if( !m_pCurrAddr )
+		{ // this means its finished
+			FreeAres();
+#ifdef HAVE_IPV6
+			if( m_iARESStatus == ARES_SUCCESS && csSockAddr.GetAFRequire() == CSSockAddr::RAF_ANY && GetIPv6() )
+			{
+				// this means that ares_host returned an ipv6 host, so try a connect right away
+				if( CreateSocksFD() && Connect( GetBindHost(), true ) )
+				{
+					SetSkipConnect( true );
+				}
+				else if( GetSockError() == ENETUNREACH )
+				{
+					// the Connect() failed, so throw a retry back in with ipv4, and let it process normally
+					CS_DEBUG( "Failed ipv6 connection with PF_UNSPEC, falling back to ipv4" );
+					m_iARESStatus = -1;
+					CloseSocksFD();
+					SetAFRequire( CSSockAddr::RAF_INET );
+					return( GetAddrInfo( sHostname, csSockAddr ) );
+				}
+			}
+#if ARES_VERSION < CREATE_ARES_VER( 1, 5, 3 )
+			if( m_iARESStatus != ARES_SUCCESS && csSockAddr.GetAFRequire() == CSSockAddr::RAF_ANY )
+			{ // this is a workaround for ares < 1.5.3 where the builtin retry on failed AF_INET6 isn't there yet
+				CS_DEBUG( "Retry for older version of c-ares with AF_INET only" );
+				// this means we tried previously with AF_INET6 and failed, so force AF_INET and retry
+				SetAFRequire( CSSockAddr::RAF_INET );
+				return( GetAddrInfo( sHostname, csSockAddr ) );
+			}
+#endif /* ARES_VERSION < CREATE_ARES_VER( 1, 5, 3 ) */
+#endif /* HAVE_IPV6 */
+			return( m_iARESStatus == ARES_SUCCESS ? 0 : ETIMEDOUT );
+		}
+		return( EAGAIN );
+	}
+#endif /* HAVE_C_ARES */
+
 	return( ::GetAddrInfo( sHostname, this, csSockAddr ) );
 }
 
@@ -2058,12 +2316,14 @@ int Csock::DNSLookup( EDNSLType eDNSLType )
 	}
 	else if ( iRet == EAGAIN )
 	{
+#ifndef HAVE_C_ARES
 		m_iDNSTryCount++;
 		if ( m_iDNSTryCount > 20 )
 		{
 			m_iDNSTryCount = 0;
 			return( ETIMEDOUT );
 		}
+#endif /* HAVE_C_ARES */
 		return( EAGAIN );
 	}
 	m_iDNSTryCount = 0;
@@ -2123,31 +2383,33 @@ void Csock::FREE_CTX()
 
 #endif /* HAVE_LIBSSL */
 
-int Csock::SOCKET( bool bListen )
+cs_sock_t Csock::CreateSocket( bool bListen )
 {
 #ifdef HAVE_IPV6
-	int iRet = socket( ( GetIPv6() ? PF_INET6 : PF_INET ), SOCK_STREAM, IPPROTO_TCP );
+	cs_sock_t iRet = socket( ( GetIPv6() ? PF_INET6 : PF_INET ), SOCK_STREAM, IPPROTO_TCP );
 #else
-	int iRet = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
+	cs_sock_t iRet = socket( PF_INET, SOCK_STREAM, IPPROTO_TCP );
 #endif /* HAVE_IPV6 */
 
-	if ( iRet >= 0 ) {
+	if ( iRet != CS_INVALID_SOCK ) 
+	{
 		set_close_on_exec( iRet );
 
-		if ( bListen ) {
+		if ( bListen ) 
+		{
 			const int on = 1;
 
-			if ( setsockopt( iRet, SOL_SOCKET, SO_REUSEADDR,
-						(char *)&on, sizeof( on ) ) != 0 )
-				PERROR( "setsockopt" );
+			if ( setsockopt( iRet, SOL_SOCKET, SO_REUSEADDR, (char *)&on, sizeof( on ) ) != 0 )
+				PERROR( "SO_REUSEADDR" );
 		}
-	} else
+	} 
+	else
 		PERROR( "socket" );
 
 	return( iRet );
 }
 
-void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
+void Csock::Init( const CS_STRING & sHostname, u_short uPort, int itimeout )
 {
 #ifdef HAVE_LIBSSL
 	m_ssl = NULL;
@@ -2155,12 +2417,12 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 	m_iRequireClientCertFlags = 0;
 #endif /* HAVE_LIBSSL */
 	m_iTcount = 0;
-	m_iReadSock = -1;
-	m_iWriteSock = -1;
+	m_iReadSock = CS_INVALID_SOCK;
+	m_iWriteSock = CS_INVALID_SOCK;
 	m_itimeout = itimeout;
 	m_bssl = false;
 	m_bIsConnected = false;
-	m_iport = iport;
+	m_uPort = uPort;
 	m_shostname = sHostname;
 	m_sbuffer.clear();
 	m_eCloseType = CLT_DONT;
@@ -2189,5 +2451,10 @@ void Csock::Init( const CS_STRING & sHostname, u_short iport, int itimeout )
 	m_bIsIPv6 = false;
 	m_bSkipConnect = false;
 	m_iLastCheckTimeoutTime = 0;
+#ifdef HAVE_C_ARES
+	m_pARESChannel = NULL;
+	m_pCurrAddr = NULL;
+	m_iARESStatus = -1;
+#endif /* HAVE_C_ARES */
 }
 
