@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -85,13 +85,11 @@ void CDCCBounce::ReachedMaxBuffer() {
 	Close();
 }
 
-void CDCCBounce::ReadData(const char* data, int len) {
-	size_t BufLen;
-
+void CDCCBounce::ReadData(const char* data, size_t len) {
 	if (m_pPeer) {
 		m_pPeer->Write(data, len);
 
-		BufLen = m_pPeer->GetInternalWriteBuffer().length();
+		size_t BufLen = m_pPeer->GetInternalWriteBuffer().length();
 
 		if (BufLen >= m_uiMaxDCCBuffer) {
 			DEBUG(GetSockName() << " The send buffer is over the "
@@ -113,7 +111,7 @@ void CDCCBounce::Timeout() {
 	if (IsRemote()) {
 		CString sHost = Csock::GetHostName();
 		if (!sHost.empty()) {
-			sHost = " to [" + sHost + ":" + CString(Csock::GetPort()) + "]";
+			sHost = " to [" + sHost + " " + CString(Csock::GetPort()) + "]";
 		} else {
 			sHost = ".";
 		}
@@ -130,7 +128,7 @@ void CDCCBounce::ConnectionRefused() {
 	CString sType = (m_bIsChat) ? "Chat" : "Xfer";
 	CString sHost = Csock::GetHostName();
 	if (!sHost.empty()) {
-		sHost = " to [" + sHost + ":" + CString(Csock::GetPort()) + "]";
+		sHost = " to [" + sHost + " " + CString(Csock::GetPort()) + "]";
 	} else {
 		sHost = ".";
 	}
@@ -145,7 +143,7 @@ void CDCCBounce::SockError(int iErrno) {
 	if (IsRemote()) {
 		CString sHost = Csock::GetHostName();
 		if (!sHost.empty()) {
-			sHost = "[" + sHost + ":" + CString(Csock::GetPort()) + "]";
+			sHost = "[" + sHost + " " + CString(Csock::GetPort()) + "]";
 		}
 
 		m_pUser->PutStatus("DCC " + sType + " Bounce (" + m_sRemoteNick + "): Socket error [" + CString(strerror(iErrno)) + "]" + sHost);
