@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -55,7 +55,7 @@ CDCCSock::~CDCCSock() {
 	m_pUser->DelDCCSock(this);
 }
 
-void CDCCSock::ReadData(const char* data, int len) {
+void CDCCSock::ReadData(const char* data, size_t len) {
 	if (!m_pFile) {
 		DEBUG("File not open! closing get.");
 		m_pUser->PutModule(m_sModuleName, ((m_bSend) ? "DCC -> [" : "DCC <- [") + m_sRemoteNick + "][" + m_sFileName + "] - File not open!");
