@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -21,7 +21,7 @@ public:
 	CDCCSock(CUser* pUser, const CString& sRemoteNick, const CString& sRemoteIP, unsigned short uRemotePort, const CString& sLocalFile, unsigned long uFileSize, const CString& sModuleName);
 	virtual ~CDCCSock();
 
-	virtual void ReadData(const char* data, int len);
+	virtual void ReadData(const char* data, size_t len);
 	virtual void ConnectionRefused();
 	virtual void SockError(int iErrno);
 	virtual void Timeout();
@@ -60,19 +60,19 @@ public:
 	// !Getters
 private:
 protected:
-	CString					m_sRemoteNick;
-	CString					m_sRemoteIP;
-	CString					m_sFileName;
-	CString					m_sLocalFile;
-	CString					m_sSendBuf;
-	CString					m_sModuleName;
-	unsigned short			m_uRemotePort;
-	unsigned long			m_uFileSize;
-	unsigned long			m_uBytesSoFar;
-	bool					m_bSend;
-	bool					m_bNoDelFile;
-	CFile*					m_pFile;
-	CUser*					m_pUser;
+	CString         m_sRemoteNick;
+	CString         m_sRemoteIP;
+	CString         m_sFileName;
+	CString         m_sLocalFile;
+	CString         m_sSendBuf;
+	CString         m_sModuleName;
+	unsigned short  m_uRemotePort;
+	unsigned long   m_uFileSize;
+	unsigned long   m_uBytesSoFar;
+	bool            m_bSend;
+	bool            m_bNoDelFile;
+	CFile*          m_pFile;
+	CUser*          m_pUser;
 };
 
 #endif // !_DCCSOCK_H
