@@ -1,13 +1,12 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#ifdef _MODULES
-
+#include "Modules.h"
 #include "HTTPSock.h"
 #include "znc.h"
 
@@ -35,20 +34,42 @@ void CHTTPSock::Init() {
 
 CHTTPSock::~CHTTPSock() {}
 
-void CHTTPSock::ReadData(const char* data, int len) {
+void CHTTPSock::ReadData(const char* data, size_t len) {
 	if (!m_bDone && m_bGotHeader && m_bPost) {
 		m_sPostData.append(data, len);
 		CheckPost();
 	}
 }
 
+bool CHTTPSock::SendCookie(const CString& sKey, const CString& sValue) {
+	if (!sKey.empty() && !sValue.empty()) {
+		if (m_msRequestCookies.find(sKey) == m_msRequestCookies.end() ||
+			m_msRequestCookies[sKey].StrCmp(sValue) != 0)
+		{
+			m_msResponseCookies[sKey] = sValue;
+		}
+		return true;
+	}
+
+	return false;
+}
+
+const MCString& CHTTPSock::GetRequestCookies() const {
+	return m_msRequestCookies;
+}
+
+CString CHTTPSock::GetRequestCookie(const CString& sKey) const {
+	MCString::const_iterator it = m_msRequestCookies.find(sKey);
+
+	return it != m_msRequestCookies.end() ? it->second : "";
+}
+
 void CHTTPSock::CheckPost() {
 	if (m_sPostData.size() >= m_uPostLen) {
-		ParseParams(m_sPostData.Left(m_uPostLen));
+		ParseParams(m_sPostData.Left(m_uPostLen), m_msvsPOSTParams);
 		GetPage();
 		m_sPostData.clear();
 		m_bDone = true;
-		Close(Csock::CLT_AFTERWRITE);
 	}
 }
 
@@ -71,6 +92,17 @@ void CHTTPSock::ReadLine(const CString& sData) {
 		m_bPost = true;
 		m_sURI = sLine.Token(1);
 		ParseURI();
+	} else if (sName.Equals("Cookie:")) {
+		VCString vsNV;
+
+		sLine.Token(1, true).Split(";", vsNV, false, "", "", true, true);
+
+		for (unsigned int a = 0; a < vsNV.size(); a++) {
+			CString s(vsNV[a]);
+
+			m_msRequestCookies[s.Token(0, false, "=").Escape_n(CString::EURL, CString::EASCII)] =
+				s.Token(1, true, "=").Escape_n(CString::EURL, CString::EASCII);
+		}
 	} else if (sName.Equals("Authorization:")) {
 		CString sUnhashed;
 		sLine.Token(2).Base64Decode(sUnhashed);
@@ -98,19 +130,20 @@ void CHTTPSock::ReadLine(const CString& sData) {
 }
 
 void CHTTPSock::GetPage() {
-	CString sPage;
-
 	DEBUG("Page Request [" << m_sURI << "] ");
 
-	if (!OnPageRequest(m_sURI, sPage)) {
-		PrintNotFound();
-		return;
+	OnPageRequest(m_sURI);
+}
+
+void CHTTPSock::PrintPage(const CString& sPage) {
+	if (!SentHeader()) {
+		PrintHeader(sPage.length());
+	} else {
+		DEBUG("PrintPage(): Header was already sent");
 	}
 
-	if (PrintHeader(sPage.length())) {
-		Write(sPage);
-		Close(Csock::CLT_AFTERWRITE);
-	}
+	Write(sPage);
+	Close(Csock::CLT_AFTERWRITE);
 }
 
 bool CHTTPSock::PrintFile(const CString& sFileName, CString sContentType) {
@@ -166,7 +199,7 @@ bool CHTTPSock::PrintFile(const CString& sFileName, CString sContentType) {
 		sETag = "-" + CString(iMTime); // lighttpd style ETag
 
 		AddHeader("ETag", "\"" + sETag + "\"");
-		AddHeader("Cache-Control", "private");
+		AddHeader("Cache-Control", "public");
 
 		if (!m_sIfNoneMatch.empty()) {
 			m_sIfNoneMatch.Trim("\\\"'");
@@ -212,12 +245,16 @@ bool CHTTPSock::PrintFile(const CString& sFileName, CString sContentType) {
 }
 
 void CHTTPSock::ParseURI() {
-	ParseParams(m_sURI.Token(1, true, "?"));
+	ParseParams(m_sURI.Token(1, true, "?"), m_msvsGETParams);
 	m_sURI = m_sURI.Token(0, false, "?");
 }
 
-void CHTTPSock::ParseParams(const CString& sParams) {
-	m_msvsParams.clear();
+CString CHTTPSock::GetPath() const {
+	return m_sURI.Token(0, false, "?");
+}
+
+void CHTTPSock::ParseParams(const CString& sParams, map<CString, VCString> &msvsParams) {
+	msvsParams.clear();
 
 	VCString vsPairs;
 	sParams.Split("&", vsPairs, true);
@@ -227,7 +264,7 @@ void CHTTPSock::ParseParams(const CString& sParams) {
 		CString sName = sPair.Token(0, false, "=").Escape_n(CString::EURL, CString::EASCII);
 		CString sValue = sPair.Token(1, true, "=").Escape_n(CString::EURL, CString::EASCII);
 
-		m_msvsParams[sName].push_back(sValue);
+		msvsParams[sName].push_back(sValue);
 	}
 }
 
@@ -256,24 +293,39 @@ const CString& CHTTPSock::GetParamString() const {
 	return m_sPostData;
 }
 
-bool CHTTPSock::HasParam(const CString& sName) const {
-	return (m_msvsParams.find(sName) != m_msvsParams.end());
+bool CHTTPSock::HasParam(const CString& sName, bool bPost) const {
+	if (bPost)
+		return (m_msvsPOSTParams.find(sName) != m_msvsPOSTParams.end());
+	return (m_msvsGETParams.find(sName) != m_msvsGETParams.end());
 }
 
-CString CHTTPSock::GetRawParam(const CString& sName) const {
+CString CHTTPSock::GetRawParam(const CString& sName, bool bPost) const {
+	if (bPost)
+		return GetRawParam(sName, m_msvsPOSTParams);
+	return GetRawParam(sName, m_msvsGETParams);
+}
+
+CString CHTTPSock::GetRawParam(const CString& sName, const map<CString, VCString>& msvsParams) {
 	CString sRet;
 
-	map<CString, VCString>::const_iterator it = m_msvsParams.find(sName);
+	map<CString, VCString>::const_iterator it = msvsParams.find(sName);
 
-	if (it != m_msvsParams.end() && it->second.size() > 0) {
+	if (it != msvsParams.end() && it->second.size() > 0) {
 		sRet = it->second[0];
 	}
 
 	return sRet;
 }
 
-CString CHTTPSock::GetParam(const CString& sName, const CString& sFilter) const {
-	CString sRet = GetRawParam(sName);
+CString CHTTPSock::GetParam(const CString& sName, bool bPost, const CString& sFilter) const {
+	if (bPost)
+		return GetParam(sName, m_msvsPOSTParams, sFilter);
+	return GetParam(sName, m_msvsGETParams, sFilter);
+}
+
+CString CHTTPSock::GetParam(const CString& sName, const map<CString, VCString>& msvsParams, const CString& sFilter) {
+	CString sRet = GetRawParam(sName, msvsParams);
+	sRet.Trim();
 
 	for (size_t i = 0; i < sFilter.length(); i++) {
 		sRet.Replace(CString(sFilter.at(i)), "");
@@ -282,14 +334,21 @@ CString CHTTPSock::GetParam(const CString& sName, const CString& sFilter) const
 	return sRet;
 }
 
-unsigned int CHTTPSock::GetParamValues(const CString& sName, set<CString>& ssRet, const CString& sFilter) const {
+unsigned int CHTTPSock::GetParamValues(const CString& sName, set<CString>& ssRet, bool bPost, const CString& sFilter) const {
+	if (bPost)
+		return GetParamValues(sName, ssRet, m_msvsPOSTParams, sFilter);
+	return GetParamValues(sName, ssRet, m_msvsGETParams, sFilter);
+}
+
+unsigned int CHTTPSock::GetParamValues(const CString& sName, set<CString>& ssRet, const map<CString, VCString>& msvsParams, const CString& sFilter) {
 	ssRet.clear();
 
-	map<CString, VCString>::const_iterator it = m_msvsParams.find(sName);
+	map<CString, VCString>::const_iterator it = msvsParams.find(sName);
 
-	if (it != m_msvsParams.end()) {
+	if (it != msvsParams.end()) {
 		for (unsigned int a = 0; a < it->second.size(); a++) {
 			CString sParam = it->second[a];
+			sParam.Trim();
 
 			for (size_t i = 0; i < sFilter.length(); i++) {
 				sParam.Replace(CString(sFilter.at(i)), "");
@@ -301,14 +360,21 @@ unsigned int CHTTPSock::GetParamValues(const CString& sName, set<CString>& ssRet
 	return ssRet.size();
 }
 
-unsigned int CHTTPSock::GetParamValues(const CString& sName, VCString& vsRet, const CString& sFilter) const {
+unsigned int CHTTPSock::GetParamValues(const CString& sName, VCString& vsRet, bool bPost, const CString& sFilter) const {
+	if (bPost)
+		return GetParamValues(sName, vsRet, m_msvsPOSTParams, sFilter);
+	return GetParamValues(sName, vsRet, m_msvsGETParams, sFilter);
+}
+
+unsigned int CHTTPSock::GetParamValues(const CString& sName, VCString& vsRet, const map<CString, VCString>& msvsParams, const CString& sFilter) {
 	vsRet.clear();
 
-	map<CString, VCString>::const_iterator it = m_msvsParams.find(sName);
+	map<CString, VCString>::const_iterator it = msvsParams.find(sName);
 
-	if (it != m_msvsParams.end()) {
+	if (it != msvsParams.end()) {
 		for (unsigned int a = 0; a < it->second.size(); a++) {
 			CString sParam = it->second[a];
+			sParam.Trim();
 
 			for (size_t i = 0; i < sFilter.length(); i++) {
 				sParam.Replace(CString(sFilter.at(i)), "");
@@ -320,8 +386,14 @@ unsigned int CHTTPSock::GetParamValues(const CString& sName, VCString& vsRet, co
 	return vsRet.size();
 }
 
-const map<CString, VCString>& CHTTPSock::GetParams() const {
-	return m_msvsParams;
+const map<CString, VCString>& CHTTPSock::GetParams(bool bPost) const {
+	if (bPost)
+		return m_msvsPOSTParams;
+	return m_msvsGETParams;
+}
+
+bool CHTTPSock::IsPost() const {
+	return m_bPost;
 }
 
 bool CHTTPSock::PrintNotFound() {
@@ -330,6 +402,7 @@ bool CHTTPSock::PrintNotFound() {
 
 bool CHTTPSock::PrintErrorPage(unsigned int uStatusId, const CString& sStatusMsg, const CString& sMessage) {
 	if (SentHeader()) {
+		DEBUG("PrintErrorPage(): Header was already sent");
 		return false;
 	}
 
@@ -346,7 +419,7 @@ CString CHTTPSock::GetErrorPage(unsigned int uStatusId, const CString& sStatusMs
 		"<html><head>\r\n<title>" + CString(uStatusId) + " " + sStatusMsg.Escape_n(CString::EHTML) + "</title>\r\n"
 		"</head><body>\r\n<h1>" + sStatusMsg.Escape_n(CString::EHTML) + "</h1>\r\n"
 		"<p>" + sMessage.Escape_n(CString::EHTML) + "</p>\r\n"
-		"<hr>\r\n<address>" + CZNC::GetTag().Escape_n(CString::EHTML) + " at " + GetLocalIP().Escape_n(CString::EHTML) + " Port " + CString(GetLocalPort()) + "</address>\r\n"
+		"<hr />\r\n<address>" + CZNC::GetTag(false).Escape_n(CString::EHTML) + " at " + GetLocalIP().Escape_n(CString::EHTML) + " Port " + CString(GetLocalPort()) + "</address>\r\n"
 		"</body></html>\r\n";
 }
 
@@ -356,6 +429,7 @@ bool CHTTPSock::ForceLogin() {
 	}
 
 	if (SentHeader()) {
+		DEBUG("ForceLogin(): Header was already sent!");
 		return false;
 	}
 
@@ -378,7 +452,7 @@ bool CHTTPSock::SentHeader() const {
 
 bool CHTTPSock::PrintHeader(off_t uContentLength, const CString& sContentType, unsigned int uStatusId, const CString& sStatusMsg) {
 	if (SentHeader()) {
-		DEBUG("- Header already sent!");
+		DEBUG("PrintHeader(): Header was already sent!");
 		return false;
 	}
 
@@ -394,13 +468,19 @@ bool CHTTPSock::PrintHeader(off_t uContentLength, const CString& sContentType, u
 
 	Write("HTTP/" + CString(m_bHTTP10Client ? "1.0 " : "1.1 ") + CString(uStatusId) + " " + sStatusMsg + "\r\n");
 	//Write("Date: Tue, 28 Jun 2005 20:45:36 GMT\r\n");
-	Write("Server: " + CZNC::GetTag() + "\r\n");
+	Write("Server: " + CZNC::GetTag(false) + "\r\n");
 	if (uContentLength > 0) {
 		Write("Content-Length: " + CString(uContentLength) + "\r\n");
 	}
 	Write("Content-Type: " + m_sContentType + "\r\n");
 
-	for (MCString::iterator it = m_msHeaders.begin(); it != m_msHeaders.end(); it++) {
+	MCString::iterator it;
+
+	for (it = m_msResponseCookies.begin(); it != m_msResponseCookies.end(); ++it) {
+		Write("Set-Cookie: " + it->first.Escape_n(CString::EURL) + "=" + it->second.Escape_n(CString::EURL) + "; path=/;\r\n");
+	}
+
+	for (it = m_msHeaders.begin(); it != m_msHeaders.end(); ++it) {
 		Write(it->first + ": " + it->second + "\r\n");
 	}
 
@@ -422,6 +502,7 @@ void CHTTPSock::AddHeader(const CString& sName, const CString& sValue) {
 
 bool CHTTPSock::Redirect(const CString& sURL) {
 	if (SentHeader()) {
+		DEBUG("Redirect() - Header was already sent");
 		return false;
 	}
 
@@ -452,5 +533,3 @@ void CHTTPSock::ReachedMaxBuffer() {
 	DEBUG(GetSockName() << " == ReachedMaxBuffer()");
 	Close();
 }
-
-#endif // _MODULES
