@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -49,7 +49,7 @@ CIRCSock::~CIRCSock() {
 
 	m_pUser->IRCDisconnected();
 
-	for (map<CString, CChan*>::iterator a = m_msChans.begin(); a != m_msChans.end(); a++) {
+	for (map<CString, CChan*>::iterator a = m_msChans.begin(); a != m_msChans.end(); ++a) {
 		delete a->second;
 	}
 
@@ -63,8 +63,9 @@ CIRCSock::~CIRCSock() {
 	GetUser()->AddBytesWritten(GetBytesWritten());
 }
 
-void CIRCSock::Quit() {
-	PutIRC("QUIT :" + m_pUser->GetQuitMsg());
+void CIRCSock::Quit(const CString& sQuitMsg) {
+	CString sMsg = (!sQuitMsg.empty()) ? sQuitMsg : m_pUser->GetQuitMsg();
+	PutIRC("QUIT :" + sMsg);
 	Close(CLT_AFTERWRITE);
 }
 
@@ -105,9 +106,16 @@ void CIRCSock::ReadLine(const CString& sData) {
 		CString sRest = sLine.Token(3, true);
 
 		switch (uRaw) {
-			case 1:	{// :irc.server.com 001 nick :Welcome to the Internet Relay Network nick
+			case 1: { // :irc.server.com 001 nick :Welcome to the Internet Relay Network nick
+				if (m_bAuthed && sServer == "irc.znc.in") {
+					// m_bAuthed == true => we already received another 001 => we might be in a traffic loop
+					m_pUser->PutStatus("ZNC seems to be connected to itself, disconnecting...");
+					Quit();
+					return;
+				}
+
 				m_pUser->SetIRCServer(sServer);
-				SetTimeout(240, TMO_READ);	// Now that we are connected, let nature take its course
+				SetTimeout(240, TMO_READ);  // Now that we are connected, let nature take its course
 				PutIRC("WHO " + sNick);
 
 				m_bAuthed = true;
@@ -144,13 +152,13 @@ void CIRCSock::ReadLine(const CString& sData) {
 			case 2:
 			case 3:
 			case 4:
-			case 250:	// highest connection count
-			case 251:	// user count
-			case 252:	// oper count
-			case 254:	// channel count
-			case 255:	// client count
-			case 265:	// local users
-			case 266:	// global users
+			case 250:  // highest connection count
+			case 251:  // user count
+			case 252:  // oper count
+			case 254:  // channel count
+			case 255:  // client count
+			case 265:  // local users
+			case 266:  // global users
 				m_pUser->UpdateRawBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 				break;
 			case 305:
@@ -159,7 +167,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 			case 306:
 				m_pUser->SetIRCAway(true);
 				break;
-			case 324: {	// MODE
+			case 324: {  // MODE
 				sRest.Trim();
 				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
 
@@ -239,7 +247,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 				break;
 			}
-			case 353: {	// NAMES
+			case 353: {  // NAMES
 				sRest.Trim();
 				// Todo: allow for non @+= server msgs
 				CChan* pChan = m_pUser->FindChan(sRest.Token(1));
@@ -259,8 +267,8 @@ void CIRCSock::ReadLine(const CString& sData) {
 				// We forwarded it already, so return
 				return;
 			}
-			case 366: {	// end of names list
-				m_pUser->PutUser(sLine);	// First send them the raw
+			case 366: {  // end of names list
+				m_pUser->PutUser(sLine);  // First send them the raw
 
 				// :irc.server.com 366 nick #chan :End of /NAMES list.
 				CChan* pChan = m_pUser->FindChan(sRest.Token(0));
@@ -282,13 +290,13 @@ void CIRCSock::ReadLine(const CString& sData) {
 					}
 				}
 
-				return;	// return so we don't send them the raw twice
+				return;  // return so we don't send them the raw twice
 			}
-			case 375: 	// begin motd
-			case 422:	// MOTD File is missing
+			case 375:  // begin motd
+			case 422:  // MOTD File is missing
 				m_pUser->ClearMotdBuffer();
-			case 372:	// motd
-			case 376:	// end motd
+			case 372:  // motd
+			case 376:  // end motd
 				m_pUser->AddMotdBuffer(":" + sServer + " " + sCmd + " ", " " + sRest);
 				break;
 			case 437:
@@ -377,6 +385,10 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 			if (Nick.GetNick().Equals(GetNick())) {
 				m_pUser->PutStatus("You quit [" + sMessage + "]");
+				// We don't call module hooks and we don't
+				// forward this quit to clients (Some clients
+				// disconnect if they receive such a QUIT)
+				return;
 			}
 
 			vector<CChan*> vFoundChans;
@@ -475,7 +487,6 @@ void CIRCSock::ReadLine(const CString& sData) {
 				}
 			} else if (sTarget == m_Nick.GetNick()) {
 				CString sModeArg = sModes.Token(0);
-//					CString sArgs = sModes.Token(1, true); Usermode changes got no params
 				bool bAdd = true;
 /* no module call defined (yet?)
 				MODULECALL(OnRawUserMode(*pOpNick, *this, sModeArg, sArgs), m_pUser, NULL, );
@@ -507,7 +518,7 @@ void CIRCSock::ReadLine(const CString& sData) {
 
 			if (pChan) {
 				pChan->RemNick(sKickedNick);
-				MODULECALL(OnKick(Nick.GetNickMask(), sKickedNick, *pChan, sMsg), m_pUser, NULL, );
+				MODULECALL(OnKick(Nick, sKickedNick, *pChan, sMsg), m_pUser, NULL, );
 			}
 
 			if (GetNick().Equals(sKickedNick) && pChan) {
@@ -627,6 +638,39 @@ void CIRCSock::ReadLine(const CString& sData) {
 			if (!m_pUser->IsUserAttached()) {
 				m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " WALLOPS ", ":" + m_pUser->AddTimestamp(sMsg), false);
 			}
+		} else if (sCmd.Equals("CAP")) {
+			// sRest.Token(0) is most likely "*". No idea why, the
+			// CAP spec don't mention this, but all implementations
+			// I've seen add this extra asterisk
+			CString sSubCmd = sRest.Token(1);
+			CString sArgs = sRest.Token(2, true).TrimPrefix_n(":");
+
+			if (sSubCmd == "LS" && !m_bAuthed) {
+				VCString vsTokens;
+				VCString::iterator it;
+				sArgs.Split(" ", vsTokens, false);
+
+				for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
+					if (*it == "multi-prefix" || *it == "userhost-in-names") {
+						PutIRC("CAP REQ " + *it);
+					}
+				}
+
+				// Tell the IRC server we are done with CAP
+				PutIRC("CAP END");
+			} else if (sSubCmd == "ACK" && !m_bAuthed) {
+				VCString vsTokens;
+				VCString::iterator it;
+				sArgs.Split(" ", vsTokens, false);
+
+				for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
+					if (*it == "multi-prefix") {
+						m_bNamesx = true;
+					} else if (*it == "userhost-in-names") {
+						m_bUHNames = true;
+					}
+				}
+			}
 		}
 	}
 
@@ -640,11 +684,18 @@ bool CIRCSock::OnCTCPReply(CNick& Nick, CString& sMessage) {
 }
 
 bool CIRCSock::OnPrivCTCP(CNick& Nick, CString& sMessage) {
+	MODULECALL(OnPrivCTCP(Nick, sMessage), m_pUser, NULL, return true);
+
 	if (sMessage.TrimPrefix("ACTION ")) {
 		MODULECALL(OnPrivAction(Nick, sMessage), m_pUser, NULL, return true);
+
+		if (!m_pUser->IsUserAttached()) {
+			// If the user is detached, add to the buffer
+			m_pUser->AddQueryBuffer(":" + Nick.GetNickMask() + " PRIVMSG ", " :\001ACTION " + m_pUser->AddTimestamp(sMessage) + "\001");
+		}
+
 		sMessage = "ACTION " + sMessage;
 	}
-	MODULECALL(OnPrivCTCP(Nick, sMessage), m_pUser, NULL, return true);
 
 	if (sMessage.Equals("DCC ", false, 4) && m_pUser && m_pUser->BounceDCCs() && m_pUser->IsUserAttached()) {
 		// DCC CHAT chat 2453612361 44592
@@ -754,15 +805,16 @@ bool CIRCSock::OnPrivMsg(CNick& Nick, CString& sMessage) {
 bool CIRCSock::OnChanCTCP(CNick& Nick, const CString& sChan, CString& sMessage) {
 	CChan* pChan = m_pUser->FindChan(sChan);
 	if (pChan) {
+		MODULECALL(OnChanCTCP(Nick, *pChan, sMessage), m_pUser, NULL, return true);
+
 		// Record a /me
 		if (sMessage.TrimPrefix("ACTION ")) {
+			MODULECALL(OnChanAction(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 			if (pChan->KeepBuffer() || !m_pUser->IsUserAttached() || pChan->IsDetached()) {
 				pChan->AddBuffer(":" + Nick.GetNickMask() + " PRIVMSG " + sChan + " :\001ACTION " + m_pUser->AddTimestamp(sMessage) + "\001");
 			}
-			MODULECALL(OnChanAction(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 			sMessage = "ACTION " + sMessage;
 		}
-		MODULECALL(OnChanCTCP(Nick, *pChan, sMessage), m_pUser, NULL, return true);
 	}
 
 	if (OnGeneralCTCP(Nick, sMessage))
@@ -818,6 +870,8 @@ void CIRCSock::Connected() {
 
 	MODULECALL(OnIRCRegistration(sPass, sNick, sIdent, sRealName), m_pUser, NULL, return);
 
+	PutIRC("CAP LS");
+
 	if (!sPass.empty()) {
 		PutIRC("PASS " + sPass);
 	}
@@ -846,7 +900,7 @@ void CIRCSock::Disconnected() {
 	// otherwise, on reconnect, it might think it still
 	// had user modes that it actually doesn't have.
 	CString sUserMode;
-	for (set<unsigned char>::const_iterator it = m_scUserModes.begin(); it != m_scUserModes.end(); it++) {
+	for (set<unsigned char>::const_iterator it = m_scUserModes.begin(); it != m_scUserModes.end(); ++it) {
 		sUserMode += *it;
 	}
 	if (!sUserMode.empty()) {
@@ -919,12 +973,14 @@ void CIRCSock::ReachedMaxBuffer() {
 }
 
 void CIRCSock::ParseISupport(const CString& sLine) {
-	unsigned int i = 0;
-	CString sArg = sLine.Token(i++);
+	VCString vsTokens;
+	VCString::iterator it;
 
-	while (!sArg.empty()) {
-		CString sName = sArg.Token(0, false, "=");
-		CString sValue = sArg.Token(1, true, "=");
+	sLine.Split(" ", vsTokens, false);
+
+	for (it = vsTokens.begin(); it != vsTokens.end(); ++it) {
+		CString sName = it->Token(0, false, "=");
+		CString sValue = it->Token(1, true, "=");
 
 		if (sName.Equals("PREFIX")) {
 			CString sPrefixes = sValue.Token(1, false, ")");
@@ -956,59 +1012,69 @@ void CIRCSock::ParseISupport(const CString& sLine) {
 				}
 			}
 		} else if (sName.Equals("NAMESX")) {
+			if (m_bNamesx)
+				continue;
 			m_bNamesx = true;
 			PutIRC("PROTOCTL NAMESX");
 		} else if (sName.Equals("UHNAMES")) {
+			if (m_bUHNames)
+				continue;
 			m_bUHNames = true;
 			PutIRC("PROTOCTL UHNAMES");
 		}
-
-		sArg = sLine.Token(i++);
 	}
 }
 
 void CIRCSock::ForwardRaw353(const CString& sLine) const {
 	vector<CClient*>& vClients = m_pUser->GetClients();
+	vector<CClient*>::iterator it;
+
+	for (it = vClients.begin(); it != vClients.end(); ++it) {
+		ForwardRaw353(sLine, *it);
+	}
+}
+
+void CIRCSock::ForwardRaw353(const CString& sLine, CClient* pClient) const {
 	CString sNicks = sLine.Token(5, true);
 	if (sNicks.Left(1) == ":")
 		sNicks.LeftChomp();
 
-	for (unsigned int a = 0; a < vClients.size(); a++) {
-		if ((!m_bNamesx || vClients[a]->HasNamesx()) && (!m_bUHNames || vClients[a]->HasUHNames())) {
-			// Client and server have both the same UHNames and Namesx stuff enabled
-			m_pUser->PutUser(sLine, vClients[a]);
-		} else {
-			// Get everything except the actual user list
-			CString sTmp = sLine.Token(0, false, " :") + " :";
+	if ((!m_bNamesx || pClient->HasNamesx()) && (!m_bUHNames || pClient->HasUHNames())) {
+		// Client and server have both the same UHNames and Namesx stuff enabled
+		m_pUser->PutUser(sLine, pClient);
+	} else {
+		// Get everything except the actual user list
+		CString sTmp = sLine.Token(0, false, " :") + " :";
 
-			unsigned int i = 0;
-			// This loop runs once for every nick on the channel
-			for (;;) {
-				CString sNick = sNicks.Token(i).Trim_n(" ");
-				if (sNick.empty())
-					break;
+		VCString vsNicks;
+		VCString::const_iterator it;
 
-				if (m_bNamesx && !vClients[a]->HasNamesx() && IsPermChar(sNick[0])) {
-					// Server has, client doesn't have NAMESX, so we just use the first perm char
-					size_t pos = sNick.find_first_not_of(GetPerms());
-					if (pos >= 2 && pos != CString::npos) {
-						sNick = sNick[0] + sNick.substr(pos);
-					}
+		// This loop runs once for every nick on the channel
+		sNicks.Split(" ", vsNicks, false);
+		for (it = vsNicks.begin(); it != vsNicks.end(); ++it) {
+			CString sNick = *it;
+			if (sNick.empty())
+				break;
+
+			if (m_bNamesx && !pClient->HasNamesx() && IsPermChar(sNick[0])) {
+				// Server has, client doesn't have NAMESX, so we just use the first perm char
+				size_t pos = sNick.find_first_not_of(GetPerms());
+				if (pos >= 2 && pos != CString::npos) {
+					sNick = sNick[0] + sNick.substr(pos);
 				}
-
-				if (m_bUHNames && !vClients[a]->HasUHNames()) {
-					// Server has, client hasnt UHNAMES,
-					// so we strip away ident and host.
-					sNick = sNick.Token(0, false, "!");
-				}
-
-				sTmp += sNick + " ";
-				i++;
 			}
-			// Strip away the spaces we inserted at the end
-			sTmp.TrimRight(" ");
-			m_pUser->PutUser(sTmp, vClients[a]);
+
+			if (m_bUHNames && !pClient->HasUHNames()) {
+				// Server has, client hasnt UHNAMES,
+				// so we strip away ident and host.
+				sNick = sNick.Token(0, false, "!");
+			}
+
+			sTmp += sNick + " ";
 		}
+		// Strip away the spaces we inserted at the end
+		sTmp.TrimRight(" ");
+		m_pUser->PutUser(sTmp, pClient);
 	}
 }
 
@@ -1086,7 +1152,7 @@ CIRCSock::EChanModeArgs CIRCSock::GetModeType(unsigned char uMode) const {
 }
 
 void CIRCSock::ResetChans() {
-	for (map<CString, CChan*>::iterator a = m_msChans.begin(); a != m_msChans.end(); a++) {
+	for (map<CString, CChan*>::iterator a = m_msChans.begin(); a != m_msChans.end(); ++a) {
 		a->second->Reset();
 	}
 }
