@@ -1,79 +1,79 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#ifdef _MODULES
-
 #include "Modules.h"
 #include "User.h"
 #include "znc.h"
 #include <dlfcn.h>
+#include "WebModules.h"
+#include "Template.h"
 
 #ifndef RTLD_LOCAL
 # define RTLD_LOCAL 0
 # warning "your crap box doesnt define RTLD_LOCAL !?"
 #endif
 
-#define _MODUNLOADCHK(func, type)								\
-	for (unsigned int a = 0; a < size(); a++) {			\
-		try {											\
-			type* pMod = (type *) (*this)[a];					\
-			CClient* pOldClient = pMod->GetClient();			\
-			pMod->SetClient(m_pClient);					\
-			if (m_pUser) {							\
-				CUser* pOldUser = pMod->GetUser();			\
-				pMod->SetUser(m_pUser);					\
-				pMod->func;						\
-				pMod->SetUser(pOldUser);				\
-			} else {							\
-				pMod->func;						\
-			}								\
-			pMod->SetClient(pOldClient);					\
-		} catch (CModule::EModException e) {			\
-			if (e == CModule::UNLOAD) {					\
-				UnloadModule((*this)[a]->GetModName());	\
-			}											\
-		}												\
+#define _MODUNLOADCHK(func, type)                                        \
+	for (unsigned int a = 0; a < size(); a++) {                      \
+		try {                                                    \
+			type* pMod = (type *) (*this)[a];                \
+			CClient* pOldClient = pMod->GetClient();         \
+			pMod->SetClient(m_pClient);                      \
+			if (m_pUser) {                                   \
+				CUser* pOldUser = pMod->GetUser();       \
+				pMod->SetUser(m_pUser);                  \
+				pMod->func;                              \
+				pMod->SetUser(pOldUser);                 \
+			} else {                                         \
+				pMod->func;                              \
+			}                                                \
+			pMod->SetClient(pOldClient);                     \
+		} catch (CModule::EModException e) {                     \
+			if (e == CModule::UNLOAD) {                      \
+				UnloadModule((*this)[a]->GetModName());  \
+			}                                                \
+		}                                                        \
 	}
 
 #define MODUNLOADCHK(func)	_MODUNLOADCHK(func, CModule)
 #define GLOBALMODCALL(func)	_MODUNLOADCHK(func, CGlobalModule)
 
-#define _MODHALTCHK(func, type)							\
-	bool bHaltCore = false;								\
-	for (unsigned int a = 0; a < size(); a++) {			\
-		try {											\
-			type* pMod = (type*) (*this)[a];			\
-			CModule::EModRet e = CModule::CONTINUE;		\
-			CClient* pOldClient = pMod->GetClient();			\
-			pMod->SetClient(m_pClient);				\
-			if (m_pUser) {								\
-				CUser* pOldUser = pMod->GetUser();			\
-				pMod->SetUser(m_pUser);					\
-				e = pMod->func;							\
-				pMod->SetUser(pOldUser);				\
-			} else {									\
-				e = pMod->func;							\
-			}											\
-			pMod->SetClient(pOldClient);				\
-			if (e == CModule::HALTMODS) {				\
-				break;									\
-			} else if (e == CModule::HALTCORE) {		\
-				bHaltCore = true;						\
-			} else if (e == CModule::HALT) {			\
-				bHaltCore = true;						\
-				break;									\
-			}											\
-		} catch (CModule::EModException e) {			\
-			if (e == CModule::UNLOAD) {					\
-				UnloadModule((*this)[a]->GetModName());	\
-			}											\
-		}												\
-	}													\
+#define _MODHALTCHK(func, type)                                          \
+	bool bHaltCore = false;                                          \
+	for (unsigned int a = 0; a < size(); a++) {                      \
+		try {                                                    \
+			type* pMod = (type*) (*this)[a];                 \
+			CModule::EModRet e = CModule::CONTINUE;          \
+			CClient* pOldClient = pMod->GetClient();         \
+			pMod->SetClient(m_pClient);                      \
+			if (m_pUser) {                                   \
+				CUser* pOldUser = pMod->GetUser();       \
+				pMod->SetUser(m_pUser);                  \
+				e = pMod->func;                          \
+				pMod->SetUser(pOldUser);                 \
+			} else {                                         \
+				e = pMod->func;                          \
+			}                                                \
+			pMod->SetClient(pOldClient);                     \
+			if (e == CModule::HALTMODS) {                    \
+				break;                                   \
+			} else if (e == CModule::HALTCORE) {             \
+				bHaltCore = true;                        \
+			} else if (e == CModule::HALT) {                 \
+				bHaltCore = true;                        \
+				break;                                   \
+			}                                                \
+		} catch (CModule::EModException e) {                     \
+			if (e == CModule::UNLOAD) {                      \
+				UnloadModule((*this)[a]->GetModName());  \
+			}                                                \
+		}                                                        \
+	}                                                                \
 	return bHaltCore;
 
 #define MODHALTCHK(func)	_MODHALTCHK(func, CModule)
@@ -102,108 +102,6 @@ CModule* CTimer::GetModule() const { return m_pModule; }
 const CString& CTimer::GetDescription() const { return m_sDescription; }
 /////////////////// !Timer ///////////////////
 
-/////////////////// Socket ///////////////////
-CSocket::CSocket(CModule* pModule) : CZNCSock() {
-	m_pModule = pModule;
-	m_pModule->AddSocket(this);
-	EnableReadLine();
-	SetMaxBufferThreshold(10240);
-}
-
-CSocket::CSocket(CModule* pModule, const CString& sHostname, unsigned short uPort, int iTimeout) : CZNCSock(sHostname, uPort, iTimeout) {
-	m_pModule = pModule;
-	m_pModule->AddSocket(this);
-	EnableReadLine();
-	SetMaxBufferThreshold(10240);
-}
-
-CSocket::~CSocket() {
-	CUser *pUser = m_pModule->GetUser();
-
-	m_pModule->UnlinkSocket(this);
-
-	if (!m_pModule->IsGlobal() && pUser) {
-		pUser->AddBytesWritten(GetBytesWritten());
-		pUser->AddBytesRead(GetBytesRead());
-	} else {
-		CZNC::Get().AddBytesWritten(GetBytesWritten());
-		CZNC::Get().AddBytesRead(GetBytesRead());
-	}
-}
-
-void CSocket::ReachedMaxBuffer() {
-	DEBUG(GetSockName() << " == ReachedMaxBuffer()");
-	PutModule("Some socket reached its max buffer limit and was closed!");
-	Close();
-}
-
-void CSocket::SockError(int iErrno) {
-	DEBUG(GetSockName() << " == SockError(" << strerror(iErrno) << ")");
-	if (iErrno == EMFILE) {
-		// We have too many open fds, this can cause a busy loop.
-		Close();
-	}
-}
-
-bool CSocket::ConnectionFrom(const CString& sHost, unsigned short uPort) {
-	return CZNC::Get().AllowConnectionFrom(sHost);
-}
-
-bool CSocket::Connect(const CString& sHostname, unsigned short uPort, bool bSSL, unsigned int uTimeout) {
-	CUser* pUser = m_pModule->GetUser();
-	CString sSockName = "MOD::C::" + m_pModule->GetModName();
-	CString sVHost;
-
-	if (pUser) {
-		sSockName += "::" + pUser->GetUserName();
-		sVHost = m_pModule->GetUser()->GetVHost();
-	}
-
-	// Don't overwrite the socket name if one is already set
-	if (!GetSockName().empty()) {
-		sSockName = GetSockName();
-	}
-
-	return m_pModule->GetManager()->Connect(sHostname, uPort, sSockName, uTimeout, bSSL, sVHost, this);
-}
-
-bool CSocket::Listen(unsigned short uPort, bool bSSL, unsigned int uTimeout) {
-	CUser* pUser = m_pModule->GetUser();
-	CString sSockName = "MOD::L::" + m_pModule->GetModName();
-
-	if (pUser) {
-		sSockName += "::" + pUser->GetUserName();
-	}
-	// Don't overwrite the socket name if one is already set
-	if (!GetSockName().empty()) {
-		sSockName = GetSockName();
-	}
-
-	return m_pModule->GetManager()->ListenAll(uPort, sSockName, bSSL, SOMAXCONN, this);
-}
-
-bool CSocket::PutIRC(const CString& sLine) {
-	return (m_pModule) ? m_pModule->PutIRC(sLine) : false;
-}
-
-bool CSocket::PutUser(const CString& sLine) {
-	return (m_pModule) ? m_pModule->PutUser(sLine) : false;
-}
-
-bool CSocket::PutStatus(const CString& sLine) {
-	return (m_pModule) ? m_pModule->PutStatus(sLine) : false;
-}
-
-bool CSocket::PutModule(const CString& sLine, const CString& sIdent, const CString& sHost) {
-	return (m_pModule) ? m_pModule->PutModule(sLine, sIdent, sHost) : false;
-}
-bool CSocket::PutModNotice(const CString& sLine, const CString& sIdent, const CString& sHost) {
-	return (m_pModule) ? m_pModule->PutModNotice(sLine, sIdent, sHost) : false;
-}
-
-void CSocket::SetModule(CModule* p) { m_pModule = p; }
-CModule* CSocket::GetModule() const { return m_pModule; }
-/////////////////// !Socket ///////////////////
 
 CModule::CModule(ModHandle pDLL, CUser* pUser, const CString& sModName, const CString& sDataDir) {
 	m_bFake = false;
@@ -489,6 +387,11 @@ void CModule::ListSockets() {
 
 CString CModule::GetModNick() const { return ((m_pUser) ? m_pUser->GetStatusPrefix() : "*") + m_sModName; }
 
+// Webmods
+bool CModule::OnWebPreRequest(CWebSock& WebSock, const CString& sPageName) { return false; }
+bool CModule::OnWebRequest(CWebSock& WebSock, const CString& sPageName, CTemplate& Tmpl) { return false; }
+// !Webmods
+
 bool CModule::OnLoad(const CString& sArgs, CString& sMessage) { sMessage = ""; return true; }
 bool CModule::OnBoot() { return true; }
 void CModule::OnPreRehash() {}
@@ -498,6 +401,9 @@ void CModule::OnIRCConnected() {}
 CModule::EModRet CModule::OnIRCConnecting(CIRCSock *IRCSock) { return CONTINUE; }
 CModule::EModRet CModule::OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) { return CONTINUE; }
 CModule::EModRet CModule::OnBroadcast(CString& sMessage) { return CONTINUE; }
+CModule::EModRet CModule::OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan) { return CONTINUE; }
+void CModule::OnWriteUserConfig(CFile& Config) {}
+void CModule::OnWriteChanConfig(CFile& Config, CChan& Chan) {}
 
 CModule::EModRet CModule::OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize) { return CONTINUE; }
 
@@ -592,9 +498,10 @@ bool CModule::PutModNotice(const CString& sLine, const CString& sIdent, const CS
 ///////////////////
 // CGlobalModule //
 ///////////////////
-CModule::EModRet CGlobalModule::OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan) { return CONTINUE; }
+CModule::EModRet CGlobalModule::OnWriteConfig(CFile& Config) { return CONTINUE; }
+CModule::EModRet CGlobalModule::OnAddUser(CUser& User, CString& sErrorRet) { return CONTINUE; }
 CModule::EModRet CGlobalModule::OnDeleteUser(CUser& User) { return CONTINUE; }
-void CGlobalModule::OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort) {}
+void CGlobalModule::OnClientConnect(CZNCSock* pClient, const CString& sHost, unsigned short uPort) {}
 CModule::EModRet CGlobalModule::OnLoginAttempt(CSmartPtr<CAuthBase> Auth) { return CONTINUE; }
 void CGlobalModule::OnFailedLogin(const CString& sUsername, const CString& sRemoteIP) {}
 CModule::EModRet CGlobalModule::OnUnknownUserRaw(CClient* pClient, CString& sLine) { return CONTINUE; }
@@ -639,6 +546,9 @@ bool CModules::OnIRCConnected() { MODUNLOADCHK(OnIRCConnected()); return false;
 bool CModules::OnIRCConnecting(CIRCSock *pIRCSock) { MODHALTCHK(OnIRCConnecting(pIRCSock)); }
 bool CModules::OnIRCRegistration(CString& sPass, CString& sNick, CString& sIdent, CString& sRealName) { MODHALTCHK(OnIRCRegistration(sPass, sNick, sIdent, sRealName)); }
 bool CModules::OnBroadcast(CString& sMessage) { MODHALTCHK(OnBroadcast(sMessage)); }
+bool CModules::OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan) { MODHALTCHK(OnConfigLine(sName, sValue, pUser, pChan)); }
+bool CModules::OnWriteUserConfig(CFile& Config) { MODUNLOADCHK(OnWriteUserConfig(Config)); return false; }
+bool CModules::OnWriteChanConfig(CFile& Config, CChan& Chan) { MODUNLOADCHK(OnWriteChanConfig(Config, Chan)); return false; }
 bool CModules::OnIRCDisconnected() { MODUNLOADCHK(OnIRCDisconnected()); return false; }
 bool CModules::OnDCCUserSend(const CNick& RemoteNick, unsigned long uLongIP, unsigned short uPort, const CString& sFile, unsigned long uFileSize) { MODHALTCHK(OnDCCUserSend(RemoteNick, uLongIP, uPort, sFile, uFileSize)); }
 bool CModules::OnChanPermission(const CNick& OpNick, const CNick& Nick, CChan& Channel, unsigned char uMode, bool bAdded, bool bNoChange) { MODUNLOADCHK(OnChanPermission(OpNick, Nick, Channel, uMode, bAdded, bNoChange)); return false; }
@@ -691,15 +601,19 @@ bool CModules::OnModCTCP(const CString& sMessage) { MODUNLOADCHK(OnModCTCP(sMess
 ////////////////////
 // CGlobalModules //
 ////////////////////
-bool CGlobalModules::OnConfigLine(const CString& sName, const CString& sValue, CUser* pUser, CChan* pChan) {
-	GLOBALMODHALTCHK(OnConfigLine(sName, sValue, pUser, pChan));
+bool CGlobalModules::OnWriteConfig(CFile& Config) {
+	GLOBALMODHALTCHK(OnWriteConfig(Config));
+}
+
+bool CGlobalModules::OnAddUser(CUser& User, CString& sErrorRet) {
+	GLOBALMODHALTCHK(OnAddUser(User, sErrorRet));
 }
 
 bool CGlobalModules::OnDeleteUser(CUser& User) {
 	GLOBALMODHALTCHK(OnDeleteUser(User));
 }
 
-void CGlobalModules::OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort) {
+void CGlobalModules::OnClientConnect(CZNCSock* pClient, const CString& sHost, unsigned short uPort) {
 	GLOBALMODCALL(OnClientConnect(pClient, sHost, uPort));
 }
 
@@ -803,6 +717,7 @@ bool CModules::LoadModule(const CString& sModule, const CString& sArgs, CUser* p
 	pModule->SetDescription(sDesc);
 	pModule->SetGlobal(bIsGlobal);
 	pModule->SetArgs(sArgs);
+	pModule->SetModPath(CDir::ChangeDir(CZNC::Get().GetCurPath(), sModPath));
 	push_back(pModule);
 
 	bool bLoaded;
@@ -836,7 +751,7 @@ bool CModules::UnloadModule(const CString& sModule) {
 }
 
 bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
-	CString sMod = sModule;	// Make a copy incase the reference passed in is from CModule::GetModName()
+	CString sMod = sModule;  // Make a copy incase the reference passed in is from CModule::GetModName()
 	CModule* pModule = FindModule(sMod);
 	sRetMsg = "";
 
@@ -846,7 +761,7 @@ bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
 	}
 
 	if (pModule->IsFake()) {
-		for (iterator it = begin(); it != end(); it++) {
+		for (iterator it = begin(); it != end(); ++it) {
 			if (*it == pModule) {
 				erase(it);
 				sRetMsg = "Fake module [" + sMod + "] unloaded";
@@ -867,7 +782,7 @@ bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
 		if (Unload) {
 			Unload(pModule);
 
-			for (iterator it = begin(); it != end(); it++) {
+			for (iterator it = begin(); it != end(); ++it) {
 				if (*it == pModule) {
 					erase(it);
 					break;
@@ -889,7 +804,7 @@ bool CModules::UnloadModule(const CString& sModule, CString& sRetMsg) {
 }
 
 bool CModules::ReloadModule(const CString& sModule, const CString& sArgs, CUser* pUser, CString& sRetMsg) {
-	CString sMod = sModule;	// Make a copy incase the reference passed in is from CModule::GetModName()
+	CString sMod = sModule;  // Make a copy incase the reference passed in is from CModule::GetModName()
 	sRetMsg = "";
 	if (!UnloadModule(sMod, sRetMsg)) {
 		return false;
@@ -992,14 +907,17 @@ bool CModules::FindModPath(const CString& sModule, CString& sModPath,
 
 CModules::ModDirList CModules::GetModDirs() {
 	ModDirList ret;
+	CString sDir;
 
+#ifdef RUN_FROM_SOURCE
 	// ./modules
-	CString sDir = CZNC::Get().GetCurPath() + "/modules/";
+	sDir = CZNC::Get().GetCurPath() + "/modules/";
 	ret.push(std::make_pair(sDir, sDir));
 
 	// ./modules/extra
 	sDir = CZNC::Get().GetCurPath() + "/modules/extra/";
 	ret.push(std::make_pair(sDir, sDir));
+#endif
 
 	// ~/.znc/modules
 	sDir = CZNC::Get().GetModPath() + "/";
@@ -1026,7 +944,19 @@ ModHandle CModules::OpenModule(const CString& sModule, const CString& sModPath,
 		}
 	}
 
-	ModHandle p = dlopen((sModPath).c_str(), RTLD_NOW | RTLD_LOCAL);
+	// The second argument to dlopen() has a long history. It seems clear
+	// that (despite what the man page says) we must include either of
+	// RTLD_NOW and RTLD_LAZY and either of RTLD_GLOBAL and RTLD_LOCAL.
+	//
+	// RTLD_NOW vs. RTLD_LAZY: We use RTLD_NOW to avoid znc dying due to
+	// failed symbol lookups later on. Doesn't really seem to have much of a
+	// performance impact.
+	//
+	// RTLD_GLOBAL vs. RTLD_LOCAL: If perl is loaded with RTLD_LOCAL and later on
+	// loads own modules (which it apparently does with RTLD_LAZY), we will die in a
+	// name lookup since one of perl's symbols isn't found. That's worse
+	// than any theoretical issue with RTLD_GLOBAL.
+	ModHandle p = dlopen((sModPath).c_str(), RTLD_NOW | RTLD_GLOBAL);
 
 	if (!p) {
 		sRetMsg = "Unable to open module [" + sModule + "] [" + dlerror() + "]";
@@ -1072,5 +1002,3 @@ ModHandle CModules::OpenModule(const CString& sModule, const CString& sModPath,
 
 	return p;
 }
-
-#endif // _MODULES
