@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -57,7 +57,7 @@ unsigned int CNick::GetCommonChans(vector<CChan*>& vRetChans, CUser* pUser) cons
 		CChan* pChan = vChans[a];
 		const map<CString,CNick*>& msNicks = pChan->GetNicks();
 
-		for (map<CString,CNick*>::const_iterator it = msNicks.begin(); it != msNicks.end(); it++) {
+		for (map<CString,CNick*>::const_iterator it = msNicks.begin(); it != msNicks.end(); ++it) {
 			if (it->first.Equals(m_sNick)) {
 				vRetChans.push_back(pChan);
 				continue;
@@ -151,3 +151,12 @@ CString CNick::GetHostMask() const {
 
 	return (sRet);
 }
+
+void CNick::Clone(const CNick& SourceNick) {
+	SetNick(SourceNick.GetNick());
+	SetIdent(SourceNick.GetIdent());
+	SetHost(SourceNick.GetHost());
+
+	m_sChanPerms = SourceNick.m_sChanPerms;
+	m_pUser = SourceNick.m_pUser;
+}
