@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2008  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -13,23 +13,21 @@
 
 class CServer {
 public:
-	CServer(const CString& sName, unsigned short uPort = 6667, const CString& sPass = "", bool bSSL = false, bool bIPV6 = false);
-	virtual ~CServer();
+	CServer(const CString& sName, unsigned short uPort = 6667, const CString& sPass = "", bool bSSL = false);
+	~CServer();
 
 	const CString& GetName() const;
 	unsigned short GetPort() const;
 	const CString& GetPass() const;
 	bool IsSSL() const;
-	bool IsIPV6() const;
 	CString GetString() const;
 	static bool IsValidHostName(const CString& sHostName);
 private:
 protected:
-	CString			m_sName;
-	unsigned short	m_uPort;
-	CString			m_sPass;
-	bool			m_bSSL;
-	bool			m_bIPV6;
+	CString         m_sName;
+	unsigned short  m_uPort;
+	CString         m_sPass;
+	bool            m_bSSL;
 };
 
 #endif // !_SERVER_H
