@@ -1,16 +1,15 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
  * by the Free Software Foundation.
  */
 
-#ifdef _MODULES
-
 #include "Template.h"
 #include "FileUtils.h"
 #include <sstream>
+#include <algorithm>
 
 using std::stringstream;
 
@@ -69,7 +68,7 @@ CString CTemplateLoopContext::GetValue(const CString& sName, bool bFromIf) {
 }
 
 CTemplate::~CTemplate() {
-	for (map<CString, vector<CTemplate*> >::iterator it = m_mvLoops.begin(); it != m_mvLoops.end(); it++) {
+	for (map<CString, vector<CTemplate*> >::iterator it = m_mvLoops.begin(); it != m_mvLoops.end(); ++it) {
 		vector<CTemplate*>& vLoop = it->second;
 		for (unsigned int a = 0; a < vLoop.size(); a++) {
 			delete vLoop[a];
@@ -90,44 +89,50 @@ void CTemplate::Init() {
 	}
 	*/
 
-	ClearPath();
+	ClearPaths();
 	m_pParent = NULL;
 }
 
-CString CTemplate::ExpandFile(const CString& sFilename) {
-	if (sFilename.Left(1) == "/" || sFilename.Left(2) == "./") {
+CString CTemplate::ExpandFile(const CString& sFilename, bool bFromInc) {
+	/*if (sFilename.Left(1) == "/" || sFilename.Left(2) == "./") {
 		return sFilename;
-	}
+	}*/
 
-	CString sFile(ResolveLiteral(sFilename));
+	CString sFile(ResolveLiteral(sFilename).TrimLeft_n("/"));
 
-	for (LCString::iterator it = m_lsPaths.begin(); it != m_lsPaths.end(); it++) {
-		CString sRoot = *it;
+	for (list<pair<CString, bool> >::iterator it = m_lsbPaths.begin(); it != m_lsbPaths.end(); ++it) {
+		CString& sRoot = it->first;
 		CString sFilePath(CDir::ChangeDir(sRoot, sFile));
 
+		// Make sure path ends with a slash because "/foo/pub*" matches "/foo/public_keep_out/" but "/foo/pub/*" doesn't
+		if (!sRoot.empty() && sRoot.Right(1) != "/") {
+			sRoot += "/";
+		}
+
+		if (it->second && !bFromInc) {
+			DEBUG("\t\tSkipping path (not from INC)  [" + sFilePath + "]");
+			continue;
+		}
+
 		if (CFile::Exists(sFilePath)) {
-			// This only works if sRoot got a trailing slash! The
-			// code which adds paths makes sure this is true.
 			if (sRoot.empty() || sFilePath.Left(sRoot.length()) == sRoot) {
-				//DEBUG("\t\tFound  [" + sFilePath + "]\n");
+				DEBUG("    Found  [" + sFilePath + "]");
 				return sFilePath;
 			} else {
 				DEBUG("\t\tOutside of root [" + sFilePath + "] !~ [" + sRoot + "]");
 			}
-		} else {
-			DEBUG("\t\tNo such file [" + sFilePath + "]");
 		}
 	}
 
-	switch (m_lsPaths.size()) {
+	switch (m_lsbPaths.size()) {
 		case 0:
 			DEBUG("Unable to find [" + sFile + "] using the current directory");
 			break;
 		case 1:
-			DEBUG("Unable to find [" + sFile + "] in the defined path [" + *m_lsPaths.begin());
+			DEBUG("Unable to find [" + sFile + "] in the defined path [" + m_lsbPaths.begin()->first + "]");
 			break;
 		default:
-			DEBUG("Unable to find [" + sFile + "] in any of the " + CString(m_lsPaths.size()) + " defined paths");
+			DEBUG("Unable to find [" + sFile + "] in any of the " + CString(m_lsbPaths.size()) + " defined paths");
 	}
 
 	return "";
@@ -138,31 +143,48 @@ void CTemplate::SetPath(const CString& sPaths) {
 	sPaths.Split(":", vsDirs, false);
 
 	for (size_t a = 0; a < vsDirs.size(); a++) {
-		AppendPath(vsDirs[a]);
+		AppendPath(vsDirs[a], false);
 	}
 }
 
-void CTemplate::PrependPath(const CString& sPath) {
-	DEBUG("CTemplate::PrependPath(" + sPath + ") == [" + CDir::ChangeDir("./", sPath + "/") + "]");
-	m_lsPaths.push_front(CDir::ChangeDir("./", sPath + "/"));
+CString CTemplate::MakePath(const CString& sPath) const {
+	CString sRet(CDir::ChangeDir("./", sPath + "/"));
+
+	if (!sRet.empty() && sRet.Right(1) != "/") {
+		sRet += "/";
+	}
+
+	return sRet;
 }
 
-void CTemplate::AppendPath(const CString& sPath) {
-	DEBUG("CTemplate::AppendPath(" + sPath + ") == [" + CDir::ChangeDir("./", sPath + "/") + "]");
-	m_lsPaths.push_back(CDir::ChangeDir("./", sPath + "/"));
+void CTemplate::PrependPath(const CString& sPath, bool bIncludesOnly) {
+	DEBUG("CTemplate::PrependPath(" + sPath + ") == [" + MakePath(sPath) + "]");
+	m_lsbPaths.push_front(make_pair(MakePath(sPath), bIncludesOnly));
+}
+
+void CTemplate::AppendPath(const CString& sPath, bool bIncludesOnly) {
+	DEBUG("CTemplate::AppendPath(" + sPath + ") == [" + MakePath(sPath) + "]");
+	m_lsbPaths.push_back(make_pair(MakePath(sPath), bIncludesOnly));
 }
 
 void CTemplate::RemovePath(const CString& sPath) {
 	DEBUG("CTemplate::RemovePath(" + sPath + ") == [" + CDir::ChangeDir("./", sPath + "/") + "]");
-	m_lsPaths.remove(CDir::ChangeDir("./", sPath + "/"));
+
+	for (list<pair<CString, bool> >::iterator it = m_lsbPaths.begin(); it != m_lsbPaths.end(); ++it) {
+		if (it->first == sPath) {
+			m_lsbPaths.remove(*it);
+			RemovePath(sPath); // @todo probably shouldn't use recursion, being lazy
+			return;
+		}
+	}
 }
 
-void CTemplate::ClearPath() {
-	m_lsPaths.clear();
+void CTemplate::ClearPaths() {
+	m_lsbPaths.clear();
 }
 
 bool CTemplate::SetFile(const CString& sFileName) {
-	m_sFileName = ExpandFile(sFileName);
+	m_sFileName = ExpandFile(sFileName, false);
 	PrependPath(sFileName + "/..");
 
 	if (sFileName.empty()) {
@@ -180,6 +202,15 @@ bool CTemplate::SetFile(const CString& sFileName) {
 	return true;
 }
 
+class CLoopSorter {
+	CString m_sType;
+public:
+	CLoopSorter(const CString& sType) : m_sType(sType) {}
+	bool operator()(CTemplate* pTemplate1, CTemplate* pTemplate2) {
+		return (pTemplate1->GetValue(m_sType, false) < pTemplate2->GetValue(m_sType, false));
+	}
+};
+
 CTemplate& CTemplate::AddRow(const CString& sName) {
 	CTemplate* pTmpl = new CTemplate(m_spOptions, this);
 	m_mvLoops[sName].push_back(pTmpl);
@@ -247,7 +278,7 @@ bool CTemplate::Print(const CString& sFileName, ostream& oOut) {
 	}
 
 	CString sLine;
-	CString	sSetBlockVar;
+	CString sSetBlockVar;
 	bool bValidLastIf = false;
 	bool bInSetBlock = false;
 	unsigned long uFilePos = 0;
@@ -311,7 +342,8 @@ bool CTemplate::Print(const CString& sFileName, ostream& oOut) {
 
 				if (!uSkip) {
 					if (sAction.Equals("INC")) {
-						if (!Print(ExpandFile(sArgs), oOut)) {
+						if (!Print(ExpandFile(sArgs, true), oOut)) {
+							DEBUG("Unable to print INC'd file [" + sArgs + "]");
 							return false;
 						}
 					} else if (sAction.Equals("SETOPTION")) {
@@ -323,7 +355,7 @@ bool CTemplate::Print(const CString& sFileName, ostream& oOut) {
 						if (sArgs.Token(1, true, " ").OptionSplit(msRow)) {
 							CTemplate& NewRow = AddRow(sLoopName);
 
-							for (MCString::iterator it = msRow.begin(); it != msRow.end(); it++) {
+							for (MCString::iterator it = msRow.begin(); it != msRow.end(); ++it) {
 								NewRow[it->first] = it->second;
 							}
 						}
@@ -365,7 +397,7 @@ bool CTemplate::Print(const CString& sFileName, ostream& oOut) {
 						sSetBlockVar = sArgs;
 						bInSetBlock = true;
 					} else if (sAction.Equals("EXPAND")) {
-						sOutput += ExpandFile(sArgs);
+						sOutput += ExpandFile(sArgs, true);
 					} else if (sAction.Equals("VAR")) {
 						sOutput += GetValue(sArgs);
 					} else if (sAction.Equals("LT")) {
@@ -407,8 +439,24 @@ bool CTemplate::Print(const CString& sFileName, ostream& oOut) {
 
 							CString sLoopName = sArgs.Token(0);
 							bool bReverse = (sArgs.Token(1).Equals("REVERSE"));
+							bool bSort = (sArgs.Token(1).Left(4).Equals("SORT"));
 							vector<CTemplate*>* pvLoop = GetLoop(sLoopName);
 
+							if (bSort && pvLoop != NULL &&  pvLoop->size() > 1) {
+								CString sKey;
+
+								if(sArgs.Token(1).TrimPrefix_n("SORT").Left(4).Equals("ASC=")) {
+									sKey = sArgs.Token(1).TrimPrefix_n("SORTASC=");
+								} else if(sArgs.Token(1).TrimPrefix_n("SORT").Left(5).Equals("DESC=")) {
+									sKey = sArgs.Token(1).TrimPrefix_n("SORTDESC=");
+									bReverse = true;
+								}
+
+								if (!sKey.empty()) {
+									std::sort(pvLoop->begin(), pvLoop->end(), CLoopSorter(sKey));
+								}
+							}
+
 							if (pvLoop) {
 								// If we found data for this loop, add it to our context vector
 								//unsigned long uBeforeLoopTag = uCurPos - iPos2 - 4;
@@ -767,11 +815,11 @@ CString CTemplate::GetValue(const CString& sArgs, bool bFromIf) {
 
 	if (!vspTagHandlers.empty()) { // @todo this should go up to the top to grab handlers
 		CTemplate* pTmpl = GetCurTemplate();
-		CString sCustomOutput;
 
 		if (sRet.empty()) {
 			for (unsigned int j = 0; j < vspTagHandlers.size(); j++) {
 				CSmartPtr<CTemplateTagHandler> spTagHandler = vspTagHandlers[j];
+				CString sCustomOutput;
 
 				if (!bFromIf && spTagHandler->HandleVar(*pTmpl, sArgs.Token(0), sArgs.Token(1, true), sCustomOutput)) {
 					sRet = sCustomOutput;
@@ -813,5 +861,3 @@ CString CTemplate::GetValue(const CString& sArgs, bool bFromIf) {
 
 	return sRet;
 }
-
-#endif // _MODULES
