@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -11,9 +11,43 @@
 #include "DCCSock.h"
 #include "IRCSock.h"
 #include "Server.h"
-#include "Timers.h"
 #include "znc.h"
 
+class CUserTimer : public CCron {
+public:
+	CUserTimer(CUser* pUser) : CCron() {
+		m_pUser = pUser;
+		SetName("CUserTimer::" + m_pUser->GetUserName());
+		Start(30);
+	}
+	virtual ~CUserTimer() {}
+
+private:
+protected:
+	virtual void RunJob() {
+		vector<CClient*>& vClients = m_pUser->GetClients();
+		CIRCSock* pIRCSock = m_pUser->GetIRCSock();
+
+		if (pIRCSock && pIRCSock->GetTimeSinceLastDataTransaction() >= 180) {
+			pIRCSock->PutIRC("PING :ZNC");
+		}
+
+		for (size_t a = 0; a < vClients.size(); a++) {
+			CClient* pClient = vClients[a];
+
+			if (pClient->GetTimeSinceLastDataTransaction() >= 180) {
+				pClient->PutClient("PING :ZNC");
+			}
+		}
+
+		if (m_pUser->IsIRCConnected()) {
+			m_pUser->JoinChans();
+		}
+	}
+
+	CUser* m_pUser;
+};
+
 CUser::CUser(const CString& sUserName) {
 	m_pIRCSock = NULL;
 	m_fTimezoneOffset = 0;
@@ -25,11 +59,9 @@ CUser::CUser(const CString& sUserName) {
 	m_uServerIdx = 0;
 	m_uBytesRead = 0;
 	m_uBytesWritten = 0;
-#ifdef _MODULES
 	m_pModules = new CModules;
-#endif
-	m_RawBuffer.SetLineCount(100);		// This should be more than enough raws, especially since we are buffering the MOTD separately
-	m_MotdBuffer.SetLineCount(200);		// This should be more than enough motd lines
+	m_RawBuffer.SetLineCount(100);   // This should be more than enough raws, especially since we are buffering the MOTD separately
+	m_MotdBuffer.SetLineCount(200);  // This should be more than enough motd lines
 	m_bMultiClients = true;
 	m_bBounceDCCs = true;
 	m_eHashType = HASH_NONE;
@@ -51,16 +83,12 @@ CUser::CUser(const CString& sUserName) {
 	m_bIRCConnectEnabled = true;
 	m_pUserTimer = new CUserTimer(this);
 	CZNC::Get().GetManager().AddCron(m_pUserTimer);
-	m_sUserPath = CZNC::Get().GetUserPath() + "/" + sUserName;
-	m_sDLPath = GetUserPath() + "/downloads";
 }
 
 CUser::~CUser() {
 	DelClients();
 
-#ifdef _MODULES
 	DelModules();
-#endif
 
 	DelServers();
 
@@ -78,7 +106,6 @@ CUser::~CUser() {
 	CZNC::Get().GetManager().DelCronByAddr(m_pUserTimer);
 }
 
-#ifdef _MODULES
 void CUser::DelModules() {
 	if (m_pModules) {
 		delete m_pModules;
@@ -93,7 +120,7 @@ bool CUser::UpdateModule(const CString &sModule) {
 	map<CUser*, CString>::iterator it2;
 	bool error = false;
 
-	for (it = Users.begin(); it != Users.end(); it++) {
+	for (it = Users.begin(); it != Users.end(); ++it) {
 		CModule *pMod = it->second->GetModules().FindModule(sModule);
 		if (pMod) {
 			Affected[it->second] = pMod->GetArgs();
@@ -102,7 +129,7 @@ bool CUser::UpdateModule(const CString &sModule) {
 	}
 
 	CString sErr;
-	for (it2 = Affected.begin(); it2 != Affected.end(); it2++) {
+	for (it2 = Affected.begin(); it2 != Affected.end(); ++it2) {
 		if (!it2->first->GetModules().LoadModule(sModule, it2->second, it2->first, sErr)) {
 			error = true;
 			DEBUG("Failed to reload [" << sModule << "] for [" << it2->first->GetUserName()
@@ -112,7 +139,6 @@ bool CUser::UpdateModule(const CString &sModule) {
 
 	return !error;
 }
-#endif
 
 void CUser::DelClients() {
 	for (unsigned int c = 0; c < m_vClients.size(); c++) {
@@ -134,19 +160,11 @@ void CUser::DelServers()
 
 void CUser::IRCConnected(CIRCSock* pIRCSock) {
 	m_pIRCSock = pIRCSock;
-
-	for (unsigned int a = 0; a < m_vClients.size(); a++) {
-		m_vClients[a]->IRCConnected(pIRCSock);
-	}
 }
 
 void CUser::IRCDisconnected() {
 	m_pIRCSock = NULL;
 
-	for (unsigned int a = 0; a < m_vClients.size(); a++) {
-		m_vClients[a]->IRCDisconnected();
-	}
-
 	SetIRCServer("");
 	m_bIRCAway = false;
 
@@ -235,7 +253,6 @@ void CUser::UserConnected(CClient* pClient) {
 		BounceAllClients();
 	}
 
-	PutStatus("Another client authenticated as your user, use the 'ListClients' command to see all clients");
 	m_vClients.push_back(pClient);
 
 	if (m_RawBuffer.IsEmpty()) {
@@ -253,22 +270,18 @@ void CUser::UserConnected(CClient* pClient) {
 	}
 
 	// Send the cached MOTD
-	if (m_MotdBuffer.IsEmpty()) {
-		PutIRC("MOTD");
-	} else {
-		unsigned int uIdx = 0;
-		CString sLine;
+	unsigned int uIdx = 0;
+	CString sLine;
 
-		while (m_MotdBuffer.GetLine(GetIRCNick().GetNick(), sLine, uIdx++)) {
-			pClient->PutClient(sLine);
-		}
+	while (m_MotdBuffer.GetLine(GetIRCNick().GetNick(), sLine, uIdx++)) {
+		pClient->PutClient(sLine);
 	}
 
 	if (GetIRCSock() != NULL) {
 		CString sUserMode("");
 		const set<unsigned char>& scUserModes = GetIRCSock()->GetUserModes();
 		for (set<unsigned char>::const_iterator it = scUserModes.begin();
-				it != scUserModes.end(); it++) {
+				it != scUserModes.end(); ++it) {
 			sUserMode += *it;
 		}
 		if (!sUserMode.empty()) {
@@ -339,6 +352,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	SetVHost(User.GetVHost());
 	SetDCCVHost(User.GetDCCVHost());
 	SetQuitMsg(User.GetQuitMsg());
+	SetSkinName(User.GetSkinName());
 	SetDefaultChanModes(User.GetDefaultChanModes());
 	SetBufferCount(User.GetBufferCount());
 	SetJoinTries(User.JoinTries());
@@ -347,7 +361,7 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	// Allowed Hosts
 	m_ssAllowedHosts.clear();
 	const set<CString>& ssHosts = User.GetAllowedHosts();
-	for (set<CString>::const_iterator it = ssHosts.begin(); it != ssHosts.end(); it++) {
+	for (set<CString>::const_iterator it = ssHosts.begin(); it != ssHosts.end(); ++it) {
 		AddAllowedHost(*it);
 	}
 
@@ -425,12 +439,13 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	// CTCP Replies
 	m_mssCTCPReplies.clear();
 	const MCString& msReplies = User.GetCTCPReplies();
-	for (MCString::const_iterator it = msReplies.begin(); it != msReplies.end(); it++) {
+	for (MCString::const_iterator it = msReplies.begin(); it != msReplies.end(); ++it) {
 		AddCTCPReply(it->first, it->second);
 	}
 	// !CTCP Replies
 
 	// Flags
+	SetIRCConnectEnabled(User.GetIRCConnectEnabled());
 	SetKeepBuffer(User.KeepBuffer());
 	SetMultiClients(User.MultiClients());
 	SetBounceDCCs(User.BounceDCCs());
@@ -444,7 +459,6 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 	SetTimezoneOffset(User.GetTimezoneOffset());
 	// !Flags
 
-#ifdef _MODULES
 	// Modules
 	set<CString> ssUnloadMods;
 	CModules& vCurMods = GetModules();
@@ -471,11 +485,10 @@ bool CUser::Clone(const CUser& User, CString& sErrorRet, bool bCloneChans) {
 		}
 	}
 
-	for (set<CString>::iterator it = ssUnloadMods.begin(); it != ssUnloadMods.end(); it++) {
+	for (set<CString>::iterator it = ssUnloadMods.begin(); it != ssUnloadMods.end(); ++it) {
 		vCurMods.UnloadModule(*it);
 	}
 	// !Modules
-#endif // !_MODULES
 
 	return true;
 }
@@ -495,7 +508,7 @@ bool CUser::IsHostAllowed(const CString& sHostMask) const {
 		return true;
 	}
 
-	for (set<CString>::const_iterator a = m_ssAllowedHosts.begin(); a != m_ssAllowedHosts.end(); a++) {
+	for (set<CString>::const_iterator a = m_ssAllowedHosts.begin(); a != m_ssAllowedHosts.end(); ++a) {
 		if (sHostMask.WildCmp(*a)) {
 			return true;
 		}
@@ -578,7 +591,7 @@ bool CUser::AddChan(const CString& sName, bool bInConfig) {
 }
 
 bool CUser::DelChan(const CString& sName) {
-	for (vector<CChan*>::iterator a = m_vChans.begin(); a != m_vChans.end(); a++) {
+	for (vector<CChan*>::iterator a = m_vChans.begin(); a != m_vChans.end(); ++a) {
 		if (sName.Equals((*a)->GetName())) {
 			delete *a;
 			m_vChans.erase(a);
@@ -589,8 +602,13 @@ bool CUser::DelChan(const CString& sName) {
 	return false;
 }
 
-bool CUser::PrintLine(CFile& File, const CString& sName, const CString& sValue) {
+bool CUser::PrintLine(CFile& File, CString sName, CString sValue) {
+	sName.Trim();
+	sValue.Trim();
+
 	if (sName.empty() || sValue.empty()) {
+		DEBUG("Refused writing an invalid line to a user config. ["
+			<< sName << "] [" << sValue << "]");
 		return false;
 	}
 
@@ -626,6 +644,7 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "QuitMsg", GetQuitMsg());
 	if (CZNC::Get().GetStatusPrefix() != GetStatusPrefix())
 		PrintLine(File, "StatusPrefix", GetStatusPrefix());
+	PrintLine(File, "Skin", GetSkinName());
 	PrintLine(File, "ChanModes", GetDefaultChanModes());
 	PrintLine(File, "Buffer", CString(GetBufferCount()));
 	PrintLine(File, "KeepBuffer", CString(KeepBuffer()));
@@ -641,11 +660,12 @@ bool CUser::WriteConfig(CFile& File) {
 	PrintLine(File, "TimezoneOffset", CString(m_fTimezoneOffset));
 	PrintLine(File, "JoinTries", CString(m_uMaxJoinTries));
 	PrintLine(File, "MaxJoins", CString(m_uMaxJoins));
+	PrintLine(File, "IRCConnectEnabled", CString(GetIRCConnectEnabled()));
 	File.Write("\n");
 
 	// Allow Hosts
 	if (!m_ssAllowedHosts.empty()) {
-		for (set<CString>::iterator it = m_ssAllowedHosts.begin(); it != m_ssAllowedHosts.end(); it++) {
+		for (set<CString>::iterator it = m_ssAllowedHosts.begin(); it != m_ssAllowedHosts.end(); ++it) {
 			PrintLine(File, "Allow", *it);
 		}
 
@@ -654,14 +674,13 @@ bool CUser::WriteConfig(CFile& File) {
 
 	// CTCP Replies
 	if (!m_mssCTCPReplies.empty()) {
-		for (MCString::iterator itb = m_mssCTCPReplies.begin(); itb != m_mssCTCPReplies.end(); itb++) {
+		for (MCString::iterator itb = m_mssCTCPReplies.begin(); itb != m_mssCTCPReplies.end(); ++itb) {
 			PrintLine(File, "CTCPReply", itb->first.AsUpper() + " " + itb->second);
 		}
 
 		File.Write("\n");
 	}
 
-#ifdef _MODULES
 	// Modules
 	CModules& Mods = GetModules();
 
@@ -678,7 +697,6 @@ bool CUser::WriteConfig(CFile& File) {
 
 		File.Write("\n");
 	}
-#endif
 
 	// Servers
 	for (unsigned int b = 0; b < m_vServers.size(); b++) {
@@ -696,6 +714,8 @@ bool CUser::WriteConfig(CFile& File) {
 		}
 	}
 
+	MODULECALL(OnWriteUserConfig(File), this, NULL,);
+
 	File.Write("</User>\n");
 
 	return true;
@@ -714,7 +734,7 @@ CChan* CUser::FindChan(const CString& sName) const {
 
 void CUser::JoinChans() {
 	// Avoid divsion by zero, it's bad!
-	if (m_vChans.size() == 0)
+	if (m_vChans.empty())
 		return;
 
 	// We start at a random offset into the channel list so that if your
@@ -893,7 +913,7 @@ CServer* CUser::GetNextServer() {
 		m_uServerIdx = 0;
 	}
 
-	return m_vServers[m_uServerIdx++];	// Todo: cycle through these
+	return m_vServers[m_uServerIdx++];
 }
 
 CServer* CUser::GetCurrentServer() const {
@@ -943,7 +963,7 @@ CString CUser::GetLocalIP() {
 		return pIRCSock->GetLocalIP();
 	}
 
-	if (m_vClients.size()) {
+	if (!m_vClients.empty()) {
 		return m_vClients[0]->GetLocalIP();
 	}
 
@@ -1098,7 +1118,7 @@ CString CUser::GetCurNick() const {
 		return pIRCSock->GetNick();
 	}
 
-	if (m_vClients.size()) {
+	if (!m_vClients.empty()) {
 		return m_vClients[0]->GetNick();
 	}
 
@@ -1113,6 +1133,10 @@ CString CUser::MakeCleanUserName(const CString& sUserName) {
 void CUser::SetUserName(const CString& s) {
 	m_sCleanUserName = CUser::MakeCleanUserName(s);
 	m_sUserName = s;
+
+	// set paths that depend on the user name:
+	m_sUserPath = CZNC::Get().GetUserPath() + "/" + m_sUserName;
+	m_sDLPath = GetUserPath() + "/downloads";
 }
 
 bool CUser::IsChan(const CString& sChan) const {
@@ -1224,4 +1248,6 @@ CString CUser::GetQuitMsg() const { return (!m_sQuitMsg.Trim_n().empty()) ? m_sQ
 const MCString& CUser::GetCTCPReplies() const { return m_mssCTCPReplies; }
 unsigned int CUser::GetBufferCount() const { return m_uBufferCount; }
 bool CUser::KeepBuffer() const { return m_bKeepBuffer; }
+//CString CUser::GetSkinName() const { return (!m_sSkinName.empty()) ? m_sSkinName : CZNC::Get().GetSkinName(); }
+CString CUser::GetSkinName() const { return m_sSkinName; }
 // !Getters
