@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -11,9 +11,7 @@
 
 #include "Buffer.h"
 #include "FileUtils.h"
-#ifdef _MODULES
 #include "Modules.h"
-#endif
 #include "Nick.h"
 #include <set>
 #include <vector>
@@ -48,7 +46,7 @@ public:
 		return CUtils::SaltedSHA256Hash(sPass, sSalt);
 	}
 
-	bool PrintLine(CFile& File, const CString& sName, const CString& sValue);
+	bool PrintLine(CFile& File, CString sName, CString sValue);
 	bool WriteConfig(CFile& File);
 	CChan* FindChan(const CString& sName) const;
 	bool AddChan(CChan* pChan);
@@ -73,7 +71,6 @@ public:
 
 	void DelClients();
 	void DelServers();
-#ifdef _MODULES
 	void DelModules();
 
 	// Unloads a module on all users who have it loaded and loads it again.
@@ -83,7 +80,6 @@ public:
 	CModules& GetModules() { return *m_pModules; }
 	const CModules& GetModules() const { return *m_pModules; }
 	// !Modules
-#endif
 
 	// Buffers
 	void AddRawBuffer(const CString& sPre, const CString& sPost, bool bIncNick = true) { m_RawBuffer.AddLine(sPre, sPost, bIncNick); }
@@ -169,6 +165,7 @@ public:
 	void SetTimezoneOffset(float b) { m_fTimezoneOffset = b; }
 	void SetJoinTries(unsigned int i) { m_uMaxJoinTries = i; }
 	void SetMaxJoins(unsigned int i) { m_uMaxJoins = i; }
+	void SetSkinName(const CString& s) { m_sSkinName = s; }
 	void SetIRCConnectEnabled(bool b) { m_bIRCConnectEnabled = b; }
 	void SetIRCAway(bool b) { m_bIRCAway = b; }
 	// !Setters
@@ -223,71 +220,72 @@ public:
 	unsigned long long BytesWritten() const { return m_uBytesWritten; }
 	unsigned int JoinTries() const { return m_uMaxJoinTries; }
 	unsigned int MaxJoins() const { return m_uMaxJoins; }
+	bool IsIRCAway() { return m_bIRCAway; }
+	CString GetSkinName() const;
 	// !Getters
 private:
 protected:
-	time_t			m_uConnectTime;
-	CString			m_sUserName;
-	CString			m_sCleanUserName;
-	CString			m_sNick;
-	CString			m_sAltNick;
-	CString			m_sIdent;
-	CString			m_sRealName;
-	CString			m_sVHost;
-	CString			m_sDCCVHost;
-	CString			m_sPass;
-	CString			m_sPassSalt;
-	CString			m_sStatusPrefix;
-	CString			m_sDefaultChanModes;
-	CString			m_sChanPrefixes;
-	CNick			m_IRCNick;
-	bool			m_bIRCAway;
-	CString			m_sIRCServer;
-	CString			m_sQuitMsg;
-	MCString		m_mssCTCPReplies;
-	CString			m_sTimestampFormat;
-	float			m_fTimezoneOffset;
-	eHashType		m_eHashType;
+	time_t                m_uConnectTime;
+	CString               m_sUserName;
+	CString               m_sCleanUserName;
+	CString               m_sNick;
+	CString               m_sAltNick;
+	CString               m_sIdent;
+	CString               m_sRealName;
+	CString               m_sVHost;
+	CString               m_sDCCVHost;
+	CString               m_sPass;
+	CString               m_sPassSalt;
+	CString               m_sStatusPrefix;
+	CString               m_sDefaultChanModes;
+	CString               m_sChanPrefixes;
+	CNick                 m_IRCNick;
+	bool                  m_bIRCAway;
+	CString               m_sIRCServer;
+	CString               m_sQuitMsg;
+	MCString              m_mssCTCPReplies;
+	CString               m_sTimestampFormat;
+	float                 m_fTimezoneOffset;
+	eHashType             m_eHashType;
 
 	// Paths
-	CString			m_sUserPath;
-	CString			m_sDLPath;
+	CString               m_sUserPath;
+	CString               m_sDLPath;
 	// !Paths
 
-	CBuffer				m_RawBuffer;
-	CBuffer				m_MotdBuffer;
-	CBuffer				m_QueryBuffer;
-	bool				m_bMultiClients;
-	bool				m_bBounceDCCs;
-	bool				m_bUseClientIP;
-	bool				m_bDenyLoadMod;
-	bool				m_bAdmin;
-	bool				m_bDenySetVHost;
-	bool				m_bKeepBuffer;
-	bool				m_bBeingDeleted;
-	bool				m_bAppendTimestamp;
-	bool				m_bPrependTimestamp;
-	bool				m_bIRCConnectEnabled;
-	CIRCSock*			m_pIRCSock;
+	CBuffer               m_RawBuffer;
+	CBuffer               m_MotdBuffer;
+	CBuffer               m_QueryBuffer;
+	bool                  m_bMultiClients;
+	bool                  m_bBounceDCCs;
+	bool                  m_bUseClientIP;
+	bool                  m_bDenyLoadMod;
+	bool                  m_bAdmin;
+	bool                  m_bDenySetVHost;
+	bool                  m_bKeepBuffer;
+	bool                  m_bBeingDeleted;
+	bool                  m_bAppendTimestamp;
+	bool                  m_bPrependTimestamp;
+	bool                  m_bIRCConnectEnabled;
+	CIRCSock*             m_pIRCSock;
 
-	CUserTimer*			m_pUserTimer;
+	CUserTimer*           m_pUserTimer;
 
-	vector<CServer*>	m_vServers;
-	vector<CChan*>		m_vChans;
-	vector<CClient*>	m_vClients;
-	set<CDCCBounce*>	m_sDCCBounces;
-	set<CDCCSock*>		m_sDCCSocks;
-	set<CString>		m_ssAllowedHosts;
-	unsigned int		m_uServerIdx; ///< Index in m_vServers of our current server + 1
-	unsigned int		m_uBufferCount;
-	unsigned long long      m_uBytesRead;
-	unsigned long long      m_uBytesWritten;
-	unsigned int		m_uMaxJoinTries;
-	unsigned int		m_uMaxJoins;
+	vector<CServer*>      m_vServers;
+	vector<CChan*>        m_vChans;
+	vector<CClient*>      m_vClients;
+	set<CDCCBounce*>      m_sDCCBounces;
+	set<CDCCSock*>        m_sDCCSocks;
+	set<CString>          m_ssAllowedHosts;
+	unsigned int          m_uServerIdx; ///< Index in m_vServers of our current server + 1
+	unsigned int          m_uBufferCount;
+	unsigned long long    m_uBytesRead;
+	unsigned long long    m_uBytesWritten;
+	unsigned int          m_uMaxJoinTries;
+	unsigned int          m_uMaxJoins;
+	CString               m_sSkinName;
 
-#ifdef _MODULES
-	CModules*		m_pModules;
-#endif
+	CModules*             m_pModules;
 };
 
 #endif // !_USER_H
