@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -17,20 +17,18 @@ public:
 	}
 
 	virtual bool OnLoad(const CString& sArgs, CString& sMessage) {
-		unsigned int a = 0;
-		CString sChan = sArgs.Token(a++);
+		VCString vsChans;
+		sArgs.Split(" ", vsChans, false);
 
-		while (!sChan.empty()) {
-			if (!Add(sChan)) {
-				PutModule("Unable to add [" + sChan + "]");
+		for (VCString::const_iterator it = vsChans.begin(); it != vsChans.end(); ++it) {
+			if (!Add(*it)) {
+				PutModule("Unable to add [" + *it + "]");
 			}
-
-			sChan = sArgs.Token(a++);
 		}
 
 		// Load our saved settings, ignore errors
 		MCString::iterator it;
-		for (it = BeginNV(); it != EndNV(); it++) {
+		for (it = BeginNV(); it != EndNV(); ++it) {
 			Add(it->first);
 		}
 
@@ -55,6 +53,11 @@ public:
 		return CONTINUE;
 	}
 
+	virtual EModRet OnChanAction(CNick& Nick, CChan& Channel, CString& sMessage) {
+		TryAttach(Channel);
+		return CONTINUE;
+	}
+
 	virtual void OnModCommand(const CString& sLine) {
 		CString sCommand = sLine.Token(0);
 
@@ -125,12 +128,12 @@ public:
 		if (sInput.Left(1) == "!") {
 			CString sChan = sInput.substr(1);
 			for (it = m_vsNegChans.begin(); it != m_vsNegChans.end();
-					it++) {
+					++it) {
 				if (*it == sChan)
 					return true;
 			}
 		} else {
-			for (it = m_vsChans.begin(); it != m_vsChans.end(); it++) {
+			for (it = m_vsChans.begin(); it != m_vsChans.end(); ++it) {
 				if (*it == sInput)
 					return true;
 			}
@@ -209,8 +212,8 @@ public:
 		return false;
 	}
 private:
-	vector<CString>	m_vsChans;
-	vector<CString>	m_vsNegChans;
+	vector<CString> m_vsChans;
+	vector<CString> m_vsNegChans;
 };
 
 MODULEDEFS(CChanAttach, "Reattaches you to channels on activity.")
