@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -11,7 +11,7 @@
 
 class CAutoOpMod;
 
-#define AUTOOP_CHALLENGE_LENGTH	32
+#define AUTOOP_CHALLENGE_LENGTH 32
 
 class CAutoOpTimer : public CTimer {
 public:
@@ -27,7 +27,7 @@ private:
 protected:
 	virtual void RunJob();
 
-	CAutoOpMod*		m_pParent;
+	CAutoOpMod* m_pParent;
 };
 
 class CAutoOpUser {
@@ -52,7 +52,7 @@ public:
 	const CString& GetHostmask() const { return m_sHostmask; }
 
 	bool ChannelMatches(const CString& sChan) const {
-		for (set<CString>::const_iterator it = m_ssChans.begin(); it != m_ssChans.end(); it++) {
+		for (set<CString>::const_iterator it = m_ssChans.begin(); it != m_ssChans.end(); ++it) {
 			if (sChan.AsLower().WildCmp(*it)) {
 				return true;
 			}
@@ -68,7 +68,7 @@ public:
 	CString GetChannels() const {
 		CString sRet;
 
-		for (set<CString>::const_iterator it = m_ssChans.begin(); it != m_ssChans.end(); it++) {
+		for (set<CString>::const_iterator it = m_ssChans.begin(); it != m_ssChans.end(); ++it) {
 			if (!sRet.empty()) {
 				sRet += " ";
 			}
@@ -100,7 +100,7 @@ public:
 	CString ToString() const {
 		CString sChans;
 
-		for (set<CString>::const_iterator it = m_ssChans.begin(); it != m_ssChans.end(); it++) {
+		for (set<CString>::const_iterator it = m_ssChans.begin(); it != m_ssChans.end(); ++it) {
 			if (!sChans.empty()) {
 				sChans += " ";
 			}
@@ -121,10 +121,10 @@ public:
 	}
 private:
 protected:
-	CString			m_sUsername;
-	CString			m_sUserKey;
-	CString			m_sHostmask;
-	set<CString>	m_ssChans;
+	CString      m_sUsername;
+	CString      m_sUserKey;
+	CString      m_sHostmask;
+	set<CString> m_ssChans;
 };
 
 class CAutoOpMod : public CModule {
@@ -135,7 +135,7 @@ public:
 		AddTimer(new CAutoOpTimer(this));
 
 		// Load the users
-		for (MCString::iterator it = BeginNV(); it != EndNV(); it++) {
+		for (MCString::iterator it = BeginNV(); it != EndNV(); ++it) {
 			const CString& sLine = it->second;
 			CAutoOpUser* pUser = new CAutoOpUser;
 
@@ -150,7 +150,7 @@ public:
 	}
 
 	virtual ~CAutoOpMod() {
-		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 			delete it->second;
 		}
 
@@ -160,14 +160,17 @@ public:
 	virtual void OnJoin(const CNick& Nick, CChan& Channel) {
 		// If we have ops in this chan
 		if (Channel.HasPerm(CChan::Op)) {
-			for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+			for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 				// and the nick who joined is a valid user
 				if (it->second->HostMatches(Nick.GetHostMask()) && it->second->ChannelMatches(Channel.GetName())) {
 					if (it->second->GetUserKey().Equals("__NOKEY__")) {
 						PutIRC("MODE " + Channel.GetName() + " +o " + Nick.GetNick());
 					} else {
 						// then insert this nick into the queue, the timer does the rest
-						m_msQueue[Nick.GetNick().AsLower()] = "";
+						CString sNick = Nick.GetNick().AsLower();
+						if (m_msQueue.find(sNick) == m_msQueue.end()) {
+							m_msQueue[sNick] = "";
+						}
 					}
 
 					break;
@@ -249,7 +252,7 @@ public:
 			Table.AddColumn("Key");
 			Table.AddColumn("Channels");
 
-			for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+			for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 				Table.AddRow();
 				Table.SetCell("User", it->second->GetUsername());
 				Table.SetCell("Hostmask", it->second->GetHostmask());
@@ -295,7 +298,7 @@ public:
 	}
 
 	CAutoOpUser* FindUserByHost(const CString& sHostmask, const CString& sChannel = "") {
-		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 			CAutoOpUser* pUser = it->second;
 
 			if (pUser->HostMatches(sHostmask) && (sChannel.empty() || pUser->ChannelMatches(sChannel))) {
@@ -337,7 +340,7 @@ public:
 		bool bMatchedHost = false;
 		CAutoOpUser* pUser = NULL;
 
-		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 			pUser = it->second;
 
 			// First verify that the guy who challenged us matches a user's host
@@ -396,7 +399,7 @@ public:
 		CString sChallenge = itQueue->second;
 		m_msQueue.erase(itQueue);
 
-		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+		for (map<CString, CAutoOpUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 			if (it->second->HostMatches(Nick.GetHostMask())) {
 				if (sResponse == CString(it->second->GetUserKey() + "::" + sChallenge).MD5()) {
 					OpUser(Nick, *it->second);
@@ -420,7 +423,7 @@ public:
 		while (bRemoved) {
 			bRemoved = false;
 
-			for (MCString::iterator it = m_msQueue.begin(); it != m_msQueue.end(); it++) {
+			for (MCString::iterator it = m_msQueue.begin(); it != m_msQueue.end(); ++it) {
 				if (!it->second.empty()) {
 					m_msQueue.erase(it);
 					bRemoved = true;
@@ -430,7 +433,7 @@ public:
 		}
 
 		// Now issue challenges for the new users in the queue
-		for (MCString::iterator it = m_msQueue.begin(); it != m_msQueue.end(); it++) {
+		for (MCString::iterator it = m_msQueue.begin(); it != m_msQueue.end(); ++it) {
 			it->second = CString::RandomString(AUTOOP_CHALLENGE_LENGTH);
 			PutIRC("NOTICE " + it->first + " :!ZNCAO CHALLENGE " + it->second);
 		}
@@ -452,8 +455,8 @@ public:
 		}
 	}
 private:
-	map<CString, CAutoOpUser*>		m_msUsers;
-	MCString						m_msQueue;
+	map<CString, CAutoOpUser*> m_msUsers;
+	MCString                   m_msQueue;
 };
 
 void CAutoOpTimer::RunJob() {
