@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  * Copyright (C) 2008 Michael "Svedrin" Ziegler diese-addy@funzt-halt.net
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -8,6 +8,7 @@
  */
 
 #include "Modules.h"
+#include "IRCSock.h"
 #include "User.h"
 
 class CAutoReplyMod : public CModule {
@@ -41,6 +42,12 @@ public:
 	}
 
 	void Handle(const CString& sNick) {
+		CIRCSock *pIRCSock = GetUser()->GetIRCSock();
+		if (!pIRCSock)
+			// WTF?
+			return;
+		if (sNick == pIRCSock->GetNick())
+			return;
 		if (m_Messaged.HasItem(sNick))
 			return;
 
@@ -78,7 +85,7 @@ public:
 	}
 
 private:
-	TCacheMap<CString>	m_Messaged;
+	TCacheMap<CString> m_Messaged;
 };
 
 MODULEDEFS(CAutoReplyMod, "Reply to queries when you are away")
