@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -83,7 +83,7 @@ public:
 		if (m_bIsAway)
 			Away(true); // reset away if we are reconnected
 		else
-			Back();	// ircd seems to remember your away if you killed the client and came back
+			Back(); // ircd seems to remember your away if you killed the client and came back
 	}
 
 	bool BootStrap()
@@ -96,7 +96,7 @@ public:
 
 			sFile.Split("\n", vsLines);
 
-			for (it = vsLines.begin(); it != vsLines.end(); it++) {
+			for (it = vsLines.begin(); it != vsLines.end(); ++it) {
 				CString sLine(*it);
 				sLine.Trim();
 				AddMessage(sLine);
@@ -248,7 +248,7 @@ public:
 				sTmp += sMessage;
 				msvOutput[sWhom].push_back(sTmp);
 			}
-			for (map< CString, vector< CString> >::iterator it = msvOutput.begin(); it != msvOutput.end(); it++)
+			for (map< CString, vector< CString> >::iterator it = msvOutput.begin(); it != msvOutput.end(); ++it)
 			{
 				PutModule(it->first, "away");
 				for (u_int a = 0; a < it->second.size(); a++)
@@ -367,8 +367,8 @@ public:
 	bool IsAway() { return(m_bIsAway); }
 
 private:
-	CString	m_sPassword;
-	bool	m_bBootError;
+	CString m_sPassword;
+	bool    m_bBootError;
 	bool DecryptMessages(CString & sBuffer)
 	{
 		CString sMessages = GetPath();
@@ -413,11 +413,11 @@ private:
 		m_vMessages.push_back(sText);
 	}
 
-	time_t			m_iLastSentData;
-	bool			m_bIsAway;
-	time_t		m_iAutoAway;
-	vector<CString>	m_vMessages;
-	CString			m_sReason;
+	time_t          m_iLastSentData;
+	bool            m_bIsAway;
+	time_t          m_iAutoAway;
+	vector<CString> m_vMessages;
+	CString         m_sReason;
 };
 
 
