@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -38,7 +38,7 @@ public:
 	virtual void OnQuit(const CNick& Nick, const CString& sMessage, const vector<CChan*>& vChans) {
 		vector<CChan*>::const_iterator it;
 		CString sMsg = Nick.GetNickMask() + " quit with message: [" + sMessage + "]";
-		for (it = vChans.begin(); it != vChans.end(); it++) {
+		for (it = vChans.begin(); it != vChans.end(); ++it) {
 			AddBuffer(**it, sMsg);
 		}
 	}
@@ -54,7 +54,7 @@ public:
 	virtual void OnNick(const CNick& OldNick, const CString& sNewNick, const vector<CChan*>& vChans) {
 		vector<CChan*>::const_iterator it;
 		CString sMsg = OldNick.GetNickMask() + " is now known as " + sNewNick;
-		for (it = vChans.begin(); it != vChans.end(); it++) {
+		for (it = vChans.begin(); it != vChans.end(); ++it) {
 			AddBuffer(**it, sMsg);
 		}
 	}
