@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -19,7 +19,7 @@ public:
 
 		m_bWriteConf = false;
 
-		for (; it != end; it++) {
+		for (; it != end; ++it) {
 			CChan *pChan = *it;
 
 			// If that channel isn't yet in the config,
@@ -68,7 +68,7 @@ public:
 	}
 
 private:
-	bool	m_bWriteConf;
+	bool m_bWriteConf;
 };
 
 MODULEDEFS(CChanSaverMod, "Keep config up-to-date when user joins/parts")
