@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -124,7 +124,7 @@ public:
 				Table.AddColumn("Target");
 				Table.AddColumn("Key");
 
-				for (MCString::iterator it = BeginNV(); it != EndNV(); it++) {
+				for (MCString::iterator it = BeginNV(); it != EndNV(); ++it) {
 					Table.AddRow();
 					Table.SetCell("Target", it->first);
 					Table.SetCell("Key", it->second);
