@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -52,7 +52,7 @@ public:
 		PutModule("is blocked after a failed login.");
 	}
 
-	virtual void OnClientConnect(CClient* pClient, const CString& sHost, unsigned short uPort) {
+	virtual void OnClientConnect(CZNCSock* pClient, const CString& sHost, unsigned short uPort) {
 		unsigned int *pCount = m_Cache.GetItem(sHost);
 		if (sHost.empty() || pCount == NULL || *pCount < m_uiAllowedFailed) {
 			return;
@@ -61,7 +61,7 @@ public:
 		// refresh their ban
 		Add(sHost, *pCount);
 
-		pClient->PutClient("ERROR :Closing link [Please try again later - reconnecting too fast]");
+		pClient->Write("ERROR :Closing link [Please try again later - reconnecting too fast]\r\n");
 		pClient->Close(Csock::CLT_AFTERWRITE);
 	}
 
@@ -91,8 +91,8 @@ public:
 	}
 
 private:
-	TCacheMap<CString, unsigned int>	m_Cache;
-	unsigned int			m_uiAllowedFailed;
+	TCacheMap<CString, unsigned int> m_Cache;
+	unsigned int                     m_uiAllowedFailed;
 };
 
 GLOBALMODULEDEFS(CFailToBanMod, "Block IPs for some time after a failed login")
