@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -89,7 +89,7 @@ public:
 	}
 
 private:
-	EType	m_eType;
+	EType m_eType;
 };
 
 
@@ -157,16 +157,16 @@ public:
 // # OnDisconnect($sockhandle)
 	virtual void Disconnected();
 // # OnData($sockhandle, $bytes, $length)
-	virtual void ReadData(const char *data, int len);
+	virtual void ReadData(const char *data, size_t len);
 // # OnReadLine($sockhandle, $line)
 	virtual void ReadLine(const CS_STRING & sLine);
 
 
 private:
-	CString		m_sModuleName;
-	CString		m_sUsername;	// NEED these so we can send the signal to the right guy
-	int			m_iParentFD;
-	VPString	m_vArgs;
+	CString  m_sModuleName;
+	CString  m_sUsername; // NEED these so we can send the signal to the right guy
+	int      m_iParentFD;
+	VPString m_vArgs;
 
 	void SetupArgs()
 	{
@@ -198,9 +198,9 @@ public:
 protected:
 	virtual void RunJob();
 
-	CString		m_sFuncName;
-	CString		m_sUserName;
-	CString		m_sModuleName;
+	CString m_sFuncName;
+	CString m_sUserName;
+	CString m_sModuleName;
 };
 
 class CModPerl : public CGlobalModule
@@ -424,10 +424,10 @@ public:
 
 	enum ECBTYPES
 	{
-		CB_LOCAL 	= 1,
-		CB_ONHOOK 	= 2,
-		CB_TIMER 	= 3,
-		CB_SOCK		= 4
+		CB_LOCAL  = 1,
+		CB_ONHOOK = 2,
+		CB_TIMER  = 3,
+		CB_SOCK   = 4
 	};
 
 	EModRet CallBack(const PString & sHookName, const VPString & vsArgs,
@@ -503,7 +503,7 @@ public:
 	void UnloadPerlMod(const CString & sModule);
 
 private:
-	PerlInterpreter	*m_pPerl;
+	PerlInterpreter *m_pPerl;
 
 };
 
@@ -1196,7 +1196,7 @@ void CPerlSock::Disconnected()
 	SOCKCB("OnDisconnect");
 }
 // # OnData($sockhandle, $bytes, $length)
-void CPerlSock::ReadData(const char *data, int len)
+void CPerlSock::ReadData(const char *data, size_t len)
 {
 	SetupArgs();
 	PString sData;
