@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -23,8 +23,11 @@ public:
 	{
 		if (sArgs.empty())
 			m_sPass = GetNV("Password");
-		else
+		else {
 			m_sPass = sArgs;
+			SetNV("Password", m_sPass);
+			SetArgs("");
+		}
 
 		return true;
 	}
@@ -70,7 +73,7 @@ public:
 	}
 
 private:
-	CString	m_sPass;
+	CString m_sPass;
 };
 
 MODULEDEFS(CNickServ, "Auths you with NickServ")
