@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -10,9 +10,9 @@
 #include "znc.h"
 
 // If you change these and it breaks, you get to keep the pieces
-#define CHAN_PREFIX_1	"~"
-#define CHAN_PREFIX_1C	'~'
-#define CHAN_PREFIX	CHAN_PREFIX_1 "#"
+#define CHAN_PREFIX_1   "~"
+#define CHAN_PREFIX_1C  '~'
+#define CHAN_PREFIX     CHAN_PREFIX_1 "#"
 
 class CPartylineChannel {
 public:
@@ -35,10 +35,10 @@ public:
 	bool IsFixedChan(const CString& s) { return m_ssFixedNicks.find(s) != m_ssFixedNicks.end(); }
 
 protected:
-	CString	m_sTopic;
-	CString	m_sName;
-	set<CString>	m_ssNicks;
-	set<CString>	m_ssFixedNicks;
+	CString      m_sTopic;
+	CString      m_sName;
+	set<CString> m_ssNicks;
+	set<CString> m_ssFixedNicks;
 };
 
 class CPartylineMod : public CGlobalModule {
@@ -62,7 +62,7 @@ public:
 	virtual bool OnLoad(const CString& sArgs, CString& sMessage) {
 		const map<CString, CUser*>& msUsers = CZNC::Get().GetUserMap();
 
-		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); ++it) {
 			CUser* pUser = it->second;
 			if (pUser->GetIRCSock()) {
 				if (pUser->GetChanPrefixes().find(CHAN_PREFIX_1) == CString::npos) {
@@ -71,12 +71,13 @@ public:
 			}
 		}
 
-		CString sChan;
-		unsigned int a = 0;
-		while (!(sChan = sArgs.Token(a++)).empty()) {
-			if (sChan.Left(2) == CHAN_PREFIX) {
-				sChan = sChan.Left(32);
-				m_ssDefaultChans.insert(sChan);
+		VCString vsChans;
+		VCString::const_iterator it;
+		sArgs.Split(" ", vsChans, false);
+
+		for (it = vsChans.begin(); it != vsChans.end(); ++it) {
+			if (it->Left(2) == CHAN_PREFIX) {
+				m_ssDefaultChans.insert(it->Left(32));
 			}
 		}
 
@@ -86,23 +87,42 @@ public:
 	}
 
 	void Load() {
-		VCString vsChannels;
-		for (MCString::iterator it = BeginNV(); it != EndNV(); it++) {
-			CUser* pUser = CZNC::Get().FindUser(it->first);
-			CPartylineChannel* pChannel;
-			it->second.Split(",", vsChannels, false);
-
-			if (!pUser) {
-				// TODO: give some usefull message?
-				continue;
+		CString sAction, sKey;
+		CPartylineChannel* pChannel;
+		for (MCString::iterator it = BeginNV(); it != EndNV(); ++it) {
+			if (it->first.find(":") != CString::npos) {
+				sAction = it->first.Token(0, false, ":");
+				sKey = it->first.Token(1, true, ":");
+			} else {
+				// backwards compatibility for older NV data
+				sAction = "fixedchan";
+				sKey = it->first;
 			}
 
-			for (VCString::iterator i = vsChannels.begin(); i != vsChannels.end(); i++) {
-				if (i->Trim_n().empty())
+			if (sAction == "fixedchan") {
+				CUser* pUser = CZNC::Get().FindUser(sKey);
+				if (!pUser) {
+					// TODO: give some useful message?
 					continue;
-				pChannel = GetChannel(*i);
-				JoinUser(pUser, pChannel);
-				pChannel->AddFixedNick(it->first);
+				}
+
+				VCString vsChannels;
+				it->second.Split(",", vsChannels, false);
+				for (VCString::iterator i = vsChannels.begin(); i != vsChannels.end(); ++i) {
+					if (i->Trim_n().empty())
+						continue;
+					pChannel = GetChannel(*i);
+					JoinUser(pUser, pChannel);
+					pChannel->AddFixedNick(sKey);
+				}
+			}
+
+			if (sAction == "topic") {
+				pChannel = FindChannel(sKey);
+				if (pChannel && !(it->second).empty()) {
+					PutChan(pChannel->GetNicks(), ":irc.znc.in TOPIC " + pChannel->GetName() + " :" + it->second);
+					pChannel->SetTopic(it->second);
+				}
 			}
 		}
 
@@ -114,21 +134,28 @@ public:
 		const CString &sUser = pUser->GetUserName();
 
 		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin();
-				it != m_ssChannels.end(); it++) {
+				it != m_ssChannels.end(); ++it) {
 			if ((*it)->IsFixedChan(sUser)) {
 				sChans += "," + (*it)->GetName();
 			}
 		}
 
 		if (!sChans.empty())
-			SetNV(sUser, sChans.substr(1)); // Strip away the first ,
+			SetNV("fixedchan:" + sUser, sChans.substr(1)); // Strip away the first ,
 		else
-			DelNV(sUser);
+			DelNV("fixedchan:" + sUser);
+	}
+
+	void SaveTopic(CPartylineChannel* pChannel) {
+		if (!pChannel->GetTopic().empty())
+			SetNV("topic:" + pChannel->GetName(), pChannel->GetTopic());
+		else
+			DelNV("topic:" + pChannel->GetName());
 	}
 
 	virtual EModRet OnDeleteUser(CUser& User) {
 		// Loop through each chan
-		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
+		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); ++it) {
 			RemoveUser(&User, *it, "KICK", true, "User deleted", true);
 		}
 
@@ -159,7 +186,7 @@ public:
 		}
 
 		// Make sure this user is in the default channels
-		for (set<CString>::iterator a = m_ssDefaultChans.begin(); a != m_ssDefaultChans.end(); a++) {
+		for (set<CString>::iterator a = m_ssDefaultChans.begin(); a != m_ssDefaultChans.end(); ++a) {
 			CPartylineChannel* pChannel = GetChannel(*a);
 			const CString& sNick = m_pUser->GetUserName();
 
@@ -181,7 +208,7 @@ public:
 
 		CString sNickMask = m_pClient->GetNickMask();
 
-		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
+		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); ++it) {
 			const set<CString>& ssNicks = (*it)->GetNicks();
 
 			if ((*it)->IsInChannel(m_pUser->GetUserName())) {
@@ -200,7 +227,7 @@ public:
 
 	virtual void OnClientDisconnect() {
 		if (!m_pUser->IsUserAttached() && !m_pUser->IsBeingDeleted()) {
-			for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
+			for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); ++it) {
 				const set<CString>& ssNicks = (*it)->GetNicks();
 
 				if (ssNicks.find(m_pUser->GetUserName()) != ssNicks.end()) {
@@ -229,6 +256,7 @@ public:
 					if (m_pUser->IsAdmin()) {
 						PutChan(ssNicks, ":" + m_pUser->GetIRCNick().GetNickMask() + " TOPIC " + sChannel + " :" + sTopic);
 						pChannel->SetTopic(sTopic);
+						SaveTopic(pChannel);
 					} else {
 						m_pUser->PutUser(":irc.znc.in 482 " +  m_pUser->GetIRCNick().GetNick() + " " + sChannel + " :You're not channel operator");
 					}
@@ -298,12 +326,13 @@ public:
 				pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + sCmd
 						+ pChannel->GetName() + " " + pUser->GetIRCNick().GetNick() + sMsg);
 				PutChan(ssNicks, ":?" + pUser->GetUserName() + "!" + pUser->GetIdent() + "@" + sHost
-						+ sCmd + pChannel->GetName() + " ?" + pUser->GetUserName() + sMsg, false);
+						+ sCmd + pChannel->GetName() + " ?" + pUser->GetUserName() + sMsg,
+						false, true, pUser);
 			} else {
 				pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + sCmd
 						+ pChannel->GetName() + sMsg);
 				PutChan(ssNicks, ":?" + pUser->GetUserName() + "!" + pUser->GetIdent() + "@" + sHost
-						+ sCmd + pChannel->GetName() + sMsg, false);
+						+ sCmd + pChannel->GetName() + sMsg, false, true, pUser);
 			}
 
 			if (ssNicks.empty()) {
@@ -313,6 +342,11 @@ public:
 		} else {
 			// some clients dont wait for the server to send an answer to a part, so we need to make them join again
 			pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + " JOIN " + pChannel->GetName());
+			if (!pChannel->GetTopic().empty()) {
+				pUser->PutUser(":" + GetIRCServer(pUser) + " 332 " + pUser->GetIRCNick().GetNickMask() + " " + pChannel->GetName() + " :" + pChannel->GetTopic());
+			}
+			const set<CString>& ssNicks = pChannel->GetNicks();
+			SendNickList(pUser, ssNicks, pChannel->GetName());
 		}
 	}
 
@@ -347,7 +381,7 @@ public:
 			}
 
 			pUser->PutUser(":" + pUser->GetIRCNick().GetNickMask() + " JOIN " + pChannel->GetName());
-			PutChan(ssNicks, ":?" + sNick + "!" + pUser->GetIdent() + "@" + sHost + " JOIN " + pChannel->GetName(), false);
+			PutChan(ssNicks, ":?" + sNick + "!" + pUser->GetIdent() + "@" + sHost + " JOIN " + pChannel->GetName(), false, true, pUser);
 
 			if (!pChannel->GetTopic().empty()) {
 				pUser->PutUser(":" + GetIRCServer(pUser) + " 332 " + pUser->GetIRCNick().GetNickMask() + " " + pChannel->GetName() + " :" + pChannel->GetTopic());
@@ -355,12 +389,12 @@ public:
 			SendNickList(pUser, ssNicks, pChannel->GetName());
 
 			if (pUser->IsAdmin()) {
-				PutChan(ssNicks, ":*" + GetModName() + "!znc@znc.in MODE " + pChannel->GetName() + " +o ?" + pUser->GetUserName(), false);
+				PutChan(ssNicks, ":*" + GetModName() + "!znc@znc.in MODE " + pChannel->GetName() + " +o ?" + pUser->GetUserName(), false, true, pUser);
 			}
 		}
 	}
 
-	virtual EModRet OnUserMsg(CString& sTarget, CString& sMessage) {
+	virtual EModRet HandleMessage(const CString& sCmd, const CString& sTarget, const CString& sMessage) {
 		if (sTarget.empty()) {
 			return CONTINUE;
 		}
@@ -383,13 +417,13 @@ public:
 				return HALT;
 			}
 
-			PutChan(sTarget, ":?" + m_pUser->GetUserName() + "!" + m_pUser->GetIdent() + "@" + sHost + " PRIVMSG " + sTarget + " :" + sMessage, true, false);
+			PutChan(sTarget, ":?" + m_pUser->GetUserName() + "!" + m_pUser->GetIdent() + "@" + sHost + " " + sCmd + " " + sTarget + " :" + sMessage, true, false);
 		} else {
 			CString sNick = sTarget.LeftChomp_n(1);
 			CUser* pUser = CZNC::Get().FindUser(sNick);
 
 			if (pUser) {
-				pUser->PutUser(":?" + m_pUser->GetUserName() + "!" + m_pUser->GetIdent() + "@" + sHost + " PRIVMSG " + pUser->GetIRCNick().GetNick() + " :" + sMessage);
+				pUser->PutUser(":?" + m_pUser->GetUserName() + "!" + m_pUser->GetIdent() + "@" + sHost + " " + sCmd + " " + pUser->GetIRCNick().GetNick() + " :" + sMessage);
 			} else {
 				m_pClient->PutClient(":" + GetIRCServer(m_pUser) + " 403 " + m_pUser->GetIRCNick().GetNick() + " " + sTarget + " :No such znc user: " + sNick + "");
 			}
@@ -398,6 +432,26 @@ public:
 		return HALT;
 	}
 
+	virtual EModRet OnUserMsg(CString& sTarget, CString& sMessage) {
+		return HandleMessage("PRIVMSG", sTarget, sMessage);
+	}
+
+	virtual EModRet OnUserNotice(CString& sTarget, CString& sMessage) {
+		return HandleMessage("NOTICE", sTarget, sMessage);
+	}
+
+	virtual EModRet OnUserAction(CString& sTarget, CString& sMessage) {
+		return HandleMessage("PRIVMSG", sTarget, "\001ACTION " + sMessage + "\001");
+	}
+
+	virtual EModRet OnUserCTCP(CString& sTarget, CString& sMessage) {
+		return HandleMessage("PRIVMSG", sTarget, "\001" + sMessage + "\001");
+	}
+
+	virtual EModRet OnUserCTCPReply(CString& sTarget, CString& sMessage) {
+		return HandleMessage("NOTICE", sTarget, "\001" + sMessage + "\001");
+	}
+
 	virtual void OnModCommand(const CString& sLine) {
 		CString sCommand = sLine.Token(0);
 
@@ -449,7 +503,7 @@ public:
 			Table.AddColumn("Channel");
 			Table.AddColumn("Users");
 
-			for (set<CPartylineChannel*>::const_iterator a = m_ssChannels.begin(); a != m_ssChannels.end(); a++) {
+			for (set<CPartylineChannel*>::const_iterator a = m_ssChannels.begin(); a != m_ssChannels.end(); ++a) {
 				Table.AddRow();
 
 				Table.SetCell("Channel", (*a)->GetName());
@@ -521,7 +575,7 @@ public:
 				return;
 			}
 
-			for (set<CPartylineChannel*>::const_iterator a = m_ssChannels.begin(); a != m_ssChannels.end(); a++) {
+			for (set<CPartylineChannel*>::const_iterator a = m_ssChannels.begin(); a != m_ssChannels.end(); ++a) {
 				if ((*a)->IsFixedChan(sUser)) {
 					PutModule((*a)->GetName());
 				}
@@ -540,14 +594,14 @@ public:
 				return;
 			}
 			const set<CString>& sNicks = pChan->GetNicks();
-			for (set<CString>::const_iterator it = sNicks.begin(); it != sNicks.end(); it++) {
+			for (set<CString>::const_iterator it = sNicks.begin(); it != sNicks.end(); ++it) {
 				if (pChan->IsFixedChan(*it)) {
 					PutModule(*it);
 				}
 			}
 			PutModule("--- End of list");
 	} else {
-			PutModule("Unkown command, try 'HELP'");
+			PutModule("Unknown command, try 'HELP'");
 		}
 	}
 
@@ -578,7 +632,7 @@ public:
 		if (!pClient)
 			pClient = m_pClient;
 
-		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+		for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); ++it) {
 			if (ssNicks.find(it->first) != ssNicks.end()) {
 				if (it->second == pUser) {
 					if (bIncludeCurUser) {
@@ -600,7 +654,7 @@ public:
 
 		const vector<CClient*>& vClients = pUser->GetClients();
 		vector<CClient*>::const_iterator it;
-		for (it = vClients.begin(); it != vClients.end(); it++) {
+		for (it = vClients.begin(); it != vClients.end(); ++it) {
 			(*it)->PutClient(sPre + (*it)->GetNick() + sPost);
 		}
 	}
@@ -608,7 +662,7 @@ public:
 	void SendNickList(CUser* pUser, const set<CString>& ssNicks, const CString& sChan) {
 		CString sNickList;
 
-		for (set<CString>::const_iterator it = ssNicks.begin(); it != ssNicks.end(); it++) {
+		for (set<CString>::const_iterator it = ssNicks.begin(); it != ssNicks.end(); ++it) {
 			CUser* pChanUser = CZNC::Get().FindUser(*it);
 
 			if (pChanUser && pChanUser->IsUserAttached()) {
@@ -633,7 +687,7 @@ public:
 	CPartylineChannel* FindChannel(const CString& sChan) {
 		CString sChannel = sChan.AsLower();
 
-		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); it++) {
+		for (set<CPartylineChannel*>::iterator it = m_ssChannels.begin(); it != m_ssChannels.end(); ++it) {
 			if ((*it)->GetName().AsLower() == sChannel)
 				return *it;
 		}
@@ -653,9 +707,9 @@ public:
 	}
 
 private:
-	set<CPartylineChannel*>	m_ssChannels;
-	set<CUser*>		m_spInjectedPrefixes;
-	set<CString>	m_ssDefaultChans;
+	set<CPartylineChannel*> m_ssChannels;
+	set<CUser*>             m_spInjectedPrefixes;
+	set<CString>            m_ssDefaultChans;
 };
 
 GLOBALMODULEDEFS(CPartylineMod, "Internal channels and queries for users connected to znc")
