@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -89,7 +89,7 @@ public:
 
 	virtual void OnIRCConnected() {
 		for (VCString::iterator it = m_vPerform.begin();
-			it != m_vPerform.end();  it++) {
+			it != m_vPerform.end();  ++it) {
 			PutIRC(GetUser()->ExpandString(*it));
 		}
 	}
@@ -98,7 +98,7 @@ private:
 	bool Save() {
 		CString sBuffer = "";
 
-		for (VCString::iterator it = m_vPerform.begin(); it != m_vPerform.end(); it++) {
+		for (VCString::iterator it = m_vPerform.begin(); it != m_vPerform.end(); ++it) {
 			sBuffer += *it + "\n";
 		}
 		SetNV("Perform", sBuffer);
@@ -106,7 +106,7 @@ private:
 		return true;
 	}
 
-	VCString	m_vPerform;
+	VCString m_vPerform;
 };
 
-MODULEDEFS(CPerform, "Adds perform capabilities")
+MODULEDEFS(CPerform, "Keeps a list of commands to be executed when ZNC connects to IRC.")
