@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -19,7 +19,7 @@ public:
 	}
 
 	virtual void OnModCommand(const CString& sCommand) {
-		m_pUser->PutUser(sCommand);
+		PutIRC(sCommand);
 	}
 
 	virtual EModRet OnUserRaw(CString& sLine) {
