@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -35,7 +35,7 @@ public:
 
 protected:
 	virtual void RunJob();
-	CString		m_sNick;
+	CString m_sNick;
 };
 
 class CSChatSock : public CSocket
@@ -54,7 +54,7 @@ public:
 
 	virtual bool ConnectionFrom(const CS_STRING & sHost, u_short iPort)
 	{
-		Close();	// close the listener after the first connection
+		Close(); // close the listener after the first connection
 		return(true);
 	}
 
@@ -92,9 +92,9 @@ public:
 	}
 
 private:
-	CSChat 					*m_pModule;
-	CString					m_sChatNick;
-	vector<CS_STRING>		m_vBuffer;
+	CSChat  *m_pModule;
+	CString  m_sChatNick;
+	VCString m_vBuffer;
 };
 
 class CSChat : public CModule
@@ -122,7 +122,7 @@ public:
 	virtual void OnClientLogin()
 	{
 		set<CSocket*>::const_iterator it;
-		for (it = BeginSockets(); it != EndSockets(); it++) {
+		for (it = BeginSockets(); it != EndSockets(); ++it) {
 			CSChatSock *p = (CSChatSock*) *it;
 
 			if (p->GetType() == CSChatSock::LISTENER)
@@ -155,7 +155,7 @@ public:
 		if (sCom.Equals("chat") && !sArgs.empty()) {
 			CString sNick = "(s)" + sArgs;
 			set<CSocket*>::const_iterator it;
-			for (it = BeginSockets(); it != EndSockets(); it++) {
+			for (it = BeginSockets(); it != EndSockets(); ++it) {
 				CSChatSock *pSock = (CSChatSock*) *it;
 
 				if (pSock->GetChatNick().Equals(sNick)) {
@@ -194,7 +194,7 @@ public:
 			Table.AddColumn("Cipher");
 
 			set<CSocket*>::const_iterator it;
-			for (it = BeginSockets(); it != EndSockets(); it++) {
+			for (it = BeginSockets(); it != EndSockets(); ++it) {
 				Table.AddRow();
 
 				CSChatSock *pSock = (CSChatSock*) *it;
@@ -231,7 +231,7 @@ public:
 				sArgs = "(s)" + sArgs;
 
 			set<CSocket*>::const_iterator it;
-			for (it = BeginSockets(); it != EndSockets(); it++) {
+			for (it = BeginSockets(); it != EndSockets(); ++it) {
 				CSChatSock *pSock = (CSChatSock*) *it;
 
 				if (sArgs.Equals(pSock->GetChatNick())) {
@@ -250,7 +250,7 @@ public:
 			Table.AddColumn("Cipher");
 
 			set<CSocket*>::const_iterator it;
-			for (it = BeginSockets(); it != EndSockets(); it++) {
+			for (it = BeginSockets(); it != EndSockets(); ++it) {
 				Table.AddRow();
 				Csock *pSock = *it;
 				Table.SetCell("SockName", pSock->GetSockName());
@@ -393,8 +393,8 @@ public:
 	}
 
 private:
-	map< CString,pair< u_long,u_short > >		m_siiWaitingChats;
-	CString		m_sPemFile;
+	map< CString,pair< u_long,u_short > > m_siiWaitingChats;
+	CString                               m_sPemFile;
 };
 
 
