@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -21,7 +21,7 @@ public:
 
 	virtual EModRet OnUserPart(CString& sChannel, CString& sMessage)
 	{
-		for (MCString::iterator it = BeginNV(); it != EndNV(); it++)
+		for (MCString::iterator it = BeginNV(); it != EndNV(); ++it)
 		{
 			if (sChannel.Equals(it->first))
 			{
@@ -59,7 +59,7 @@ public:
 		else if ((sCmdName == "list") && (sChannel.empty()))
 		{
 			int i = 1;
-			for (MCString::iterator it = BeginNV(); it != EndNV(); it++, i++)
+			for (MCString::iterator it = BeginNV(); it != EndNV(); ++it, i++)
 			{
 				if (it->second.empty())
 					PutModule(CString(i) + ": " + it->first);
@@ -79,7 +79,7 @@ public:
 		if (!m_pUser->GetIRCSock())
 			return;
 
-		for (MCString::iterator it = BeginNV(); it != EndNV(); it++)
+		for (MCString::iterator it = BeginNV(); it != EndNV(); ++it)
 		{
 			CChan *pChan = m_pUser->FindChan(it->first);
 			if (!pChan) {
@@ -100,7 +100,44 @@ public:
 			}
 		}
 	}
-private:
+
+	virtual CString GetWebMenuTitle() { return "Sticky Chans"; }
+
+	virtual bool OnWebRequest(CWebSock& WebSock, const CString& sPageName, CTemplate& Tmpl) {
+		if (sPageName.empty() || sPageName == "index") {
+			bool bSubmitted = (WebSock.GetParam("submitted").ToInt() != 0);
+
+			const vector<CChan*>& Channels = m_pUser->GetChans();
+			for (unsigned int c = 0; c < Channels.size(); c++) {
+				const CString sChan = Channels[c]->GetName();
+				bool bStick = FindNV(sChan) != EndNV();
+
+				if(bSubmitted) {
+					bool bNewStick = WebSock.GetParam("stick_" + sChan).ToBool();
+					if(bNewStick && !bStick)
+						SetNV(sChan, ""); // no password support for now unless chansaver is active too
+					else if(!bNewStick && bStick) {
+						MCString::iterator it = FindNV(sChan);
+						if(it != EndNV())
+							DelNV(it);
+					}
+					bStick = bNewStick;
+				}
+
+				CTemplate& Row = Tmpl.AddRow("ChannelLoop");
+				Row["Name"] = sChan;
+				Row["Sticky"] = CString(bStick);
+			}
+
+			if(bSubmitted) {
+				WebSock.GetSession()->AddSuccess("Changes have been saved!");
+			}
+
+			return true;
+		}
+
+		return false;
+	}
 
 };
 
@@ -116,7 +153,7 @@ bool CStickyChan::OnLoad(const CString& sArgs, CString& sMessage)
 	VCString::iterator it;
 	sArgs.Split(",", vsChans, false);
 
-	for (it = vsChans.begin(); it != vsChans.end(); it++) {
+	for (it = vsChans.begin(); it != vsChans.end(); ++it) {
 		CString sChan = it->Token(0);
 		CString sKey = it->Token(1, true);
 		SetNV(sChan, sKey);
