@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -29,8 +29,8 @@ public:
 	// !Setters
 private:
 protected:
-	bool	m_bNegated;
-	CString	m_sSource;
+	bool    m_bNegated;
+	CString m_sSource;
 };
 
 class CWatchEntry {
@@ -130,31 +130,28 @@ public:
 	void SetPattern(const CString& s) { m_sPattern = s; }
 	void SetDisabled(bool b = true) { m_bDisabled = b; }
 	void SetSources(const CString& sSources) {
-		unsigned int uIdx = 1;
-		CString sSrc = sSources.Token(0);
+		VCString vsSources;
+		VCString::iterator it;
+		sSources.Split(" ", vsSources, false);
 
 		m_vsSources.clear();
 
-		while (sSrc.size()) {
-			if (sSrc[0] == '!') {
-				if (sSrc.size() > 1) {
-					m_vsSources.push_back(CWatchSource(sSrc.substr(1), true));
-				}
+		for (it = vsSources.begin(); it != vsSources.end(); ++it) {
+			if (it->at(0) == '!' && it->size() > 1) {
+				m_vsSources.push_back(CWatchSource(it->substr(1), true));
 			} else {
-				m_vsSources.push_back(CWatchSource(sSrc, false));
+				m_vsSources.push_back(CWatchSource(*it, false));
 			}
-
-			sSrc = sSources.Token(uIdx++);
 		}
 	}
 	// !Setters
 private:
 protected:
-	CString		m_sHostMask;
-	CString		m_sTarget;
-	CString		m_sPattern;
-	bool		m_bDisabled;
-	vector<CWatchSource>	m_vsSources;
+	CString              m_sHostMask;
+	CString              m_sTarget;
+	CString              m_sPattern;
+	bool                 m_bDisabled;
+	vector<CWatchSource> m_vsSources;
 };
 
 class CWatcherMod : public CModule {
@@ -289,7 +286,7 @@ public:
 
 private:
 	void Process(const CNick& Nick, const CString& sMessage, const CString& sSource) {
-		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++) {
+		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); ++it) {
 			CWatchEntry& WatchEntry = *it;
 
 			if (WatchEntry.IsMatch(Nick, sMessage, sSource, m_pUser)) {
@@ -306,7 +303,7 @@ private:
 
 	void SetDisabled(unsigned int uIdx, bool bDisabled) {
 		if (uIdx == (unsigned int) ~0) {
-			for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++) {
+			for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); ++it) {
 				(*it).SetDisabled(bDisabled);
 			}
 
@@ -315,7 +312,7 @@ private:
 			return;
 		}
 
-		uIdx--;	// "convert" index to zero based
+		uIdx--; // "convert" index to zero based
 		if (uIdx >= m_lsWatchers.size()) {
 			PutModule("Invalid Id");
 			return;
@@ -391,7 +388,7 @@ private:
 	}
 
 	void SetSources(unsigned int uIdx, const CString& sSources) {
-		uIdx--;	// "convert" index to zero based
+		uIdx--; // "convert" index to zero based
 		if (uIdx >= m_lsWatchers.size()) {
 			PutModule("Invalid Id");
 			return;
@@ -407,7 +404,7 @@ private:
 	}
 
 	void Remove(unsigned int uIdx) {
-		uIdx--;	// "convert" index to zero based
+		uIdx--; // "convert" index to zero based
 		if (uIdx >= m_lsWatchers.size()) {
 			PutModule("Invalid Id");
 			return;
@@ -478,7 +475,7 @@ private:
 			CWatchEntry WatchEntry(sHostMask, sTarget, sPattern);
 
 			bool bExists = false;
-			for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++) {
+			for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); ++it) {
 				if (*it == WatchEntry) {
 					sMessage = "Entry for [" + WatchEntry.GetHostMask() + "] already exists.";
 					bExists = true;
@@ -505,7 +502,7 @@ private:
 
 	void Save() {
 		ClearNV(false);
-		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); it++) {
+		for (list<CWatchEntry>::iterator it = m_lsWatchers.begin(); it != m_lsWatchers.end(); ++it) {
 			CWatchEntry& WatchEntry = *it;
 			CString sSave;
 
@@ -530,7 +527,7 @@ private:
 
 		bool bWarn = false;
 
-		for (MCString::iterator it = BeginNV(); it != EndNV(); it++) {
+		for (MCString::iterator it = BeginNV(); it != EndNV(); ++it) {
 			VCString vList;
 			it->first.Split("\n", vList);
 
@@ -552,8 +549,8 @@ private:
 			PutModule("WARNING: malformed entry found while loading");
 	}
 
-	list<CWatchEntry>	m_lsWatchers;
-	CBuffer				m_Buffer;
+	list<CWatchEntry>  m_lsWatchers;
+	CBuffer            m_Buffer;
 };
 
 MODULEDEFS(CWatcherMod, "Copy activity from a specific user into a separate window")
