@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2009  See the AUTHORS file for details.
+ * Copyright (C) 2004-2010  See the AUTHORS file for details.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published
@@ -11,6 +11,7 @@
 #include "IRCSock.h"
 #include "Server.h"
 #include "User.h"
+#include "Listener.h"
 #include <pwd.h>
 #include <list>
 
@@ -18,10 +19,10 @@ namespace
 { // private namespace for local things
 	struct CGlobalModuleConfigLine
 	{
-		CString	m_sName;
-		CString	m_sValue;
-		CUser	*m_pUser;
-		CChan	*m_pChan;
+		CString  m_sName;
+		CString  m_sValue;
+		CUser   *m_pUser;
+		CChan   *m_pChan;
 	};
 }
 
@@ -31,11 +32,9 @@ CZNC::CZNC() {
 		exit(-1);
 	}
 
-#ifdef _MODULES
 	m_pModules = new CGlobalModules();
-#endif
 	m_pISpoofLockFile = NULL;
-	m_uiConnectDelay = 30;
+	m_uiConnectDelay = 5;
 	m_uiAnonIPLimit = 10;
 	SetISpoofFormat(""); // Set ISpoofFormat to default
 	m_uBytesRead = 0;
@@ -43,25 +42,24 @@ CZNC::CZNC() {
 	m_pConnectUserTimer = NULL;
 	m_eConfigState = ECONFIG_NOTHING;
 	m_TimeStarted = time(NULL);
+	m_sConnectThrottle.SetTTL(30000);
 }
 
 CZNC::~CZNC() {
 	if (m_pISpoofLockFile)
 		ReleaseISpoof();
 
-#ifdef _MODULES
 	m_pModules->UnloadAll();
 
-	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); ++a) {
 		a->second->GetModules().UnloadAll();
 	}
-#endif
 
 	for (size_t b = 0; b < m_vpListeners.size(); b++) {
 		delete m_vpListeners[b];
 	}
 
-	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); ++a) {
 		a->second->SetBeingDeleted(true);
 	}
 
@@ -70,9 +68,7 @@ CZNC::~CZNC() {
 	m_Manager.Cleanup();
 	DeleteUsers();
 
-#ifdef _MODULES
 	delete m_pModules;
-#endif
 
 	ShutdownCsocket();
 	DeletePidFile();
@@ -107,17 +103,15 @@ CString CZNC::GetUptime() const {
 }
 
 bool CZNC::OnBoot() {
-#ifdef _MODULES
 	if (!GetModules().OnBoot()) {
 		return false;
 	}
 
-	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 		if (!it->second->GetModules().OnBoot()) {
 			return false;
 		}
 	}
-#endif
 
 	return true;
 }
@@ -146,14 +140,19 @@ bool CZNC::ConnectUser(CUser *pUser) {
 	if (!pServer)
 		return false;
 
+	if (m_sConnectThrottle.GetItem(pServer->GetName()))
+		return false;
+
 	if (!WriteISpoof(pUser)) {
 		DEBUG("ISpoof could not be written");
 		pUser->PutStatus("ISpoof could not be written, retrying...");
 		return true;
 	}
 
-	DEBUG("User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << ":" << pServer->GetPort() << "] ...");
-	pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + ":" + CString(pServer->GetPort()) + "] ...");
+	m_sConnectThrottle.AddItem(pServer->GetName());
+
+	DEBUG("User [" << pUser->GetUserName() << "] is connecting to [" << pServer->GetName() << " " << pServer->GetPort() << "] ...");
+	pUser->PutStatus("Attempting to connect to [" + pServer->GetName() + " " + CString(pServer->GetPort()) + "] ...");
 
 	pIRCSock = new CIRCSock(pUser);
 	pIRCSock->SetPass(pServer->GetPass());
@@ -186,20 +185,18 @@ bool CZNC::HandleUserDeletion()
 	map<CString, CUser*>::iterator it;
 	map<CString, CUser*>::iterator end;
 
-	if (m_msDelUsers.size() == 0)
+	if (m_msDelUsers.empty())
 		return false;
 
 	end = m_msDelUsers.end();
-	for (it = m_msDelUsers.begin(); it != end; it++) {
+	for (it = m_msDelUsers.begin(); it != end; ++it) {
 		CUser* pUser = it->second;
 		pUser->SetBeingDeleted(true);
 
-#ifdef _MODULES
 		if (GetModules().OnDeleteUser(*pUser)) {
 			pUser->SetBeingDeleted(false);
 			continue;
 		}
-#endif
 		m_msUsers.erase(pUser->GetUserName());
 
 		// Don't use pUser->GetIRCSock(), as that only returns something if the
@@ -211,9 +208,8 @@ bool CZNC::HandleUserDeletion()
 		}
 
 		pUser->DelClients();
-#ifdef _MODULES
 		pUser->DelModules();
-#endif
+		CWebSock::FinishUserSessions(*pUser);
 		AddBytesRead(pUser->BytesRead());
 		AddBytesWritten(pUser->BytesWritten());
 		delete pUser;
@@ -315,7 +311,7 @@ void CZNC::ReleaseISpoof() {
 	m_pISpoofLockFile = NULL;
 }
 
-bool CZNC::WritePidFile(int iPid) {
+CFile* CZNC::InitPidFile() {
 	if (!m_sPidFile.empty()) {
 		CString sFile;
 
@@ -325,40 +321,45 @@ bool CZNC::WritePidFile(int iPid) {
 		else
 			sFile = m_sPidFile;
 
-		CFile File(sFile);
-		CUtils::PrintAction("Writing pid file [" + sFile + "]");
-
-		if (File.Open(O_WRONLY | O_TRUNC | O_CREAT)) {
-			File.Write(CString(iPid) + "\n");
-			File.Close();
-			CUtils::PrintStatus(true);
-			return true;
-		}
-
-		CUtils::PrintStatus(false);
+		return new CFile(sFile);
 	}
 
-	return false;
+	return NULL;
+}
+
+bool CZNC::WritePidFile(int iPid) {
+	CFile* File = InitPidFile();
+	if (File == NULL)
+		return false;
+
+	CUtils::PrintAction("Writing pid file [" + File->GetLongName() + "]");
+
+	bool bRet = false;
+	if (File->Open(O_WRONLY | O_TRUNC | O_CREAT)) {
+		File->Write(CString(iPid) + "\n");
+		File->Close();
+		bRet = true;
+	}
+
+	delete File;
+	CUtils::PrintStatus(bRet);
+	return bRet;
 }
 
 bool CZNC::DeletePidFile() {
-	if (!m_sPidFile.empty()) {
-		CString sFile;
-		// absolute path or relative to the data dir?
-		if (m_sPidFile[0] != '/')
-			sFile = GetZNCPath() + "/" + m_sPidFile;
-		else
-			sFile = m_sPidFile;
+	CFile* File = InitPidFile();
+	if (File == NULL)
+		return false;
 
-		CFile File(sFile);
-		CUtils::PrintAction("Deleting pid file [" + sFile + "]");
-		if (File.Delete()) {
-			CUtils::PrintStatus(true);
-			return true;
-		}
-		CUtils::PrintStatus(false);
-	}
-	return false;
+	CUtils::PrintAction("Deleting pid file [" + File->GetLongName() + "]");
+
+	bool bRet = false;
+	if (File->Delete())
+		bRet = true;
+
+	delete File;
+	CUtils::PrintStatus(bRet);
+	return bRet;
 }
 
 bool CZNC::WritePemFile() {
@@ -398,7 +399,7 @@ bool CZNC::WritePemFile() {
 }
 
 void CZNC::DeleteUsers() {
-	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); ++a) {
 		a->second->SetBeingDeleted(true);
 		delete a->second;
 	}
@@ -418,7 +419,7 @@ Csock* CZNC::FindSockByName(const CString& sSockName) {
 }
 
 bool CZNC::IsHostAllowed(const CString& sHostMask) const {
-	for (map<CString,CUser*>::const_iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+	for (map<CString,CUser*>::const_iterator a = m_msUsers.begin(); a != m_msUsers.end(); ++a) {
 		if (a->second->IsHostAllowed(sHostMask)) {
 			return true;
 		}
@@ -472,9 +473,9 @@ void CZNC::InitDirs(const CString& sArgvPath, const CString& sDataDir) {
 	}
 }
 
-CString CZNC::GetConfPath() const {
+CString CZNC::GetConfPath(bool bAllowMkDir) const {
 	CString sConfPath = m_sZNCPath + "/configs";
-	if (!CFile::Exists(sConfPath)) {
+	if (bAllowMkDir && !CFile::Exists(sConfPath)) {
 		CDir::MakeDir(sConfPath);
 	}
 
@@ -501,16 +502,16 @@ CString CZNC::GetModPath() const {
 }
 
 
-CString CZNC::ExpandConfigPath(const CString& sConfigFile) {
+CString CZNC::ExpandConfigPath(const CString& sConfigFile, bool bAllowMkDir) {
 	CString sRetPath;
 
 	if (sConfigFile.empty()) {
-		sRetPath = GetConfPath() + "/znc.conf";
+		sRetPath = GetConfPath(bAllowMkDir) + "/znc.conf";
 	} else {
 		if (sConfigFile.Left(2) == "./" || sConfigFile.Left(3) == "../") {
 			sRetPath = GetCurPath() + "/" + sConfigFile;
 		} else if (sConfigFile.Left(1) != "/") {
-			sRetPath = GetConfPath() + "/" + sConfigFile;
+			sRetPath = GetConfPath(bAllowMkDir) + "/" + sConfigFile;
 		} else {
 			sRetPath = sConfigFile;
 		}
@@ -541,6 +542,10 @@ bool CZNC::WriteConfig() {
 		return false;
 	}
 
+	if (GetModules().OnWriteConfig(m_LockFile)) {
+		return false;
+	}
+
 	m_LockFile.Write("AnonIPLimit  = " + CString(m_uiAnonIPLimit) + "\n");
 
 	for (size_t l = 0; l < m_vpListeners.size(); l++) {
@@ -551,12 +556,31 @@ bool CZNC::WriteConfig() {
 			sHostPortion = sHostPortion.FirstLine() + " ";
 		}
 
-		CString s6 = (pListener->IsIPV6()) ? "6" : " ";
+		CString sAcceptProtocol;
+		if(pListener->GetAcceptType() == CListener::ACCEPT_IRC)
+			sAcceptProtocol = "irc_only ";
+		else if(pListener->GetAcceptType() == CListener::ACCEPT_HTTP)
+			sAcceptProtocol = "web_only ";
 
-		m_LockFile.Write("Listen" + s6 + "      = " + sHostPortion + CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\n");
+		CString s6;
+		switch (pListener->GetAddrType()) {
+			case ADDR_IPV4ONLY:
+				s6 = "4";
+				break;
+			case ADDR_IPV6ONLY:
+				s6 = "6";
+				break;
+			case ADDR_ALL:
+				s6 = " ";
+				break;
+		}
+
+		m_LockFile.Write("Listener" + s6 + "    = " + sAcceptProtocol + sHostPortion +
+			CString((pListener->IsSSL()) ? "+" : "") + CString(pListener->GetPort()) + "\n");
 	}
 
 	m_LockFile.Write("ConnectDelay = " + CString(m_uiConnectDelay) + "\n");
+	m_LockFile.Write("ServerThrottle = " + CString(m_sConnectThrottle.GetTTL()/1000) + "\n");
 
 	if (!m_sISpoofFile.empty()) {
 		m_LockFile.Write("ISpoofFile   = " + m_sISpoofFile.FirstLine() + "\n");
@@ -568,6 +592,11 @@ bool CZNC::WriteConfig() {
 	if (!m_sPidFile.empty()) {
 		m_LockFile.Write("PidFile      = " + m_sPidFile.FirstLine() + "\n");
 	}
+
+	if (!m_sSkinName.empty()) {
+		m_LockFile.Write("Skin         = " + m_sSkinName.FirstLine() + "\n");
+	}
+
 	if (!m_sStatusPrefix.empty()) {
 		m_LockFile.Write("StatusPrefix = " + m_sStatusPrefix.FirstLine() + "\n");
 	}
@@ -580,7 +609,6 @@ bool CZNC::WriteConfig() {
 		m_LockFile.Write("VHost        = " + m_vsVHosts[v].FirstLine() + "\n");
 	}
 
-#ifdef _MODULES
 	CGlobalModules& Mods = GetModules();
 
 	for (unsigned int a = 0; a < Mods.size(); a++) {
@@ -593,9 +621,8 @@ bool CZNC::WriteConfig() {
 
 		m_LockFile.Write("LoadModule   = " + sName.FirstLine() + sArgs + "\n");
 	}
-#endif
 
-	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); it++) {
+	for (map<CString,CUser*>::iterator it = m_msUsers.begin(); it != m_msUsers.end(); ++it) {
 		CString sErr;
 
 		if (!it->second->IsValid(sErr)) {
@@ -654,10 +681,10 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 	}
 #endif
 
-	CString s6 = " ";
+	CString s6 = "4";
 #ifdef HAVE_IPV6
 	if (CUtils::GetBoolInput("Would you like ZNC to listen using ipv6?", false)) {
-		s6 = "6";
+		s6 = " ";
 	}
 #endif
 
@@ -668,14 +695,14 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 		sListenHost += " ";
 	}
 
-	vsLines.push_back("Listen" + s6 + "    = " + sListenHost + sSSL + CString(uListenPort));
+	vsLines.push_back("Listener" + s6 + "  = " + sListenHost + sSSL + CString(uListenPort));
 	// !Listen
 
-#ifdef _MODULES
 	set<CModInfo> ssGlobalMods;
 	GetModules().GetAvailableMods(ssGlobalMods, true);
+	size_t uNrOtherGlobalMods = FilterUncommonModules(ssGlobalMods);
 
-	if (ssGlobalMods.size()) {
+	if (!ssGlobalMods.empty()) {
 		CUtils::PrintMessage("");
 		CUtils::PrintMessage("-- Global Modules --");
 		CUtils::PrintMessage("");
@@ -686,7 +713,7 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 			Table.AddColumn("Description");
 			set<CModInfo>::iterator it;
 
-			for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); it++) {
+			for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); ++it) {
 				const CModInfo& Info = *it;
 				Table.AddRow();
 				Table.SetCell("Name", Info.GetName());
@@ -698,9 +725,13 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 				CUtils::PrintMessage(sLine);
 			}
 
+			if (uNrOtherGlobalMods > 0) {
+				CUtils::PrintMessage("And " + CString(uNrOtherGlobalMods) + " other (uncommon) modules. You can enable those later.");
+			}
+
 			CUtils::PrintMessage("");
 
-			for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); it++) {
+			for (it = ssGlobalMods.begin(); it != ssGlobalMods.end(); ++it) {
 				const CModInfo& Info = *it;
 				CString sName = Info.GetName();
 
@@ -714,7 +745,6 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 			}
 		}
 	}
-#endif
 
 	// User
 	CUtils::PrintMessage("");
@@ -774,9 +804,9 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 			vsLines.push_back("\tChanModes  = " + sAnswer);
 		}
 
-#ifdef _MODULES
 		set<CModInfo> ssUserMods;
 		GetModules().GetAvailableMods(ssUserMods);
+		size_t uNrOtherUserMods = FilterUncommonModules(ssUserMods);
 
 		if (ssUserMods.size()) {
 			vsLines.push_back("");
@@ -790,7 +820,7 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 				Table.AddColumn("Description");
 				set<CModInfo>::iterator it;
 
-				for (it = ssUserMods.begin(); it != ssUserMods.end(); it++) {
+				for (it = ssUserMods.begin(); it != ssUserMods.end(); ++it) {
 					const CModInfo& Info = *it;
 					Table.AddRow();
 					Table.SetCell("Name", Info.GetName());
@@ -802,9 +832,13 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 					CUtils::PrintMessage(sLine);
 				}
 
+				if (uNrOtherUserMods > 0) {
+					CUtils::PrintMessage("And " + CString(uNrOtherUserMods) + " other (uncommon) modules. You can enable those later.");
+				}
+
 				CUtils::PrintMessage("");
 
-				for (it = ssUserMods.begin(); it != ssUserMods.end(); it++) {
+				for (it = ssUserMods.begin(); it != ssUserMods.end(); ++it) {
 					const CModInfo& Info = *it;
 					CString sName = Info.GetName();
 
@@ -818,7 +852,6 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 				}
 			}
 		}
-#endif
 
 		vsLines.push_back("");
 		CUtils::PrintMessage("");
@@ -941,24 +974,42 @@ bool CZNC::WriteNewConfig(const CString& sConfigFile) {
 	return bFileOpen && CUtils::GetBoolInput("Launch znc now?", true);
 }
 
+size_t CZNC::FilterUncommonModules(set<CModInfo>& ssModules) {
+	const char* ns[] = { "webadmin", "admin",
+		"chansaver", "keepnick", "simple_away", "partyline",
+		"kickrejoin", "nickserv", "perform" };
+	const set<CString> ssNames(ns, ns + sizeof(ns) / sizeof(ns[0]));
+
+	size_t uNrRemoved = 0;
+	for(set<CModInfo>::iterator it = ssModules.begin(); it != ssModules.end(); ) {
+		if(ssNames.count(it->GetName()) > 0) {
+			it++;
+		} else {
+			set<CModInfo>::iterator it2 = it++;
+			ssModules.erase(it2);
+			uNrRemoved++;
+		}
+	}
+
+	return uNrRemoved;
+}
+
 bool CZNC::ParseConfig(const CString& sConfig)
 {
 	CString s;
 
-	m_sConfigFile = ExpandConfigPath(sConfig);
+	m_sConfigFile = ExpandConfigPath(sConfig, false);
 
 	return DoRehash(s);
 }
 
 bool CZNC::RehashConfig(CString& sError)
 {
-#ifdef _MODULES
 	GetModules().OnPreRehash();
 	for (map<CString, CUser*>::iterator itb = m_msUsers.begin();
-			itb != m_msUsers.end(); itb++) {
+			itb != m_msUsers.end(); ++itb) {
 		itb->second->GetModules().OnPreRehash();
 	}
-#endif
 
 	// This clears m_msDelUsers
 	HandleUserDeletion();
@@ -968,20 +1019,18 @@ bool CZNC::RehashConfig(CString& sError)
 	m_msUsers.clear();
 
 	if (DoRehash(sError)) {
-#ifdef _MODULES
 		GetModules().OnPostRehash();
 		for (map<CString, CUser*>::iterator it = m_msUsers.begin();
-				it != m_msUsers.end(); it++) {
+				it != m_msUsers.end(); ++it) {
 			it->second->GetModules().OnPostRehash();
 		}
-#endif
 
 		return true;
 	}
 
 	// Rehashing failed, try to recover
 	CString s;
-	while (m_msDelUsers.size()) {
+	while (!m_msDelUsers.empty()) {
 		AddUser(m_msDelUsers.begin()->second, s);
 		m_msDelUsers.erase(m_msDelUsers.begin());
 	}
@@ -1039,20 +1088,18 @@ bool CZNC::DoRehash(CString& sError)
 	m_vsMotd.clear();
 
 	// Delete all listeners
-	while (m_vpListeners.size()) {
+	while (!m_vpListeners.empty()) {
 		delete m_vpListeners[0];
 		m_vpListeners.erase(m_vpListeners.begin());
 	}
 
 	CString sLine;
-	bool bCommented = false;	// support for /**/ style comments
-	CUser* pUser = NULL;	// Used to keep track of which user block we are in
-	CUser* pRealUser = NULL;	// If we rehash a user, this is the real one
-	CChan* pChan = NULL;	// Used to keep track of which chan block we are in
+	bool bCommented = false;     // support for /**/ style comments
+	CUser* pUser = NULL;         // Used to keep track of which user block we are in
+	CUser* pRealUser = NULL;     // If we rehash a user, this is the real one
+	CChan* pChan = NULL;         // Used to keep track of which chan block we are in
 	unsigned int uLineNum = 0;
-#ifdef _MODULES
-	MCString msModules;	// Modules are queued for later loading
-#endif
+	MCString msModules;          // Modules are queued for later loading
 
 	std::list<CGlobalModuleConfigLine> lGlobalModuleConfigLine;
 
@@ -1354,6 +1401,9 @@ bool CZNC::DoRehash(CString& sError)
 					} else if (sName.Equals("PrependTimestamp")) {
 						pUser->SetTimestampPrepend(sValue.ToBool());
 						continue;
+					} else if (sName.Equals("IRCConnectEnabled")) {
+						pUser->SetIRCConnectEnabled(sValue.ToBool());
+						continue;
 					} else if (sName.Equals("Timestamp")) {
 						if (!sValue.Trim_n().Equals("true")) {
 							if (sValue.Trim_n().Equals("append")) {
@@ -1379,10 +1429,19 @@ bool CZNC::DoRehash(CString& sError)
 					} else if (sName.Equals("MaxJoins")) {
 						pUser->SetMaxJoins(sValue.ToUInt());
 						continue;
+					} else if (sName.Equals("Skin")) {
+						pUser->SetSkinName(sValue);
+						continue;
 					} else if (sName.Equals("LoadModule")) {
 						CString sModName = sValue.Token(0);
+
+						// XXX Legacy crap, added in znc 0.089
+						if (sModName == "discon_kick") {
+							CUtils::PrintMessage("NOTICE: [discon_kick] was renamed, loading [disconkick] instead");
+							sModName = "disconkick";
+						}
+
 						CUtils::PrintAction("Loading Module [" + sModName + "]");
-#ifdef _MODULES
 						CString sModRet;
 						CString sArgs = sValue.Token(1, true);
 
@@ -1398,22 +1457,31 @@ bool CZNC::DoRehash(CString& sError)
 							sError = sModRet;
 							return false;
 						}
-#else
-						sError = "Modules are not enabled.";
-						CUtils::PrintStatus(false, sError);
-#endif
 						continue;
 					}
 				}
 			} else {
-				if (sName.Equals("Listen") || sName.Equals("ListenPort") || sName.Equals("Listen6")) {
-					bool bSSL = false;
-					bool bIPV6 = sName.Equals("Listen6");
-					CString sPort;
+				if (sName.Equals("Listen") || sName.Equals("Listen6") || sName.Equals("Listen4")
+						|| sName.Equals("Listener") || sName.Equals("Listener6") || sName.Equals("Listener4")) {
+					EAddrType eAddr = ADDR_ALL;
+					if (sName.Equals("Listen4") || sName.Equals("Listen") || sName.Equals("Listener4")) {
+						eAddr = ADDR_IPV4ONLY;
+					}
+					if (sName.Equals("Listener6")) {
+						eAddr = ADDR_IPV6ONLY;
+					}
 
+					CListener::EAcceptType eAccept = CListener::ACCEPT_ALL;
+					if (sValue.TrimPrefix("irc_only "))
+						eAccept = CListener::ACCEPT_IRC;
+					else if (sValue.TrimPrefix("web_only "))
+						eAccept = CListener::ACCEPT_HTTP;
+
+					bool bSSL = false;
+					CString sPort;
 					CString sBindHost;
 
-					if (!bIPV6) {
+					if (ADDR_IPV4ONLY == eAddr) {
 						sValue.Replace(":", " ");
 					}
 
@@ -1437,15 +1505,22 @@ bool CZNC::DoRehash(CString& sError)
 
 					CString sIPV6Comment;
 
-					if (bIPV6) {
-						sIPV6Comment = " using ipv6";
+					switch (eAddr) {
+						case ADDR_ALL:
+							sIPV6Comment = "";
+							break;
+						case ADDR_IPV4ONLY:
+							sIPV6Comment = " using ipv4";
+							break;
+						case ADDR_IPV6ONLY:
+							sIPV6Comment = " using ipv6";
 					}
 
 					unsigned short uPort = sPort.ToUShort();
 					CUtils::PrintAction("Binding to port [" + CString((bSSL) ? "+" : "") + CString(uPort) + "]" + sHostComment + sIPV6Comment);
 
 #ifndef HAVE_IPV6
-					if (bIPV6) {
+					if (ADDR_IPV6ONLY == eAddr) {
 						sError = "IPV6 is not enabled";
 						CUtils::PrintStatus(false, sError);
 						return false;
@@ -1483,10 +1558,11 @@ bool CZNC::DoRehash(CString& sError)
 						return false;
 					}
 
-					CListener* pListener = new CListener(uPort, sBindHost, bSSL, bIPV6);
+					CListener* pListener = new CListener(uPort, sBindHost, bSSL, eAddr, eAccept);
 
 					if (!pListener->Listen()) {
-						sError = "Unable to bind [" + CString(strerror(errno)) + "]";
+						sError = (errno == 0 ? CString("unknown error, check the host name") : CString(strerror(errno)));
+						sError = "Unable to bind [" + sError + "]";
 						CUtils::PrintStatus(false, sError);
 						delete pListener;
 						return false;
@@ -1497,7 +1573,6 @@ bool CZNC::DoRehash(CString& sError)
 
 					continue;
 				} else if (sName.Equals("LoadModule")) {
-#ifdef _MODULES
 					CString sModName = sValue.Token(0);
 					CString sArgs = sValue.Token(1, true);
 
@@ -1508,9 +1583,6 @@ bool CZNC::DoRehash(CString& sError)
 						return false;
 					}
 					msModules[sModName] = sArgs;
-#else
-					CUtils::PrintError("Modules are not enabled.");
-#endif
 					continue;
 				} else if (sName.Equals("ISpoofFormat")) {
 					m_sISpoofFormat = sValue;
@@ -1531,12 +1603,18 @@ bool CZNC::DoRehash(CString& sError)
 				} else if (sName.Equals("PidFile")) {
 					m_sPidFile = sValue;
 					continue;
+				} else if (sName.Equals("Skin")) {
+					SetSkinName(sValue);
+					continue;
 				} else if (sName.Equals("StatusPrefix")) {
 					m_sStatusPrefix = sValue;
 					continue;
 				} else if (sName.Equals("ConnectDelay")) {
 					m_uiConnectDelay = sValue.ToUInt();
 					continue;
+				} else if (sName.Equals("ServerThrottle")) {
+					m_sConnectThrottle.SetTTL(sValue.ToUInt()*1000);
+					continue;
 				} else if (sName.Equals("AnonIPLimit")) {
 					m_uiAnonIPLimit = sValue.ToUInt();
 					continue;
@@ -1562,9 +1640,8 @@ bool CZNC::DoRehash(CString& sError)
 		}
 	}
 
-#ifdef _MODULES
 	// First step: Load and reload new modules or modules with new arguments
-	for (MCString::iterator it = msModules.begin(); it != msModules.end(); it++) {
+	for (MCString::iterator it = msModules.begin(); it != msModules.end(); ++it) {
 		CString sModName = it->first;
 		CString sArgs = it->second;
 		CString sModRet;
@@ -1614,7 +1691,7 @@ bool CZNC::DoRehash(CString& sError)
 			ssUnload.insert(pCurMod->GetModName());
 	}
 
-	for (set<CString>::iterator it = ssUnload.begin(); it != ssUnload.end(); it++) {
+	for (set<CString>::iterator it = ssUnload.begin(); it != ssUnload.end(); ++it) {
 		if (GetModules().UnloadModule(*it))
 			CUtils::PrintMessage("Unloaded Global Module [" + *it + "]");
 		else
@@ -1622,16 +1699,20 @@ bool CZNC::DoRehash(CString& sError)
 	}
 
 	// last step, throw unhandled config items at global config
-	for (std::list<CGlobalModuleConfigLine>::iterator it = lGlobalModuleConfigLine.begin(); it != lGlobalModuleConfigLine.end(); it++)
+	for (std::list<CGlobalModuleConfigLine>::iterator it = lGlobalModuleConfigLine.begin(); it != lGlobalModuleConfigLine.end(); ++it)
 	{
 		if ((pChan && pChan == it->m_pChan) || (pUser && pUser == it->m_pUser))
 			continue; // skip unclosed user or chan
-		if (!GetModules().OnConfigLine(it->m_sName, it->m_sValue, it->m_pUser, it->m_pChan))
-		{
+		bool bHandled = false;
+		if (it->m_pUser) {
+			MODULECALL(OnConfigLine(it->m_sName, it->m_sValue, it->m_pUser, it->m_pChan), it->m_pUser, NULL, bHandled = true);
+		} else {
+			bHandled = GetModules().OnConfigLine(it->m_sName, it->m_sValue, it->m_pUser, it->m_pChan);
+		}
+		if (!bHandled) {
 			CUtils::PrintMessage("unhandled global module config line [GM:" + it->m_sName + "] = [" + it->m_sValue + "]");
 		}
 	}
-#endif
 
 	if (pChan) {
 		// TODO last <Chan> not closed
@@ -1643,19 +1724,21 @@ bool CZNC::DoRehash(CString& sError)
 		delete pUser;
 	}
 
-	if (m_msUsers.size() == 0) {
+	if (m_msUsers.empty()) {
 		sError = "You must define at least one user in your config.";
 		CUtils::PrintError(sError);
 		return false;
 	}
 
-	if (m_vpListeners.size() == 0) {
+	if (m_vpListeners.empty()) {
 		sError = "You must supply at least one Listen port in your config.";
 		CUtils::PrintError(sError);
 		return false;
 	}
 
-	// Make sure that users that want to connect do so
+	// Make sure that users that want to connect do so and also make sure a
+	// new ConnectDelay setting is applied.
+	DisableConnectUser();
 	EnableConnectUser();
 
 	return true;
@@ -1682,7 +1765,7 @@ bool CZNC::AddVHost(const CString& sHost) {
 
 bool CZNC::RemVHost(const CString& sHost) {
 	VCString::iterator it;
-	for (it = m_vsVHosts.begin(); it != m_vsVHosts.end(); it++) {
+	for (it = m_vsVHosts.begin(); it != m_vsVHosts.end(); ++it) {
 		if (sHost.Equals(*it)) {
 			m_vsVHosts.erase(it);
 			return true;
@@ -1694,7 +1777,7 @@ bool CZNC::RemVHost(const CString& sHost) {
 
 void CZNC::Broadcast(const CString& sMessage, bool bAdminOnly,
 		CUser* pSkipUser, CClient *pSkipClient) {
-	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); a++) {
+	for (map<CString,CUser*>::iterator a = m_msUsers.begin(); a != m_msUsers.end(); ++a) {
 		if (bAdminOnly && !a->second->IsAdmin())
 			continue;
 
@@ -1707,6 +1790,24 @@ void CZNC::Broadcast(const CString& sMessage, bool bAdminOnly,
 	}
 }
 
+CModule* CZNC::FindModule(const CString& sModName, const CString& sUsername) {
+	if (sUsername.empty()) {
+		return CZNC::Get().GetModules().FindModule(sModName);
+	}
+
+	CUser* pUser = FindUser(sUsername);
+
+	return (!pUser) ? NULL : pUser->GetModules().FindModule(sModName);
+}
+
+CModule* CZNC::FindModule(const CString& sModName, CUser* pUser) {
+	if (pUser) {
+		return pUser->GetModules().FindModule(sModName);
+	}
+
+	return CZNC::Get().GetModules().FindModule(sModName);
+}
+
 CUser* CZNC::FindUser(const CString& sUsername) {
 	map<CString,CUser*>::iterator it = m_msUsers.find(sUsername);
 
@@ -1739,10 +1840,58 @@ bool CZNC::AddUser(CUser* pUser, CString& sErrorRet) {
 				<< sErrorRet << "]");
 		return false;
 	}
+	if (GetModules().OnAddUser(*pUser, sErrorRet)) {
+		DEBUG("AddUser [" << pUser->GetUserName() << "] aborted by a module ["
+			<< sErrorRet << "]");
+		return false;
+	}
 	m_msUsers[pUser->GetUserName()] = pUser;
 	return true;
 }
 
+CListener* CZNC::FindListener(u_short uPort, const CString& sBindHost, EAddrType eAddr) {
+	vector<CListener*>::iterator it;
+
+	for (it = m_vpListeners.begin(); it < m_vpListeners.end(); ++it) {
+		if ((*it)->GetPort() != uPort)
+			continue;
+		if ((*it)->GetBindHost() != sBindHost)
+			continue;
+		if ((*it)->GetAddrType() != eAddr)
+			continue;
+		return *it;
+	}
+	return NULL;
+}
+
+bool CZNC::AddListener(CListener* pListener) {
+	if (!pListener->GetRealListener()) {
+		// Listener doesnt actually listen
+		delete pListener;
+		return false;
+	}
+
+	// We don't check if there is an identical listener already listening
+	// since one can't listen on e.g. the same port multiple times
+
+	m_vpListeners.push_back(pListener);
+	return true;
+}
+
+bool CZNC::DelListener(CListener* pListener) {
+	vector<CListener*>::iterator it;
+
+	for (it = m_vpListeners.begin(); it < m_vpListeners.end(); ++it) {
+		if (*it == pListener) {
+			m_vpListeners.erase(it);
+			delete pListener;
+			return true;
+		}
+	}
+
+	return false;
+}
+
 CZNC& CZNC::Get() {
 	static CZNC* pZNC = new CZNC;
 	return *pZNC;
@@ -1758,25 +1907,25 @@ CZNC::TrafficStatsMap CZNC::GetTrafficStats(TrafficStatsPair &Users,
 	uiZNC_in  = BytesRead();
 	uiZNC_out = BytesWritten();
 
-	for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); it++) {
+	for (map<CString, CUser*>::const_iterator it = msUsers.begin(); it != msUsers.end(); ++it) {
 		ret[it->first] = TrafficStatsPair(it->second->BytesRead(), it->second->BytesWritten());
 		uiUsers_in  += it->second->BytesRead();
 		uiUsers_out += it->second->BytesWritten();
 	}
 
-	for (CSockManager::const_iterator it = m_Manager.begin(); it != m_Manager.end(); it++) {
+	for (CSockManager::const_iterator it = m_Manager.begin(); it != m_Manager.end(); ++it) {
+		CUser *pUser = NULL;
 		if ((*it)->GetSockName().Left(5) == "IRC::") {
-			CIRCSock *p = (CIRCSock *) *it;
-			ret[p->GetUser()->GetUserName()].first  += p->GetBytesRead();
-			ret[p->GetUser()->GetUserName()].second += p->GetBytesWritten();
-			uiUsers_in  += p->GetBytesRead();
-			uiUsers_out += p->GetBytesWritten();
+			pUser = ((CIRCSock *) *it)->GetUser();
 		} else if ((*it)->GetSockName().Left(5) == "USR::") {
-			CClient *p = (CClient *) *it;
-			ret[p->GetUser()->GetUserName()].first  += p->GetBytesRead();
-			ret[p->GetUser()->GetUserName()].second += p->GetBytesWritten();
-			uiUsers_in  += p->GetBytesRead();
-			uiUsers_out += p->GetBytesWritten();
+			pUser = ((CClient*) *it)->GetUser();
+		}
+
+		if (pUser) {
+			ret[pUser->GetUserName()].first  += (*it)->GetBytesRead();
+			ret[pUser->GetUserName()].second += (*it)->GetBytesWritten();
+			uiUsers_in  += (*it)->GetBytesRead();
+			uiUsers_out += (*it)->GetBytesWritten();
 		} else {
 			uiZNC_in  += (*it)->GetBytesRead();
 			uiZNC_out += (*it)->GetBytesWritten();
@@ -1791,12 +1940,10 @@ CZNC::TrafficStatsMap CZNC::GetTrafficStats(TrafficStatsPair &Users,
 }
 
 void CZNC::AuthUser(CSmartPtr<CAuthBase> AuthClass) {
-#ifdef _MODULES
 	// TODO unless the auth module calls it, CUser::IsHostAllowed() is not honoured
 	if (GetModules().OnLoginAttempt(AuthClass)) {
 		return;
 	}
-#endif
 
 	CUser* pUser = GetUser(AuthClass->GetUsername());
 
@@ -1892,7 +2039,7 @@ protected:
 	}
 
 private:
-	size_t	m_uiPosNextUser;
+	size_t m_uiPosNextUser;
 };
 
 void CZNC::EnableConnectUser() {
